﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using glb;

namespace evc
{
	// １制御点を管理します。
	// 



	// アニメーションタイプ
	public enum FeAnimType
	{
		Non = 0,	// なし
		Liner,		// 線形補間
		Point,		// 点補間
		Hermite,	// エルミート曲線補間
		Saw,		// のこぎり補間
	};

	// 補間アニメーションベース
	public partial interface IFeAnimBase
	{
		// フレーム(0.0f～1.0f)に対する進行度を取得
		float GetFramePos( float fAnimFrame, FePointInfo befoInfo, FePointInfo nowInfo );

		// ロード
		void Load ( IseStateInput inInput );
	}


	// １編集点管理(再生用)
	public partial class FePointInfo
	{
		public float Rate  { get; set; }	// 前のキーから今のキーまでを0.0f～1.0fとした経過時間
		public float Value { get; set; }	// 値

		// 補間タイプ別処理を取得
		public IFeAnimBase AnimFunc { get; private set; }


		// 内部アニメーション処理を作成
		public void CreateAnimFunc( FeAnimType eAnimType )
		{
			switch (eAnimType)
			{
				case FeAnimType.Liner: this.AnimFunc = new FpAnimLiner(); break;
				case FeAnimType.Point: this.AnimFunc = new FpAnimPoint(); break;
				case FeAnimType.Hermite: this.AnimFunc = new FpAnimHermite(); break;
				case FeAnimType.Saw: this.AnimFunc = new FpAnimSaw(); break;

				case FeAnimType.Non: 
				default: 
					this.AnimFunc = null;
					break;
			}
		}

		// コンストラクタ
		internal FePointInfo(){}
		public FePointInfo ( float fRate, float fValue )
		{
			this.Rate = fRate;
			this.Value = fValue;
		}

		// 入力
		public void Load( IseStateInput input )
		{
			this.Rate = input.ReadFloat();
			this.Value = input.ReadFloat();

			FeAnimType eAnimType = input.ReadEnum<FeAnimType>();
			this.CreateAnimFunc( eAnimType );
			if ( this.AnimFunc != null )
			{
				this.AnimFunc.Load( input );
			}
		}

		// 前の位置と経過フレームから今の位置を取得
		public float GetPos( float fFrame, FePointInfo inBfoInfo )
		{
			return (this.AnimFunc == null) ? inBfoInfo.Value: 
				this.AnimFunc.GetFramePos( fFrame, inBfoInfo, this );
		}


		// 今のフレームの位置を取得
		public static float GetPos( float fFrame, List<FePointInfo> lisPoint)
		{
			FePointInfo bfoInfo = lisPoint[ 0 ];
			foreach (FePointInfo nowInfo in lisPoint)
			{
				if ( nowInfo.Rate > fFrame )
				{
					return nowInfo.GetPos( fFrame, bfoInfo );
				}
				bfoInfo = nowInfo;
			}

			// 最後のデータを返す
			return lisPoint[ lisPoint.Count - 1 ].Value;
		}
	}
}
