﻿using System;
using System.Collections.Generic;
using System.Text;
using glb;

namespace evc
{
	// 入出力管理
	public interface siIOManager
	{
		// マトリクスを作成
		IGLMatrix CreateMatrix();

		// １ファイルロード管理を作成
		// @param strPath		 : ロードを要求するファイルのパス
		// @param inSceneManager : 要求したファイルが属するシーン(null=ルートシーン)
		siFile CreateLoader( seSceneManager inSceneManager, string strPath );


		#region オブジェクト作成関係

		// カラースプライトの作成が要求された時に呼び出されます。
		siObjectColorSprite CreateColorSprite ( int iID );

		// テクスチャスプライトの作成が要求された時に呼び出されます。
		siObjectTextureSprite CreateTextureSprite ( int iID );

		// サウンドの作成が要求された時に呼び出されます。
		siObjectSound CreateSound ( int iID, siResourceSound inResource );

		// 文字列描画管理作成
		siObjectFont CreateFont( int iID );

		// メッシュ描画管理作成
		// @param indexType		: indexの並びタイプ
		// @param iVertexCount	: 頂点数
		// @param iIndexCount	: Index数
		siObjectMesh CreateMesh( int iID, MeshIndex indexType, int iVertexCount, int iIndexCount );


		#endregion



		#region リソース作成関係

		// テクスチャリソース作成時に呼び出されます。
		// @parama strName:入力ファイルパス
		// @retval null:失敗  !null:成功
		siResourceTexture CraeteTextureResource ( seSceneManager inSceneManager, string strTextureName, glbVector2I sTexSize );

		// サウンドのリソース作成時に呼び出されます。
		// @parama strName:入力ファイルパス
		// @retval null:失敗  !null:成功
		siResourceSound CraeteSoundResource ( seSceneManager inSceneManager, string strSoundName );

		#endregion


		#region マスク関係

		// 新しいマスクを登録
		// マスク描画が呼ばれる直前に新しいマスクの全体領域を表すマトリクスを引数として
		// このメソッドが呼ばれます。
		// RegisterMaskは新しいマスクが設定する度に呼ばれます。
		void RegisterMask ( IGLMatrix mMask, glbVector2 sSize );

		// マスクの終了を通知します。
		// ユーザがマスク終了設定をした要素を描画した後に呼ばれます。
		void ClearMask ();

		#endregion


		// デバッグ用出力
		void DebugString( string strMsg );


		// 解放
		void Dispose();
	}
}
