﻿using System;
using System.Collections.Generic;
using System.Text;
using glb;


namespace evc
{
	// カラースプライト描画管理
	public interface siObjectColorSprite
	{
		// サイズが更新された
		void UpdateSize( glbVector2 sSize );

		// 描画要求
		void Draw( int iElemID, IGLMatrix m, glbColor sColor );
		
		// 解放
		void Dispose();
	}


	// テクスチャスプライト描画管理
	public interface siObjectTextureSprite
	{
		// テクスチャが更新された
		void UpdateTexture( siResourceTexture inTexture );
		// サイズが更新された
		void UpdateSize( glbVector2 sSize );
		// UVが更新された
		void UpdateUV( glbVector2 sUV );

		// 描画要求
		void Draw( int iElemID, IGLMatrix m, glbColor sColor );
		void Draw( int iElemID, IGLMatrix m, glbColor sColor, IGLMatrix mUV );
		
		// 解放
		void Dispose();
	}



	// インディックスのタイプ
	public enum MeshIndex
	{
		// ３つのindexで１つの三角形を現します。
		// UpdateIndexに渡される配列は３の倍数であることが保障されます。
		TriangleList,
	};

	// テクスチャメッシュ描画管理
	public interface siObjectMesh
	{
		// テクスチャが更新された
		void UpdateTexture( siResourceTexture inTexture );

		// 通常、SetPos,SetUVはまとまって何度も呼び出されます。
		// BeginEditVertex()、EndEditVertex()はその最初と最後に呼び出されます。
		// 頂点の編集開始前、終了後に処理をしたい場合、
		// こちらのメソッド内に記述して下さい。
		void BeginEditVertex( int iElemID );
		void EndEditVertex();

		// 頂点の新しい座標を設定します。
		// このメソッドはBeginEditVertexとEndEditVertexの間に呼ばれます。
		// @param iIndex : 設定する頂点index(0～(頂点数-1)の範囲で呼び出されます
		// @param pos : 座標
		void SetPos( int iIndex, glbVector2 pos );
		// 頂点の新しいuv座標を設定します。
		// このメソッドはBeginEditVertexとEndEditVertexの間に呼ばれます。
		// @param iIndex : 設定する頂点index(0～(頂点数-1)の範囲で呼び出されます
		// @param uv : UV座標
		void SetUV( int iIndex, glbVector2 uv );


		// idの配列数はiIndexCountと同じです。
		// 渡される配列は、描画中保持され続ける事が保障されます。
		// @param indexType : indexのタイプ
		// @param id[] : index配列
		void SetIndex( short[] id );


		// 描画要求
		void Draw( int iElemID, IGLMatrix m, glbColor color, IGLMatrix mUV );
		
		// 解放
		void Dispose();
	}




	// サウンド管理
	public interface siObjectSound
	{
		// パンが更新された
		void UpdatePan( int iPan );
		// ボリュームが更新された
		void UpdateVolume( int iVolume );

		// 再生要求
		void Play( int iElemID, float fStartTime, bool bLoop );
		void Play( int iElemID, float fStartTime, float fLength, bool bLoop );
		
		// 停止要求
		void Stop();
		
		// デバッグ用描画
		void Draw( IGLMatrix m, int iVolume, int iPan );
		
		// 解放
		void Dispose();
	}



}
