﻿using System;
using System.Collections.Generic;
using System.Text;
using glb;

namespace evc
{
	// １文字情報管理
	public interface siObjectFontChar
	{
		// サイズ取得
		glbVector2 Size { get; }
	
		// @param mParent  : マトリクス
		void Draw( int iElemID, IGLMatrix mParent, glbColor sColor );
	}
	
	// 文字列表示管理
	public interface siObjectFont
	{
		// フォントスタイルを作成
		// 表示に使うフォントスタイルを通知します。
		// 戻り値をIDとして描画文字のスタイルを指定します。
		// 
		// @param strFontName	: フォントファミリーが指定されます
		// @param fHeight		: フォントの高さが指定されます。
		// @param Style			: フォントのスタイルが指定されます。seFontStyleを参照して下さい。
		// @retval 識別ID
		int CreateFontStyle( string strFontName, float fHeight, seFontStyle Style );


		// 文字クラスの作成開始を宣言
		// CreateCharが呼ばれる前に１度だけ呼ばれます。
		void BeginCharCreating();

		// 文字列を登録
		// 描画に使うスタイルと文字が呼ばれます。文字は種類分呼ばれます。
		// 例："ああいう"という文字列データを表示する場合、'あ','い','う'を指定した
		// CreateCharがそれぞれ呼ばれ、戻り値のsiObjectFontCharが複数回呼ばれ画面に文字を描画します。
		// 但し、iFontStyleが違うと別文字と判定されます。
		// 例："ああ"という文字のうち、１文字目と２文字目のフォントスタイルIDが違う場合
		// CreateCharはそれぞれの"あ"に対して呼ばれます。
		// 
		// @param iFontStyle	: フォントのスタイルＩＤ。CreateFontStyleの戻り値。
		// @param cChar			: 文字
		// @retval				: 文字描画クラス
		siObjectFontChar CreateChar( int iFontStyle, char cChar );

		// 文字クラスの作成終了を宣言
		// CreateCharの登録が終了した後に呼ばれます。
		void EndCharCreating();

		// データを全て削除
		void Clear();
		
		// 解放
		void Dispose();
	}


}
