﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Reflection;
using System.Security;
using System.IO;

namespace evc
{
	// evc共通宣言

	// オブジェクトタイプ
	public enum emObjectType
	{
		Null = 0,		// 該当オブジェクトなし
		Color,			// カラースプライト
		Texture,		// テクスチャスプライト
		Scene,			// シーン
		Sound,			// サウンド
		Font,			// 文字列
		Num,			// 数値
		Frame,			// フレーム(テクスチャが部分的に伸張)
		Mesh,			// メッシュ
		MeshChild,		// メッシュの制御点(オブジェクトとしては存在しない。特殊)
		Grid,			// グリッド
		Particles,		// パーティクル
	}

	// エレメントステータス
	public enum emElemState
	{
		// エレメント共通ステータス
		// 継承で使用したり、エレメントに必ず必要なステータスです。
		Priority = 0,	// 描画優先順位
		Attribute,		// 属性
		Pos,			// 位置
		Rot,			// 回転
		Scale,			// スケール
		Center,			// 中心位置
		MaskAlpha,		// マスクの有効アルファ値
		Color,			// カラー

		// オブジェクト別ステータス
		// 対応オブジェクトによっては必要になるステータスです。
		FrameCtrl,		// フレームコントロール(root)に専用
		Texture,		// テクスチャ(切り替えアニメーション用)
		TextureControl,	// テクスチャ制御(ヒット判定、繰り返しモード)
		UV,				// UV
		Loop,			// ループ
		Num,			// 数値
		NumState,		// 文字表示ステータス
		SceneAnim,		// シーンアニメーション名
		Load,			// シーンロード
		TextControl,	// テキストコントロール(ページの制御)
		Volume,			// ボリューム
		Pan,			// パン
		SoundControl,	// サウンドコントロール
		GridPos1,		// グリッド頂点１
		GridPos2,		// グリッド頂点２
		GridPos3,		// グリッド頂点３
		GridPos4,		// グリッド頂点４
		ParticleControl,// パーティクルコントロール

		// 編集用ステータス
		// 編集時のみ作成されます。
		ZPath,			// 3Dパス(Ｚ移動用)

	}


	// リソースタイプ
	public enum seResourceType
	{
		Image = 0,
		Sound,
		Scene,
	}



	internal class seDefine
	{
		// マジック："pra\0"
		internal const int Magic = 0x617270;
		// フォーマットバージョン：ファイル構造が変わった場合は値をＵＰする
		internal const int FileVarsion = 20090130;


		// エレメント共通ステータスの最後
		internal const emElemState LastCommonState = emElemState.Color;

#if DEBUG
		// ここをtrueにすると表示します。
		private static bool bEnableDebugTrace = false;
		private static BinaryReader binReader = null;


		// バイナリ入力用トレース開始
		private static int iTopTabIndex = -1;
		internal static void StartDebugTrace( BinaryReader reader )
		{
			if( !seDefine.bEnableDebugTrace ) return;
			seDefine.iTopTabIndex = -1;
			seDefine.binReader = reader;
		}
		// バイナリデータデバッグ表示
		internal static void DebugTrace( string strMsg, int iCount )
		{
			if( !seDefine.bEnableDebugTrace ) return;

			StackTrace st = new StackTrace();
			if ( seDefine.iTopTabIndex < 0 )
			{
				seDefine.iTopTabIndex = st.FrameCount;
				Console.WriteLine( "=====================================" );
			}

			int iTabCount = st.FrameCount - seDefine.iTopTabIndex;

			StackFrame inStack = st.GetFrame(1);
			MethodBase mb = inStack.GetMethod();
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < iTabCount; i++)
			{
				sb.Append( "\t" );
			}
			sb.Append( mb.ReflectedType.Name );
			sb.Append( "." );
			sb.Append( mb.Name );
			sb.Append( " : " );
			sb.Append( strMsg );
			sb.Append( '[' );
			sb.Append( iCount );
			sb.Append( ']' );
			sb.Append( '(' );
			// 現在のファイルロード位置を出力
			sb.Append( seDefine.binReader.BaseStream.Position );
			sb.Append( ')' );
			Console.WriteLine( sb.ToString() );
		}
#else
		internal static void StartDebugTrace( BinaryReader reader ){}
		internal static void DebugTrace( string strMsg, int iCount ){}
#endif

	}

}
