﻿using System;
using System.Collections.Generic;
using System.Text;
using glb;

namespace evc
{
	// イベント演算子
	public enum evOperator
	{
		Non,		// 無処理

		// イベント用
		Add,		// += 加算
		Sub,		// -= 減算
		Set,		// ← 代入
		Mul,		// *= 乗算
		Div,		// /= 割り算
		Quotient,	// %= 余り

		// 条件用
		Over,		// > 大きい
		Under,		// < 未満
		Same,		// == 同じ
		Diff,		// != 違う
		SameOrOver,	// >= 以上
		SameOrUnder,// <= 以下
	};


	public partial class evCmdList
	{
		// 命令インスタンス作成
		public static evCmdCond CraeteCmd( evCond eCmd, IevCmdUI ui )
		{
			evCmdCond cmd = new evCmdCond();
			cmd.CmdType = eCmd;

			// このメソッドでは内部データは作られない。
			// ロード時か、編集時に作られる。

			switch (eCmd)
			{
				// 矩形：ポインタと最初にヒットした
				case evCond.InTopRect:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						if ( !cmdUI.Target.IsVisible ) return false;
						return (cmdUI.Target.Hit == seHit.TopHit);
					};
					break;
				// 矩形：ポインタとヒットしている
				case evCond.InRect:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						if ( !cmdUI.Target.IsVisible ) return false;

						switch (cmdUI.Target.Hit)
						{
							case seHit.TopHit:
							case seHit.Hit:
								return true;
							default:
								break;
						}
						return false;
					};
					break;
				// 矩形：ポインタがヒットしていない
				case evCond.OutRect:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						if ( !cmdUI.Target.IsVisible ) return false;
						return (cmdUI.Target.Hit == seHit.Non);
					};
					break;
				// 矩形：矩形とヒットしている
				case evCond.HitRect:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						int iElemID = target.Data[ 0 ].Value;
						
						seElem elem = cmdUI.SceneManager.ElemManager.Find( iElemID );
						if ( (elem == null) || (!elem.IsVisible) ) return false;

						seElem elemTarget = cmdUI.Target;
						if ( (elemTarget == null) || (!elemTarget.IsVisible) ) return false;
						
						return seMath.IsHitPoly( elem.GetRect(), elemTarget.GetRect() );
					};
					break;
				// 矩形：任意の座標とヒットしている
				case evCond.InRectPoint:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						// 非表示の場合は失敗
						if ( (cmdUI.Target == null) || (!cmdUI.Target.IsVisible) )return false;
						glbVector2 pos = new glbVector2( (float)target.Data[ 0 ].Value,
														 (float)target.Data[ 1 ].Value );
						return cmdUI.Target.IsInRect( pos );
					};
					break;
				// 矩形：任意の座標とヒットしていない
				case evCond.OutRectPoint:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						// 非表示の場合は失敗
						if ( (cmdUI.Target == null) || (!cmdUI.Target.IsVisible) )return false;
						glbVector2 pos = new glbVector2( (float)target.Data[ 0 ].Value,
														 (float)target.Data[ 1 ].Value );
						return !cmdUI.Target.IsInRect( pos );
					};
					break;
				// ステータス変数：持っている
				case evCond.IsState:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						evdVariableStateItem item = (evdVariableStateItem)target.Data[ 0 ];
						if ( item.DataVariable == null ) return false;
						return item.DataVariable.IsOn( item.ItemID );
					};
					break;

				// ステータス変数：持っていない
				case evCond.IsNotState:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						evdVariableStateItem item = (evdVariableStateItem)target.Data[ 0 ];
						if ( item.DataVariable == null ) return false;
						return !item.DataVariable.IsOn( item.ItemID );
					};
					break;

				// 数値比較
				case evCond.ComparisonValue:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						int dat1 = target.Data[ 0 ].Value;
						int dat2 = target.Data[ 1 ].Value;

						switch ( target.Data[ 1 ].Operator )
						{
							// 実行条件
							case evOperator.Over:  return (dat1 > dat2);	// >
							case evOperator.Under: return (dat1 < dat2);	// <
							case evOperator.Same:  return (dat1 == dat2);	// ==
							case evOperator.Diff:  return (dat1 != dat2);	// !=
							case evOperator.SameOrOver:  return (dat1 >= dat2);	// !=
							case evOperator.SameOrUnder: return (dat1 <= dat2);	// !=
							default: break;
						}
						return false;
					};
					break;
				
				// ボタン：押した
				case evCond.IsPush:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						return evInputControl.Instance.IsPush( (ButtonType)target.Data[ 0 ].Value );
					};
					break;
				// ボタン：押し始めた
				case evCond.IsFirstPush:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						return evInputControl.Instance.IsFirstPush( (ButtonType)target.Data[ 0 ].Value );
					};
					break;
				// ボタン：離した
				case evCond.IsPop:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						return !evInputControl.Instance.IsPush( (ButtonType)target.Data[ 0 ].Value );
					};
					break;

				// アニメーション：開始した
				case evCond.IsStartAnim:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						evCmdData data = target.Data[ 0 ];
						return cmdUI.SceneManager.IsPlayNextAnim( data.Value );
					};
					break;
				// ポイント：移動した
				case evCond.MovePoint:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						return evInputControl.Instance.IsMovePointer;
					};
					break;

				// コメント
				case evCond.Comment:
					break;
				// その他(処理なし)
				default:
					break;
			}

			// 命令文字列を入れる
			return cmd;
		}

		// 命令インスタンス作成
		public static evCmdEvent CraeteCmd( evEvent eCmd, IevCmdUI ui)
		{
			evCmdEvent cmd = new evCmdEvent();
			cmd.CmdType = eCmd;

			// このメソッドでは内部データは作られない。
			// ロード時か、編集時に作られる。

			switch (eCmd)
			{
				// アニメーション：再生
				case evEvent.AnimPlay:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						cmdUI.SceneManager.RequestAnim( target.Data[ 0 ].Value );
						return true;
					};
					break;
				// アニメーション：停止
//				case evEvent.AnimStop: inHeader = new evCmdAnimStop(); break;
				// アニメーション：一時停止
				case evEvent.AnimPause:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						cmdUI.Target.StopAnim = true;
						return true;
					};
					break;

				// アニメーション：再開
				case evEvent.AnimContinue:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						cmdUI.Target.StopAnim = false;
						cmdUI.Target.StopAnimAll = false;
						return true;
					};
					break;

				// アニメーション：ジャンプ
//				case evEvent.AnimJump: inHeader = new evCmdAnimJump(); break;

				// アニメーション再生
				case evEvent.SceneAnim:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						evdString s = (evdString)target.Data[ 0 ];
						// オブジェクトがシーンならアニメーションをリクエスト
						seObjectWork objWork = cmdUI.Target.ObjectWork;
						if ( objWork.Object.ElemType == emObjectType.Scene )
						{
							seoSceneWork scene = (seoSceneWork)objWork;
							scene.LocalScene.RequestAnim( s.Text );
						}
						return true;
					};
					break;

				// 保留

				// 入力：一時停止
				case evEvent.InputPause:
					break;
				// 入力：再開
				case evEvent.InputContinue:
					break;


				// ステータス変数：設定
				case evEvent.SetStatus:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						evdVariableStateItem item = (evdVariableStateItem)target.Data[ 0 ];
						if ( item.DataVariable == null ) return true;
						item.DataVariable.Value |= item.ItemID;
						return true;
					};
					break;
				// ステータス変数：削除
				case evEvent.RemoveStatus:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						evdVariableStateItem item = (evdVariableStateItem)target.Data[ 0 ];
						if ( item.DataVariable == null ) return true;
						item.DataVariable.Value &= ~item.ItemID;
						return true;
					};
					break;

				// ステータス変数：クリア
				case evEvent.ClearStatus:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						evdVariableState item = (evdVariableState)target.Data[ 0 ];
						if ( item.DataVariable == null ) return true;
						item.DataVariable.Value = 0;
						return true;
					};
					break;

				// 数値演算
				case evEvent.OperatorValue:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						int iArgValue = target.Data[1].Value;
						switch ( target.Data[1].Operator )
						{
							// イベント処理
							case evOperator.Add: target.Data[0].Value += iArgValue; break;	// +=
							case evOperator.Sub: target.Data[0].Value -= iArgValue; break;	// -=
							case evOperator.Set: target.Data[0].Value  = iArgValue; break;	// =
							case evOperator.Mul: target.Data[0].Value *= iArgValue; break;	// *=
							case evOperator.Div: target.Data[0].Value /= iArgValue; break;	// /=
							case evOperator.Quotient: target.Data[0].Value %= iArgValue; break;	// %=
							default: break;
						}
						return true;
					};
					break;

				// デバッグ：文字列出力
				case evEvent.DebugString:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						evdString s = (evdString)target.Data[ 0 ];
						seRootScene.Instance.IOManager.DebugString(s.Text);
						return true;
					};
					break;

				// デバッグ：変数出力
				case evEvent.DbWriteNum:
					cmd.OnFunc = delegate(IevCmdUI cmdUI, evCmd target )
					{
						StringBuilder sb = new StringBuilder();
						evCmdData cmdData = target.Data[ 0 ];

						if ( cmdData is evdVariableNum )
						{
							evdVariableNum num = (evdVariableNum)cmdData;
							if ( num.DataVariable == null ) return true;
							sb.Append( num.DataVariable.Name );
							sb.Append(" : ");
							sb.Append( num.DataVariable.Value );
						}
						else if ( cmdData is evdVariableState )
						{
							evdVariableState state = (evdVariableState)cmdData;
							evVariableState st = state.DataVariable;
							if ( st == null ) return true;
							sb.Append( st.Name );
							sb.Append(" : ");
#if PL_EDITOR
							st.DoAllItem(delegate(string strItem, int iID)
							{
								if ((st.Value & iID) != 0)
								{
									sb.Append("[");
									sb.Append(strItem);
									sb.Append("]");
								}
								return true;
							});
#else
							sb.Append( st.Value );
#endif
						}
						seRootScene.Instance.IOManager.DebugString(sb.ToString());
						return true;
					};
					break;

				// コメント
				case evEvent.Comment:
					break;

				// 未サポート(処理なし)
				default:
					break;
			}

			// 命令文字列を入れる
			return cmd;
		}

	}

	// 命令官吏
	public abstract partial class evCmd : evCmdHeader
	{
		public List<evCmdData> Data { get; private set; }

		// 実行ルーチン
		public glbFunc<IevCmdUI, evCmd, bool> OnFunc { get; set; }

		// コンストラクタ
		public evCmd() : base()
		{
			this.Data = new List<evCmdData>();
		}

		public void AddData( evCmdData data )
		{
			this.Data.Add( data );
		}

		// 任意の位置のデータを差し替え
		public void SetData( evCmdData data, int iIndex )
		{
			this.Data.RemoveAt( iIndex );
			this.Data.Insert( iIndex, data );
		}

		// コンストラクタ(クローン用)
		public evCmd( evCmd mst ) : base( mst )
		{
			this.OnFunc = mst.OnFunc;
			this.Data = new List<evCmdData>();
			mst.Data.ForEach( delegate(evCmdData data )
			{
				this.Data.Add( (evCmdData)data.Clone() );
			});
		}

		// 全てのデータに大して処理を実行
		// これはEditの方に移動するかも。
		public bool DoAllData( glbFunc<evCmdData,bool> func )
		{
			foreach (var item in this.Data)
			{
				if ( !func(item) ) return false;
			}
			return true;
		}

		// 処理を実行
		public override bool DoFunc(IevCmdUI cmd)
		{
			if (this.OnFunc == null) return true;
			return this.OnFunc( cmd, this );
		}


		// ロード
		public override void Load(IseStateInput inReader, IevCmdUI inCmdUI)
		{
 			base.Load(inReader, inCmdUI);

			int dataCount = inReader.ReadInt();
			for (int i = 0; i < dataCount; i++)
			{
				evDataType eType = inReader.ReadEnum<evDataType>();
				evCmdData cmdData = evData.CreateData( eType, inCmdUI );
				cmdData.Load( inReader, inCmdUI );

				this.Data.Add( cmdData );
			}
		}
	}


	public partial class evCmdEvent : evCmd
	{
		public evEvent CmdType { get; set; }

		public evCmdEvent(){}
		public evCmdEvent( evCmdEvent mst ) : base( mst )
		{
			this.CmdType = mst.CmdType;
		}

		public override evCmdHeader Clone()
		{
			return new evCmdEvent( this );
		}
	}
	

	public partial class evCmdCond : evCmd
	{
		public evCond CmdType { get; set; }

		public evCmdCond(){}
		public evCmdCond( evCmdCond mst ) : base( mst )
		{
			this.CmdType = mst.CmdType;
		}


		public override evCmdHeader Clone()
		{
			return new evCmdCond( this );
		}
	}
}
