﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using evc;
using glb;

namespace evc
{
	// イベントデータベース
	public interface ICmdBase
	{
		ICmdBase Clone();	// クローン生成
		void Load( IseStateInput inReader, IevCmdUI inCmdUI );	// データロード
	}


	// 命令管理ベース
	public abstract partial class evCmdHeader
	{
		public bool Enable { get; set; }		 // 命令が有効か否か
		public evCmdFlowIf Parent { get; set; }	 // 親フロー

		public evCmdHeader()
		{
			this.Enable = true;
		}
		// クローン生成時用
		public evCmdHeader( evCmdHeader mst )
		{
			this.Enable = mst.Enable;
			this.Parent = mst.Parent;
		}

		// 処理実行
		public abstract bool DoFunc(IevCmdUI inCmdUI);

		// データロード
		public virtual void Load( IseStateInput inReader, IevCmdUI inCmdUI )
		{
			this.Enable = inReader.ReadBool();
		}

		// クローン生成
		public abstract evCmdHeader Clone();
	}




	// 命令関係ユーティリティ
	internal abstract partial class evCmdUtil
	{
		// 変数参照情報をロードして対応する変数を返す
		// 事前に変数リストを全てロードした後で実行
		public static T Load<T>( IseStateInput inReader, IevCmdUI inCmdInput)
			where T : evVariable
		{
			// Group
			VariableGroupType eGroupType = inReader.ReadEnum<VariableGroupType>();
			int iVariableID = (int)inReader.ReadShort();

			evVariableManager vm = inCmdInput.SceneManager.VariableList;
			evVariableGroup inGroup = null;
			switch (eGroupType)
			{
				case VariableGroupType.Global: inGroup = vm.GlobalGroup; break;
				case VariableGroupType.Local: inGroup = inCmdInput.LocalVariable; break;
				case VariableGroupType.Sp: inGroup = vm.SpGroup; break;
				default: break;
			}
			Debug.Assert(inGroup != null);

			return (T)inGroup.Find(iVariableID);
		}
	}
}
