﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using glb;

namespace evc
{
	// イベントデータ別管理クラス
	// 引数及び代入先のデータを管理します。
	// 




	// 引数データタイプ
	public enum evDataType
	{
		Non,			// 操作外

		// int１つのデータ
		Int,			// 数値
		AnimID,			// アニメーションID
		// 後でValue設定時にseElemを更新するクラスにする？
		Elem,			// エレメント 
		Button,			// ボタン

		UserCall,		// ユーザー名呼び出し


		// 文字列１つのデータ
		String,			// 文字列
		DebugMsg,		// デバッグメッセージ
		
		// ポインタ位置を保持するデータ
		ElemState,		// エレメントステータス
		KeyName,		// キー名+Index
		VariableState,
		VariableStateItem,
		VariableNum,
	};


	public partial class evData 
	{
		// エレメントのステータス取得
		internal static int GetValue( IseStateType stype, emElemState stateType, int offet )
		{
			// 出力先別処理
			switch (stateType)
			{
				case emElemState.Pos:
					glbVector2 p = ((seStatePos)stype).Value;
					switch (offet)
					{
						case 0: return (int)p.X;
						case 1: return (int)p.Y;
						default: break;
					}
					break;

				case emElemState.Scale:
				case emElemState.UV:
				case emElemState.Center:
					glbVector2 s = ((seStateFloat2)stype).Value;
					switch (offet)
					{
						case 0: return (int)s.X;
						case 1: return (int)s.Y;
						default: break;
					}
					break;

				case emElemState.Rot:
					return (int)(((seStateRot)stype).Rot * 180.0f / (float)Math.PI);

				case emElemState.Color:
					glbColor c = ((seStateColor)stype).Color;
					switch (offet)
					{
						case 0: return (int)c.R;
						case 1: return (int)c.G;
						case 2: return (int)c.B;
						case 3: return (int)c.A;
						default: break;
					}
					break;

				case emElemState.Volume:
				case emElemState.Pan:
				case emElemState.Num:
					return ((seStateInt1)stype).Value;

				// サポート外の設定
				default:
					break;
			}

			Debug.Assert(false);
			return 0;
		}

		// エレメントのステータス設定
		internal static void SetValue( IseStateType stype, emElemState stateType, int offet, int value )
		{ 
			// 出力先別処理
			switch (stateType)
			{
				case emElemState.Pos:
					seStatePos p = (seStatePos)stype;
					glbVector2 ps = p.Value;
					switch (offet)
					{
						case 0: ps.X = (float)value; break;
						case 1: ps.Y = (float)value; break;
						default: break;
					}
					p.Value = ps;
					break;
				case emElemState.Scale:
				case emElemState.UV:
				case emElemState.Center:
					seStateFloat2 f2 = (seStateFloat2)stype;
					glbVector2 s = f2.Value;
					switch (offet)
					{
						case 0: s.X = (float)value; break;
						case 1: s.Y = (float)value; break;
						default: break;
					}
					f2.Value = s;
					break;

				case emElemState.Rot:
					((seStateRot)stype).Rot = (float)value * (float)Math.PI / 180.0f;
//					((seStateRot)stype).Rot = (float)value;
					break;

				case emElemState.Color:
					seStateColor cs = (seStateColor)stype;
					glbColor c = cs.Color;
					switch (offet)
					{
						case 0: c.R = (byte)value; break;
						case 1: c.G = (byte)value; break;
						case 2: c.B = (byte)value; break;
						case 3: c.A = (byte)value; break;
						default: break;
					}
					cs.Color = c;
					break;

				case emElemState.Volume:
				case emElemState.Pan:
				case emElemState.Num:
					((seStateInt1)stype).Value = value;
					break;
				// サポート外の設定
				default:
					Debug.Assert(false);
					break;
			}
		}

		// データタイプからインスタンス作成
		public static evCmdData CreateData( evDataType eType, IevCmdUI cmdUI )
		{
			switch (eType)
			{
				case evDataType.AnimID: return new evdInt( eType, seAnimManager.InvalidID );
				case evDataType.Button: return new evdInt( eType, (int)default(ButtonType) );
				case evDataType.Elem: return new evdInt( eType, seElemManager.InvalidID );
				case evDataType.Int: return new evdInt( eType, 0 );
	
				case evDataType.UserCall:
					return new evdUserCall( cmdUI.Target );

				case evDataType.String:
				case evDataType.DebugMsg:
					return new evdString( eType, 0 );

				case evDataType.ElemState:
					return new evdElemState( cmdUI.Target );

				case evDataType.KeyName:
					return new evdKeyValue( cmdUI.Target, cmdUI.SceneManager.KeyControlManager );

				case evDataType.VariableNum:
					return new evdVariableNum();

				case evDataType.VariableState:
					return new evdVariableState();

				case evDataType.VariableStateItem:
					return new evdVariableStateItem();

				default:
					break;
			}

			return null;
		}
	}


	// 指定クラス:このインタフェースを持つ命令は引数に使用できます。
	public abstract partial class evCmdData : ICmdBase
	{
		// 直前のデータとの演算式
		public evOperator Operator { get; set; }


		public evCmdData(){}
		public evCmdData( evCmdData mst )
		{
			this.Operator = mst.Operator;
		}


		public abstract ICmdBase Clone();	// クローン生成
		// データロード
		public virtual void Load( IseStateInput inReader, IevCmdUI inCmdUI )
		{
			this.Operator = inReader.ReadEnum<evOperator>();
		}

		public abstract evDataType DataType { get; set; }
		public abstract int Value { set; get; }
	}



	//==================================
	// int1つのデータ
	public partial class evdInt : evCmdData
	{
		public override evDataType DataType { get; set; }
		public override int Value { get; set; }

		// コンストラクタ
		public evdInt(evDataType eType, int iValue)
		{
			this.DataType = eType;
			this.Value = iValue;
		}
		// コンストラクタ(クローン用)
		public evdInt( evdInt mst ) : base( mst )
		{
			this.DataType = mst.DataType;
			this.Value = mst.Value;
		}


		public override ICmdBase Clone()
		{
			return new evdInt(this); 
		}

		public override void Load(IseStateInput inReader, IevCmdUI inCmdUI)
		{
			base.Load( inReader, inCmdUI );
			this.Value = inReader.ReadInt();
		}
	}


	//==================================
	// string1つのデータ(保留)
	public partial class evdString : evCmdData
	{
		public override evDataType DataType { get; set; }
		public override int Value { get; set; }	// 使用しない

		public string Text { get; set; }


		// コンストラクタ
		public evdString(evDataType eType, int iValue)
		{
			this.DataType = eType;
			this.Value = iValue;
			this.Text = string.Empty;
		}
		public evdString(evdString mst ) : base( mst )
		{
			this.DataType = mst.DataType;
			this.Text = mst.Text;
			this.Value = mst.Value;
		}


		public override ICmdBase Clone()
		{
			return new evdString(this); 
		}

		public override void Load(IseStateInput inReader, IevCmdUI inCmdUI)
		{
			base.Load( inReader, inCmdUI );
			this.Value = inReader.ReadInt();
			this.Text = inReader.ReadString();
		}
	}



	//==================================
	// 数値変数
	public partial class evdVariableNum : evCmdData
	{

		public override evDataType DataType
		{
			get { return evDataType.VariableNum; } 
			set {}
		}

		// 操作対象
		public evVariableNum DataVariable { get; set; }

		// データ値
		public override int Value 
		{
			get
			{
				if ( this.DataVariable == null ) return 0;
				return this.DataVariable.Value;
			}
			set
			{
				if ( this.DataVariable == null ) return;
				this.DataVariable.Value = value; 
			}
		}

		public evdVariableNum()
		{
			this.DataVariable = null;
		}
		public evdVariableNum( evdVariableNum mst ) : base( mst )
		{
			this.DataVariable = mst.DataVariable;
		}


		public override ICmdBase Clone()
		{
			return new evdVariableNum( this );
		}

		public override void Load(IseStateInput inReader, IevCmdUI inCmdUI)
		{
			base.Load( inReader, inCmdUI );
			this.DataVariable = evCmdUtil.Load<evVariableNum>( inReader, inCmdUI );
		}
	}


	//==================================
	// ステータス変数
	public partial class evdVariableState : evCmdData
	{
		public override evDataType DataType
		{
			get { return evDataType.VariableState; } 
			set {}
		}

		// 操作対象
		public evVariableState DataVariable { get; set; }

		// データ値
		public override int Value 
		{
			get { return this.DataVariable.Value; }
			set { this.DataVariable.Value = value; }
		}

		// コンストラクタ
		public evdVariableState()
		{
			this.DataVariable = null;
		}
		public evdVariableState( evdVariableState mst ) : base( mst )
		{
			this.DataVariable = mst.DataVariable;
		}


		public override ICmdBase Clone()
		{
			return new evdVariableState(this);
		}

		public override void Load(IseStateInput inReader, IevCmdUI inCmdUI)
		{
			base.Load( inReader, inCmdUI );
			this.DataVariable = evCmdUtil.Load<evVariableState>( inReader, inCmdUI );
		}
	}


	//==================================
	// ステータス変数
	public partial class evdVariableStateItem : evCmdData
	{
		public override evDataType DataType
		{
			get { return evDataType.VariableStateItem; }
			set {}
		}

		// 操作対象
		public evVariableState DataVariable { get; set; }
		public int ItemID { get; set; }

		// データ値
		public override int Value 
		{
			get { return this.ItemID; }
			set { Debug.Assert(false); }
		}


		// コンストラクタ
		public evdVariableStateItem()
		{
			this.DataVariable = null;
			this.ItemID = 0;
		}
		public evdVariableStateItem( evdVariableStateItem mst ) : base( mst )
		{
			this.DataVariable = mst.DataVariable;
			this.ItemID = mst.ItemID;
		}


		public override ICmdBase Clone()
		{
			return new evdVariableStateItem(this);
		}

		public override void Load(IseStateInput inReader, IevCmdUI inCmdUI)
		{
			base.Load( inReader, inCmdUI );
			this.DataVariable = evCmdUtil.Load<evVariableState>( inReader, inCmdUI );
			this.ItemID = inReader.ReadInt();
		}
	}



	//==================================
	// エレメントデータ
	public partial class evdElemState : evCmdData
	{
		public override evDataType DataType
		{ 
			get { return evDataType.ElemState; } 
			set {}
		}

		public seElem Target { get; set; }			// ターゲットエレメント(セーブはしない)
		public emElemState StateType { get; set; }	// 操作対象エレメントステータス
		public int Offset { get; set; }				// 操作対象オフセット(最大で４)

		// コンストラクタ
		public evdElemState( seElem elem )
		{
			this.Target = elem;
			this.StateType = default(emElemState);
			this.Offset = 0;
		}
		public evdElemState( evdElemState mst ) : base( mst )
		{
			this.Target = mst.Target;
			this.StateType = mst.StateType;
			this.Offset = mst.Offset;
		}

	
		// データ値
		public override int Value 
		{
			get 
			{
				if ( this.Target == null ) return 0;
				IseStateType stype = this.Target.State[ this.StateType ];
				return evData.GetValue( stype, this.StateType, this.Offset );
			}
			set 
			{ 
				if ( this.Target == null ) return;

				IseStateType stype = this.Target.State[ this.StateType ];
				evData.SetValue( stype, this.StateType, this.Offset, value );

				// 自分のステータスを更新し、自分以下の継承処理を再実行
				this.Target.State.Apply();
				this.Target.DoAll( delegate( seElem childElem )
				{
					childElem.Succession( seRootScene.Instance.Camera );
					return true;
				});
			}
		}


		public override ICmdBase Clone()
		{
			return new evdElemState(this);
		}

		public override void Load(IseStateInput inReader, IevCmdUI inCmdUI)
		{
			base.Load( inReader, inCmdUI );

			this.Target = inCmdUI.Target;
			this.StateType = inReader.ReadEnum<emElemState>();
			this.Offset = inReader.ReadInt();
		}
	}


	//==================================
	// キーデータ
	public partial class evdKeyValue : evCmdData
	{
		public const int NoKey = -1;	// 無効なキー
	
		public int KeyID { get; set; }		// キー名取得
		public int Index { get; set; }		// キー編集対象Index取得

		public seElem Target { get; set; }
		public seKeyControlManager KeyManger { get; set; }

		public override evDataType DataType 
		{
			get { return evDataType.KeyName; } 
			set {}
		}


		public evdKeyValue( seElem elem, seKeyControlManager keyManager )
		{
			this.Target = elem;
			this.KeyManger = keyManager;
			this.Index = evdKeyValue.NoKey;
		}
		public evdKeyValue( evdKeyValue mst ) : base( mst )
		{
			this.Target = mst.Target;
			this.KeyManger = mst.KeyManger;
			this.KeyID = mst.KeyID;
			this.Index = mst.Index;
		}

		// データ値
		public override int Value 
		{
			get 
			{
				if ( this.KeyManger == null ) return 0;
				// エレメントステータスに値を加算or設定
				seStateKeyInfo keyInfo = this.KeyManger.Find( this.KeyID );
				if (keyInfo == null) return 0;

				return evData.GetValue( keyInfo.Data, keyInfo.StateType, this.Index );
			}
			set 
			{ 
				if ( this.KeyManger == null ) return;
				seStateKeyInfo keyInfo = this.KeyManger.Find( this.KeyID );
				if (keyInfo == null) return;
				evData.SetValue( keyInfo.Data, keyInfo.StateType, this.Index, value );

				// 自分のステータスを更新し、自分以下の継承処理を再実行
				this.Target.State.Apply();
				this.Target.DoAll( delegate( seElem childElem )
				{
					childElem.Succession( seRootScene.Instance.Camera );
					return true;
				});
			}
		}

		public override ICmdBase Clone()
		{
			return new evdKeyValue( this );
		}

		public override void Load(IseStateInput inReader, IevCmdUI inCmdUI)
		{
			base.Load( inReader, inCmdUI );

			this.Target = inCmdUI.Target;
			this.KeyManger = inCmdUI.SceneManager.KeyControlManager;

			this.KeyID = inReader.ReadInt();
			this.Index = inReader.ReadInt();
		}

	}


	//==================================
	// ユーザー処理呼び出し
	public partial class evdUserCall : evCmdData
	{
		public override evDataType DataType 
		{
			get { return evDataType.UserCall; } 
			set {}
		}

		public seElem Target { get; set; }
	
		// 処理名
		public string Name { get; set; }

		// データ値
		public override int Value 
		{
			get 
			{
				siEventListener el = seRootScene.Instance.EventListener;
				if (el == null) return 0;
				return el.GetValue( this.Name, this.Target.ElemID );
			}
			set
			{
				siEventListener el = seRootScene.Instance.EventListener;
				if (el == null) return;
				el.CallCommand(this.Name, this.Target.ElemID, value);
			}
		}

		// コンストラクタ
		public evdUserCall( seElem target )
		{
			this.Target = target;
			this.Name = string.Empty;
		}
		public evdUserCall( evdUserCall mst ) : base( mst )
		{
			this.Target = mst.Target;
			this.Name = mst.Name;
		}


		public override ICmdBase Clone()
		{
			return new evdUserCall(this);
		}

		public override void Load(IseStateInput inReader, IevCmdUI inCmdUI)
		{
			this.Target = inCmdUI.Target;
			base.Load( inReader, inCmdUI );
			this.Name = inReader.ReadString();
		}
	}


}
