﻿using System;
using System.Collections.Generic;
using System.Text;
using glb;

namespace evc
{
	// イベント入力コントロール
	public class evInputControl
	{
		// シングルトン
		private static evInputControl inInstance = new evInputControl();
		public static evInputControl Instance { get { return evInputControl.inInstance; } }


		private glbVector2	sBfoPointer = new glbVector2();			// 直前のマウス位置
		private ButtonType	eFirstPushButton = default(ButtonType);	// 直前まで押されてなく、今回の更新で押されたボタン
		private ButtonType	ePushButton = default(ButtonType);		// 現在の押されているボタン
		private ButtonType	eBfoPushButton = default(ButtonType);	// 直前の押されていたボタン

		public seElem		MostTopHit	  { get; private set; }		// 最初にヒットしたエレメント
		public glbVector2	Pointer		  { get; private set; }		// ポインタ位置を取得
		public bool			IsMovePointer { get; private set; }		// ポインタが動いたか否か


		// 直前まで押されてなく、今回の更新で押されたボタンか否か
		// 複数指定の場合は、いずれかが押されたならtrue。
		internal bool IsFirstPush( ButtonType eBtnType )
		{
			return ( (this.eFirstPushButton & eBtnType) != default(ButtonType) );
		}
		// ボタンが押されている
		// 複数指定の場合は、いずれかが押されたならtrue。
		internal bool IsPush( ButtonType eBtnType )
		{
			return ( (this.ePushButton & eBtnType) != default(ButtonType) );
		}

		// 前回からの移動量を取得
		internal glbVector2 GetMouseMoveOffset()
		{
			glbVector2 sRet = this.Pointer;
			sRet.Sub( this.sBfoPointer );
			return sRet;
		}


		// 入力イベント更新
		public void Update()
		{
			siEventIO inEventIO = seRootScene.Instance.EventIO;
			if ( inEventIO == null ) return;

			// 直前の状態を保存
			this.sBfoPointer = this.Pointer;
			this.eBfoPushButton = this.ePushButton;
			
			inEventIO.Update();
			
			this.Pointer = inEventIO.Pointer;
			this.ePushButton = inEventIO.PushButton;
			this.IsMovePointer = inEventIO.IsMovePointer;

			// 今回の更新で押されたボタンを設定			
			this.eFirstPushButton = (this.ePushButton ^ this.eBfoPushButton) & this.ePushButton;

			// 更新状態を特殊変数へ通知
			evVariableSp.Instance.Update( this );
		}
	}
}
