﻿using System;
using System.Collections.Generic;
using System.Text;
using evc;
using glb;


namespace evc
{
	// 命令実行時データやりとり用インタフェース
	public interface IevCmdUI
	{
		// イベントターゲット
		seElem Target { get ; }

		// 編集リスト
		evVariableGroup LocalVariable { get ; }

		// サブルーチンリスト
		evCmdFlowCallManager CallManager { get; }

		// 対象のシーンマネージャ
		seSceneManager SceneManager { get ; }
	}


	// エレメント毎に存在するイベント管理
	public partial class evManager : IevCmdUI
	{

		#region IevCmdUI メンバ

		public seElem Target { get; set; }

		// サブルーチンリスト
		public evCmdFlowCallManager CallManager { get; private set; }

		// 対象のシーンマネージャ:イベント実行前に毎フレーム設定されます。
		public seSceneManager SceneManager { get; private set; }


		#endregion

		// エレメント命令
		public evCmdFlowIf RootFlow { get; private set; }

		// ローカル変数グループ
		public evVariableGroup LocalVariable { get; private set; }


		// コンストラクタ
		public evManager( seElem inTarget, seSceneManager inManager )
		{
			this.Target = inTarget;
			this.SceneManager = inManager;
			this.CallManager = new evCmdFlowCallManager();
			this.RootFlow = new evCmdFlowIf();
			this.LocalVariable = new evVariableGroup();
		}

		// コンストラクタ(クローン用)
		private evManager( seElem inInTarget,
							seSceneManager inManager,
							evCmdFlowIf inInCmdFlow,
							evVariableGroup inInLocalGroup,
							evCmdFlowCallManager inInCallManager )
		{
			this.Target = inInTarget;
			this.SceneManager = inManager;
			this.RootFlow = inInCmdFlow;
			this.LocalVariable = inInLocalGroup;
			this.CallManager = inInCallManager;
		}


		// エレメントのイベント実行
		public void DoFunc( seSceneManager inTargetScene )
		{
			// 操作対象シーン設定
			this.SceneManager = inTargetScene;
			this.RootFlow.DoFunc( this );
		}


		// クローン作成
		// このクローンは再生時のみ使用することが前提です。
		// 再生時に編集しないものは参照コピーです。
		// 完全にデータを複製したクローンが必要な場合evManagerEditのDeepCloneを使用して下さい。
		public evManager Clone(seElem inTargetElem, seSceneManager inManager)
		{
			evManager inNewEvent = new evManager(
					inTargetElem,
					inManager,
				// 変数を直接参照しているので命令も複製する必要あり。
				// ID参照にすれば複製は必要ありませんが、処理速度とのトレードオフになります。
					(evCmdFlowIf)this.RootFlow.Clone(),
					this.LocalVariable.Clone(),
					this.CallManager );

			// イベントの変数参照を更新
			// 変数の参照を更新
			// 元の変数から対応するクローン変数を取得
			glbFunc<evVariable, evVariable> GetNewVariable = delegate(evVariable inOldVariable)
			{
				if (inOldVariable == null) return null;
				// ローカル変数なら
				int iID = this.LocalVariable.Find(inOldVariable);
				if (iID != evVariableGroup.InvalidID)
				{
					return inNewEvent.LocalVariable.Find(iID);
				}
				// それ以外の変数はクローンは作られないのでそのまま返す。
				return inOldVariable;
			};


			// 全ての命令に対して処理を実行
			// 参照を新しいクローンのものに更新
			inNewEvent.RootFlow.DoAll(delegate(evCmdHeader inCmdHeader)
			{
				evCmd cmd = inCmdHeader as evCmd;
				if ( cmd == null ) return true;

				cmd.DoAllData( delegate( evCmdData cmdData )
				{
					switch (cmdData.DataType)
					{
						// ステータス変数を持つなら
						case evDataType.VariableState:
							{
								evdVariableState v = (evdVariableState)cmdData;
								v.DataVariable = (evVariableState)GetNewVariable( v.DataVariable );
							}
							break;
						case evDataType.VariableStateItem:
							{
								evdVariableStateItem v = (evdVariableStateItem)cmdData;
								v.DataVariable = (evVariableState)GetNewVariable(v.DataVariable);
							}
							break;
						// 数値変数を持つなら
						case evDataType.VariableNum:
							{
								evdVariableNum v = (evdVariableNum)cmdData;
								v.DataVariable = (evVariableNum)GetNewVariable(v.DataVariable);
							}
							break;
						default:
							break;
					}
					return true;
				});
				return true;
			});

			return inNewEvent;
		}


		// コンストラクタ：入力データからインスタンスを作成
		public void Load ( IseStateInput inInput )
		{
			// グローバル変数ロード
			this.LocalVariable.Load( inInput );
			// サブルーチンリストロード
			this.CallManager.Load( inInput, this );
			// イベントフローロード
			this.RootFlow.Load( inInput, this );
		}
	}
}
