﻿using System;
using System.Collections.Generic;
using System.Text;
using evc;
using glb;

namespace evc
{
	// 変数ベース
	public abstract partial class evVariable
	{
		public string Name { get; set; }

		// データを初期値に設定
		public abstract void Init();

		// コンストラクタ：入力データからインスタンスを作成
		public abstract void Load ( IseStateInput inInput );

		public abstract evVariable Clone();

	}

	// ステータス変数
	public partial class evVariableState : evVariable
	{
		public int DefaultValue { get; set; }	// 初期値取得
		public bool MultiSelect { get; set; }	// 複数選択の許可・不許可
		public int Value { get; set; }			// ステータス


		// コンストラクタ
		public evVariableState()
		{
		}

		// データを初期値にする
		public override void Init()
		{
			this.Value = this.DefaultValue;
		}

		// ステータスが有効か否か判定
		public bool IsOn( int iStateID ) { return ((this.Value & iStateID) == iStateID); }


		// ステータスを足す
		public void Add( int iStateID )
		{
			// 複数選択でないなら一度削除
			if ( !this.MultiSelect )
			{
				this.Value = 0;
			}
			this.Value |= iStateID; 
		}

		// ステータスを削除
		public void Remove( int iStateID ) { this.Value &= ~iStateID; }

		// ステータスクリア
		public void Clear() { this.Value = 0; }

	
		// ロード
		public override void Load( IseStateInput inInput )
		{
			this.MultiSelect = inInput.ReadBool();
			seDefine.DebugTrace( "MultiSelect", this.MultiSelect ? 1 : 0 );
			this.DefaultValue = inInput.ReadInt();
			seDefine.DebugTrace( "DefaultValue", this.DefaultValue );
			this.Value = inInput.ReadInt();
			seDefine.DebugTrace( "Value", this.Value );
		}

	
		// クローン作成
		// このメソッドは再生時のみ使用されることが前提です。
		// 編集用データは複製されません。
		public override evVariable Clone()
		{
			evVariableState inNewState = new evVariableState();
			inNewState.Name = this.Name;
			inNewState.MultiSelect = this.MultiSelect;
			inNewState.DefaultValue = this.DefaultValue;
			inNewState.Value = this.Value;
			return inNewState;
		}

	}


	// 数値変数
	public partial class evVariableNum : evVariable
	{
		public int Min { get; set; }			// 最小値の取得・設定
		public int Max { get; set; }			// 最大値の取得・設定
		public bool Loop { get; set; }			// true:ループする false:切り捨てる
		public bool EnableMin { get; set; }		// 下限チェックをつけるか否か
		public bool EnableMax { get; set; }		// 上限チェックをつけるか否か
		public int DefaultValue { get; set; }	// 初期データを取得


		// ステータス
		private int iValue;
		public virtual int Value 
		{
			get { return this.iValue; }
			set 
			{
				this.iValue = value;
				if ( this.Loop && this.EnableMax && this.EnableMin )
				{
					int iLen = this.Max - this.Min;
					this.iValue = (iLen == 0) ? this.Min : (this.iValue % iLen) + this.Min;
				}
				else
				{
					// 上限チェック
					if ( this.EnableMin )
					{
						this.iValue = Math.Max( this.iValue, this.Min );
					}
					if ( this.EnableMax )
					{
						this.iValue = Math.Min( this.iValue, this.Max );
					}
				}

			}
		}

		// コンストラクタ
		public evVariableNum()
		{
		}


		// データを初期値にする
		public override void Init()
		{
			this.Value = this.DefaultValue;
		}

		// コンストラクタ：入力データからインスタンスを作成
		public override void Load ( IseStateInput inInput )
		{
			this.Min = inInput.ReadInt();
			seDefine.DebugTrace( "Min", this.Min );
			this.Max = inInput.ReadInt();
			seDefine.DebugTrace( "Max", this.Max );
			this.EnableMin = inInput.ReadBool();
			seDefine.DebugTrace( "EnableMin", this.EnableMin ? 1 : 0 );
			this.EnableMax = inInput.ReadBool();
			seDefine.DebugTrace( "EnableMax", this.EnableMax ? 1 : 0 );
			this.DefaultValue = inInput.ReadInt();
			seDefine.DebugTrace( "DefaultValue", this.DefaultValue );
			this.Loop = inInput.ReadBool();
			seDefine.DebugTrace( "Loop", this.Loop ? 1 : 0 );
//			this.iValue = inInput.ReadInt();
		}

		// クローン作成
		public override evVariable Clone()
		{
			evVariableNum inNum = new evVariableNum();
			inNum.Name = this.Name;
			inNum.Min = this.Min;
			inNum.Max = this.Max;
			inNum.Loop = this.Loop;
			inNum.EnableMin = this.EnableMin;
			inNum.EnableMax = this.EnableMax;
			inNum.DefaultValue = this.DefaultValue;
			inNum.iValue = this.iValue;

			return inNum;
		}
	}


	// 乱数
	public partial class evVariableRandomNum : evVariableNum
	{
		private Random inRandom = new Random( DateTime.Now.Millisecond );

		// 取得する度にランダムな値を返す。
		public override int Value
		{
			get 
			{
				base.Value = this.inRandom.Next();
				return base.Value;
			}

			set { base.Value = value; }
		}

	}

}
