﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace evc
{
	// 変数グループ
	public partial class evVariableGroup : seIDManager<evVariable>
	{
		// コンストラクタ
		public evVariableGroup( int iBaseID ) :base( iBaseID ){}

		private const int LocalBaseID = 1000;
		public evVariableGroup() :base( evVariableGroup.LocalBaseID ){}

		// 再生時に編集されるもののみを複製、それ以外は参照コピーします。
		// 完全なコピーが必要な場合はDeepCloneを使用して下さい。
		public evVariableGroup Clone()
		{
			evVariableGroup inClone = new evVariableGroup( this.BaseID );
			this.DoAll( delegate( int iID, evVariable inVariable )
			{
				inClone.Add( iID, inVariable.Clone() );
				return true;
			});

			return inClone;
		}
	
		// ロード
		public void Load( IseStateInput inReader )
		{
			int iCount = (int)inReader.ReadShort();
			seDefine.DebugTrace( "VariableCount", iCount );
			for (int i = 0; i < iCount; i++)
			{
				evVariable inVariable = inReader.ReadBool() ?
										(evVariable)new evVariableState() :
										(evVariable)new evVariableNum();
				int iID = (int)inReader.ReadShort();
				seDefine.DebugTrace( "Variable ID", iID );
				inVariable.Load( inReader );
				// 初期値を設定
				inVariable.Init();
				this.Add( iID, inVariable );
			}
		}
	}

	// 変数のグループタイプ
	public enum VariableGroupType
	{
		Global,	// 全体
		Local,	// エレメント
		Sp,		// 特殊
	}


	// 変数総括管理(1シーンに１つ)
	public partial class evVariableManager
	{
		private const int GlobalBaseID = 5000;	// 全体変数ID

		// 全体変数管理
		public evVariableGroup GlobalGroup { get; private set; }
		// 特殊変数管理
		public evVariableGroup SpGroup { get { return evVariableSp.Instance; } }


		// コンストラクタ
		public evVariableManager()
		{
			this.GlobalGroup = new evVariableGroup( evVariableManager.GlobalBaseID );
		}
		// コンストラクタ(クローン用)
		public evVariableManager( evVariableGroup inGlobal )
		{
			this.GlobalGroup = inGlobal;
		}


		// 変数が所属するグループを取得
		public VariableGroupType GetGroupType( evVariable inVariable )
		{
			if ( this.GlobalGroup.ContainsValue( inVariable ) ) return VariableGroupType.Global;
			if ( this.SpGroup.ContainsValue( inVariable ) ) return VariableGroupType.Sp;
			return VariableGroupType.Local;
		}

		public void Clear()
		{
			// 全体領域の変数をクリア
			this.GlobalGroup.Clear();
		}

	
		// クローン
		// 再生時に編集されるもののみを複製、それ以外は参照コピーします。
		// 完全なコピーが必要な場合はDeepCloneを使用して下さい。
		public evVariableManager Clone()
		{
			// 特殊変数は簡易コピー
			return new evVariableManager( this.GlobalGroup.Clone() );
		}


		// ファイルのロード
		public void Load( IseStateInput inReader )
		{
			// 全体変数グループのみ。
			// ローカル変数はエレメントと一緒に、特殊変数は動的に作成する
			this.GlobalGroup.Load( inReader );
		}

	}
}
