﻿using System;
using System.Collections.Generic;
using System.Text;
using glb;

namespace evc
{
	public enum VariableSpType
	{
		MouseX = evVariableSp.SpBaseID, // マウス位置Ｘ
		MouseY,							// マウス位置Ｙ
		MouseAddX,						// マウス前回からの移動量Ｘ
		MouseAddY,						// マウス前回からの移動量Ｙ
		Random,							// ランダムな整数を取得
	}

	// 特殊変数リスト
	// プログラムからイベントにデータを渡す場合、
	// この特殊変数を介してやりとりをします。
	public partial class evVariableSp : evVariableGroup
	{
		internal const int SpBaseID = 8000;		// 特殊変数ID

		// シングルトン
		private static evVariableSp inInstance = new evVariableSp();
		public static evVariableSp Instance { get { return evVariableSp.inInstance; } }

		// 特殊変数を取得
		public evVariable Find( VariableSpType eSpType )
		{
			return this.Find( (int)eSpType );
		}

		// コンストラクタ
		public evVariableSp() : base( evVariableSp.SpBaseID )
		{
			// 数値変数を作成
			glbAction<VariableSpType> AddNumVariable = delegate( VariableSpType eSpType )
			{
				this.Add( (int)eSpType, new evVariableNum() );
			};
			AddNumVariable( VariableSpType.MouseAddX );
			AddNumVariable( VariableSpType.MouseAddY );
			AddNumVariable( VariableSpType.MouseX );
			AddNumVariable( VariableSpType.MouseY );
			this.Add( (int)VariableSpType.Random, new evVariableRandomNum() );
		}

		// 状態更新
		public void Update( evInputControl inEventInput )
		{
			// 特殊変数の数値変数に値を設定
			glbAction<VariableSpType,int> SetNumValue = delegate( VariableSpType eSpType, int iInValue )
			{
				evVariableNum inNum = (evVariableNum)this.Find( eSpType );
				inNum.Value = iInValue;
			};

			// マウスが移動したなら
			if ( inEventInput.IsMovePointer )
			{
				glbVector2 sMousePos = inEventInput.Pointer;
				SetNumValue(VariableSpType.MouseX, (int)sMousePos.X );
				SetNumValue(VariableSpType.MouseY, (int)sMousePos.Y );

				glbVector2 sMoveOffset = inEventInput.GetMouseMoveOffset();
				SetNumValue(VariableSpType.MouseAddX, (int)sMoveOffset.X );
				SetNumValue(VariableSpType.MouseAddY, (int)sMoveOffset.Y );
			}
			// 移動していないなら
			else
			{
				SetNumValue(VariableSpType.MouseAddX, 0 );
				SetNumValue(VariableSpType.MouseAddY, 0 );
			}
		}
	}
}
