﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace evc
{
	// リーダークラス
	public class bnInput : IseStateInput
	{
		internal BinaryReader Stream { get; private set; }

		// オープン
		public void Open(string strPath)
		{
			this.Stream = new BinaryReader(new FileStream(strPath, FileMode.Open));
		}

		// オープン
		public void Open( byte[] abBuffer )
		{
			this.Stream = new BinaryReader( new MemoryStream( abBuffer ) );
		}


		// クローズ
		public void Close()
		{
			if ( this.Stream != null )
			{
				this.Stream.Close();
				this.Stream = null;
			}
		}

		public bool ReadBool()		{ return (this.Stream.ReadByte() == 1); }
		public byte ReadByte()		{ return this.Stream.ReadByte(); }

		public short ReadShort()
		{
			ushort s1 = (ushort)Stream.ReadByte();
			ushort s2 = (ushort)Stream.ReadByte();
			s2 *= 256;
			return (short)(s1 + s2);
		}

		public int ReadInt()
		{
			uint i1 = (uint)Stream.ReadByte();
			uint i2 = (uint)Stream.ReadByte() * 256;
			uint i3 = (uint)Stream.ReadByte() * 256 * 256;
			uint i4 = (uint)Stream.ReadByte() * 256 * 256 * 256;
			return (int)(i1 + i2 + i3 + i4);
		}

		[StructLayout(LayoutKind.Explicit)]
		struct cnv_fi
		{
			[FieldOffset(0)] public int i;
			[FieldOffset(0)] public float f;
		};

		public float ReadFloat()
		{
			cnv_fi c;
			c.f = 0.0f;
			c.i = this.ReadInt();
			return c.f;
		}
		public string ReadString()	
		{
			// 最初の２バイトは文字バイト数
			int iBytes = (int)this.ReadShort();
			if ( iBytes <= 0 ) return string.Empty;

			// 次の１バイトが文字列タイプ
			seTextFormat eFormat = (seTextFormat)this.ReadByte();
			byte[] abText = this.Stream.ReadBytes( iBytes );
			Encoding inNowEncode = null;
			switch ( eFormat )
			{
				case seTextFormat.Utf16: inNowEncode = Encoding.Unicode; break;
				case seTextFormat.Shift_Jis: inNowEncode = Encoding.GetEncoding("Shift_JIS"); break;
				default: Debug.Assert( false );  break;
			}

			// string にエンコード
			return inNowEncode.GetString( abText, 0, abText.Length );
		}


		public T ReadEnum<T>() where T : struct
		{
			// Enumの中で一番値の大きなものを取得(正数のみ)
			int iEnumMaxValue = seIOUtil.EnumMaxValue<T>();
			if ( iEnumMaxValue <= byte.MaxValue ) return (T)Enum.ToObject( typeof(T), this.ReadByte() );
			if ( iEnumMaxValue <= short.MaxValue ) return (T)Enum.ToObject( typeof(T), this.ReadShort() );
			return (T)Enum.ToObject( typeof(T), this.ReadInt());
		}

	}
}
