﻿using System;
using System.Collections.Generic;
using System.Text;

namespace evc
{
	public class seAnimInfo
	{
		// アニメーション名の取得・設定
		public string Name		{ get; set; }
		// コンストラクタ
		internal seAnimInfo ( string strName )
		{
			this.Name = strName;
		}
	}

	// シーンアニメーション管理
	public partial class seAnimManager : seIDManager<seAnimInfo>
	{
		// アクティブなアニメーション取得
		public int ActiveAnim { get; set; }


		public seAnimManager()
		{
			this.ActiveAnim = seAnimManager.InvalidID;
		}

		// 全データクリア
		public new void Clear()
		{
			base.Clear();
			this.ActiveAnim = seAnimManager.InvalidID;
		}

		// アニメーション名の登録
		public int SetAnim( string strAnim )
		{
			this.SetAnim( strAnim, base.GetFreeID() );
			return this.ActiveAnim;
		}
		// アニメーションＩＤ指定
		public void SetAnim( string strAnim, int iAnimID )
		{
			// 既に登録されているならそのＩＤを返して終了
			this.ActiveAnim = this.FindID( strAnim );
		
			// なければ新規に追加
			if ( this.ActiveAnim == seAnimManager.InvalidID )
			{
				this.ActiveAnim = iAnimID;
				base.Add( iAnimID, new seAnimInfo( strAnim ) );
			}
		}

		// 最初のアニメーションIDを取得
		public int GetFirstAnimID()
		{
			int iRetID = seAnimManager.InvalidID;
			base.DoAll( delegate( int iAnimID, seAnimInfo inAnimInfo )
			{ 
				// 最初のを返して終了
				iRetID = iAnimID;
				return false;
			});

			return iRetID;
		}

		// 名前から登録してあるＩＤを検索
		public int FindID( string strName )
		{
			int iRetID = seAnimManager.InvalidID;
			base.DoAll( delegate( int iAnimID, seAnimInfo inAnimInfo )
			{ 
				// 該当する名前があればそのＩＤを返して終了
				if ( inAnimInfo.Name == strName )
				{
					iRetID = iAnimID;
					return false;
				}
				return true;
			});
			return iRetID;
		}


		// ロード
		internal void Load( IseStateInput inInput )
		{
			// アニメーションリストをロード
			this.Clear();
			int iAnimCount = (int)inInput.ReadShort();
			seDefine.DebugTrace( "AnimCount", iAnimCount );
			for (int i = 0; i < iAnimCount; i++)
			{
				int iAnimID = (int)inInput.ReadShort();
				string strName = inInput.ReadString();
				this.SetAnim( strName, iAnimID ); 
			}
		}
	
	}
}
