﻿using System.Collections.Generic;
using System.Diagnostics;
using glb;


namespace evc
{
	/**
	 * 表示オブジェクト管理クラス
	 */
	public partial class seObjectManager : seIDManager<seObject>
	{
		private const int OBJECT_BASE_ID = 6000;	// ベースオブジェクトＩＤ
	
		private seSceneManager _scene;

		// コンストラクタ
		internal seObjectManager( seSceneManager scene )
			: base( seObjectManager.OBJECT_BASE_ID )
		{
			this._scene = scene;
		}


		public override int Add(seObject obj)
		{
			int iNewID = base.Add(obj);
			obj.Init( iNewID, this._scene );
			return iNewID;
		}

		// IDを指定してオブジェクトを追加する時
		public override void Add(int iNewObjectID, seObject newObj )
		{
			seObject oldObj = this.Find(iNewObjectID);

			base.Add( iNewObjectID, newObj );
			// 上書きなら全ての要素に対してオブジェクトの再設定をする
			if (oldObj != null)
			{
				this._scene.ElemManager.ResetObjectWork(newObj, oldObj);
				oldObj.Dispose();
			}
			newObj.Init( iNewObjectID, this._scene );
		}



		// ロード
		internal void Load( IseStateInput input )
		{
			this.Clear();
			// オブジェクトデータをロード
			int iObjectCount = (int)input.ReadShort();
			seDefine.DebugTrace( "ObjectCount", iObjectCount );
			for (int i = 0; i < iObjectCount; i++)
			{
				emObjectType eObjectType = input.ReadEnum<emObjectType>();
				seObject inObject = seObject.Create( eObjectType );
				inObject.Load( input, this._scene );
				this.Add( inObject.ObjectID, inObject );
			}

			// オブジェクトのロードが全て終わった後に処理するもの
			this.DoAll(delegate( int iObjID, seObject obj )
			{
				// メッシュオブジェクトの親子構造を構築する。
				if ( obj.ElemType == emObjectType.Mesh )
				{
					seoMesh mesh = (seoMesh)obj;
					mesh.LinkMechChild( this );
				}
				return true;
			});
		}

	}

}
