﻿using glb;

namespace evc
{
	// 文字フォーマットタイプ
	public enum seTextFormat
	{
		Utf16 = 0,
		Shift_Jis,
	}

	// パス設定
	public enum seOutputPath
	{
		Non,	// パスなし(ファイル名のみ)
		Global,	// 絶対パス
		Local,	// 相対パス
	}


	// ファイル別オプション管理
	public partial class seOption
	{
		public glbColor BgColor {get; set;}			// 背景色の取得・設定
		public int FrameNum {get; set;}				// フレーム数の取得・設定
		public glbVector2 ScreenSize {get; set;}	// スクリーンサイズの取得・設定
		public float FrameTime {get; set;}			// １フレームの秒数を取得・設定
		public seOutputPath PathMode {get; set; }	// パス出力モード
		public bool AddExt { get;set; }				// 拡張子を出力
		public bool NoDebugData { get; set; }		// デバッグデータを出力しない
		public bool CopyFile { get; set; }			// 使用するファイルをコピー
		public bool IncludeElemName { get; set; }	// エレメント名をデータ内に持つ
		public bool IncludeKeyWord { get; set; }	// キーワードをデータ内に持つ
		public seTextFormat MsgFormat { get; set; }	// テキストフォーマット
		public seTextFormat InsideName { get; set; }// 内部文字列(リソース名、アニメーション名)フォーマット


		// コンストラクタ
		internal seOption ()
		{
			this.Clear();
		}
		
		// クリア:初期値に戻す
		internal void Clear()
		{
			this.FrameNum = 0;	// 最大フレーム(現在は未使用)
			this.BgColor = new glbColor( 0, 0, 0, 255 );
			this.ScreenSize = new glbVector2( 200.0f, 200.0f );
			this.FrameTime = 100.0f;
			this.PathMode = seOutputPath.Local;
			this.AddExt = true;
			this.CopyFile = false;
			this.IncludeElemName = false;
			this.IncludeKeyWord = true;
			this.NoDebugData = false;

			this.MsgFormat = seTextFormat.Utf16;			// 出力文字列フォーマット
			this.InsideName = seTextFormat.Shift_Jis;		// 内部文字列フォーマット
#if PL_EDITOR
			// 編集専用クリア
			this.ClearEdit();
#endif	
		}

		// 入力
		public void Load( IseStateInput inInput )
		{
			this.BgColor = seIOUtil.ReadColor( inInput ); 
			this.ScreenSize = seIOUtil.ReadVector2( inInput );

			this.FrameNum = inInput.ReadInt();
			this.FrameTime = inInput.ReadFloat();

			this.PathMode = inInput.ReadEnum<seOutputPath>();
			this.AddExt = inInput.ReadBool();
			this.CopyFile = inInput.ReadBool();
			this.IncludeElemName = inInput.ReadBool();
			this.IncludeKeyWord = inInput.ReadBool();
			this.NoDebugData = inInput.ReadBool();

			this.MsgFormat = inInput.ReadEnum<seTextFormat>();
			this.InsideName = inInput.ReadEnum<seTextFormat>();
		}

	}
}
