﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using glb;

namespace evc
{
	// 外部ファイルデータを管理

	public partial class seResourceManager : seIDManager<siResourceBase>
	{
		// リソース名からリソースを取得
		public siResourceBase Find( string name )
		{
			siResourceBase retResource = null;
			this.DoAll( delegate( int iID, siResourceBase resource )
			{
				if (resource.Name == name)
				{
					retResource = resource;
					return false;
				}
				return true;
			});

			return retResource;
		}

		// リソースを登録
		public siResourceBase Register( seSceneManager scene,
										string fileName, 
										int iResourceID, 
										seResourceType eResType, 
										glbVector2I sTexSize, // テクスチャリソース専用
										bool bAutoLoad )
		{
			// 既に登録済みなら
			siResourceBase resource = this.Find( iResourceID );
			if ( resource != null ) return resource;

			siIOManager inIOManager = seRootScene.Instance.IOManager;
			switch ( eResType )
			{
				// テクスチャ
				case seResourceType.Image:
					resource = inIOManager.CraeteTextureResource( scene, fileName, sTexSize );
					break;
				// サウンド
				case seResourceType.Sound:
					resource = inIOManager.CraeteSoundResource( scene, fileName ); 
					resource.Register( null );
					break;
				// シーン
				case seResourceType.Scene:
					resource = new seResourceScene( fileName ); 
					break;
			}

			// 自動でロードするか否か
			resource.IsAutoLoad = bAutoLoad;

			// リソースリストに登録
			this.Add( iResourceID, resource );
			return resource;
		}


		// コンストラクタ
		public seResourceManager (){}

		// 全データクリア
		internal new void Clear()
		{
#if PL_EDITOR
			this.iActiveID = seResourceManager.InvalidID;
#endif
			base.DoAll( delegate( int iResourceID, siResourceBase resource )
			{
				resource.Dispose();
				return true;
			});
			base.Clear();
		}

		// リソース解放
		public void Release( int iResourceID )
		{
			siResourceBase resource = base.Find( iResourceID );
			if ( resource == null ) return;
			resource.Dispose();
			base.Remove( iResourceID );
		}

		// データロード
		internal void Load( IseStateInput input, seSceneManager scene )
		{
			int iCount = (int)input.ReadShort();
			seDefine.DebugTrace( "ResourceCount", iCount );

			for (int i = 0; i < iCount; i++)
			{
				int iResourceID = input.ReadShort();
				bool bAutoLoad = input.ReadBool();
				seResourceType eType = input.ReadEnum<seResourceType>();
				glbVector2I sTexSize = glbVector2I.Zero;
				// テクスチャの場合、タイプ指定の後にサイズが入る
				if ( eType == seResourceType.Image )
				{
					sTexSize.X = (int)input.ReadShort();
					sTexSize.Y = (int)input.ReadShort();
				}
				string strName = input.ReadString();
				// ロード処理を実行
				// リソースの登録のみをし、ロードはアニメーション内で指定されたタイミングで行います。
				this.Register( scene, strName, iResourceID, eType, sTexSize, bAutoLoad );
			}
		}
	}

}
