﻿using System.Diagnostics;
using glb;

namespace evc
{
	
	
	// 一番上のシーン管理
	public sealed partial class seRootScene
	{
		// シーンがロードされる前の背景色
		private static glbColor DefaultBgColor = new glbColor( 0,0,0,255 );
		// 遅延シングルトン
		public static seRootScene Instance { get; private set; }

		// アクティブなシーンの取得
		private seResourceScene	_sceneResource = null;		// シーンリソース
		private string _animName = string.Empty;			// ロード終了時に実行するアニメーション


		public siIOManager	  IOManager { get; private set; }	// 入出力管理取得
		public IGLMatrix	  Camera	{ get; private set; }	// カメラマトリクスを取得
		public seSceneManager RootScene	{ get; private set; }	// ルートシーンを取得
		public bool			  EditMode	{ get; set;			}	// 編集用、再生用を取得・設定
		public siEventIO	  EventIO	{ get; private set; }	// イベント入出力を取得
		public siMaterial	  Material	{ get; private set; }	// マテリアルを取得

		// 最初のシーン初期化時処理
		public event glbAction<seSceneManager> OnInit = null;

		// イベントリスナー
		public siEventListener EventListener { get; set; }
		// アニメーションリスナー
		public siAnimListener AnimListener { get; set; }


		// ルートシーンを読み込み中か否か
		public bool IsRootLoading { get { return (this._sceneResource != null); } }

		// ルートシーンをキャンセル
		public void RootCancel()
		{
			if ( this._sceneResource == null ) return;
			this._sceneResource.Cancel();
			this._sceneResource.Dispose();
			this._sceneResource = null;
		}

		// ＢＧカラーを取得
		public glbColor BgColor
		{
			get
			{
				if ( this._sceneResource == null ) return seRootScene.DefaultBgColor;
				if ( !this._sceneResource.IsEffective ) return seRootScene.DefaultBgColor;
				return this._sceneResource.Scene.Option.BgColor;
			}
		}


		public void Load( string strSceneName, string strInAnimName )
		{
			// ロード中にロード要求が来た場合。
			if ( this._sceneResource != null )
			{
				this._sceneResource.Dispose();
			}

			this._sceneResource = new seResourceScene( strSceneName );
			this._animName = strInAnimName;
			this._sceneResource.Load( null );
		}

		// 入力更新
		public void UpdateInput()
		{
			if ( this.RootScene == null ) return;
			seElemManager em = this.RootScene.ElemManager;
			if ( em == null ) return;

			// ヒット要素の更新
			em.UpdateHit( evInputControl.Instance.Pointer, true );

			// 入力イベント処理開始
			evInputControl.Instance.Update();

			// イベント実行処理
			em.DoEvent();
		}

		// アニメーション実行
		public void DoAnim( float fAddTime )
		{
			// ルートのシーンをロード中なら
			if ( this._sceneResource != null )
			{
				this._sceneResource.Update();
				if ( this._sceneResource.IsEffective )
				{
					// アクティブシーンを切り替え
					if ( this.RootScene != null )
					{
						this.RootScene.Clear();
					}
					
					this.RootScene = this._sceneResource.PassesScene();
					this.RootScene.RequestAnim( this._animName );
					// リソースの削除
					this._sceneResource.Dispose();
					this._sceneResource = null;

					if (this.OnInit != null)
					{
						this.OnInit(this.RootScene);
					}
					// アニメーション開始処理
					this.RootScene.ElemManager.ResetAnim();
					return;
				}
			}

			// アニメーションの更新
			if ( this.RootScene != null )
			{
				this.RootScene.OnProc( fAddTime );
			}
		}


		// 描画
		public void Draw()
		{
			if ( this.RootScene == null ) return;

			this.RootScene.ElemManager.Draw();
		}


		// 全てリセット
		public void Clear()
		{
			if ( this.RootScene == null ) return;

			// シーンをクリア
			this.RootScene.Clear();
		}

		// コンストラクタ
		public seRootScene( siIOManager ioManager, siEventIO eventIO, siMaterial material )
		{
			Debug.Assert(seRootScene.Instance == null);

			seRootScene.Instance = this;

			this.EventListener = null;
			this.AnimListener = null;

			this.IOManager = ioManager;
			this.Camera = this.IOManager.CreateMatrix();
			this.Camera.Identity();
			this.EventIO = eventIO;
			this.Material = material;
		}
	}
}
