﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using glb;


namespace evc
{
	// キー移動制御
	public enum seKeyMoveContorl
	{
		Non = 0,	// 未設定
		LoopStart,	// ループスタート
		LoopEnd,	// ループ終了
		Wait		// キーを進めない
	}

	// キー継承設定
	public enum seKeySuccess
	{
		Non = 0,	// 未設定
		On,			// 継承ON
		Off,		// 継承OFF
	}


	// 1アニメーションキーフレーム管理クラス
	public partial class seStateKeyInfo
	{
		public const int Infinity = -1;	// 無限

		private List<FePointInfo>	lisPoint = new List<FePointInfo>();	// 制御点リスト

		// データ無効(フラグのみを有効にさせたい場合に立てる)		
		public bool DisableData				{ get; set; }
		public seKeySuccess Success			{ get; set; }			// 継承制御
		public seKeyMoveContorl MoveControl { get; set; }			// キー移動制御
		public int LoopCount				{ get; set; }			// ループ回数
		public int ReturnFrame				{ get; set; }			// ジャンプから戻る時のフレーム
		public string Name					{ get; set; }			// 名前の取得・設定
		public IseStateType Data			{ get; set; }			// キーの値を取得
		public int Frame					{ get; internal set; }	// 時間を取得(前のキーからの差分)
		public emElemState StateType		{ get; private set; }	// ステータスタイプの取得


		// コンストラクタ
		internal seStateKeyInfo()
		{
			this.Name = string.Empty;
		}

		// 補間アニメーションを持つか否か
		internal bool HasSpeed { get { return (this.lisPoint.Count > 0); } }


		// 時間から移動レートを取得
		internal float GetRate(float fTime) 
		{
			// 制御点が無い場合はアニメーションなし
			if ( this.lisPoint.Count <= 0 ) return 0.0f;
			
			// 制御点アニメーションを取得
			return FePointInfo.GetPos( fTime, this.lisPoint );
		}

		// 入力
		public int Load( IseStateInput input )
		{
			int iKeyID = seKeyControlManager.InvalidID;
			this.Name = input.ReadString();
			// キー名がある場合のみキーデータを取得
			if ( this.Name.Length > 0 )
			{
				iKeyID = input.ReadShort();
			}
			this.Frame = (int)input.ReadShort();

			this.DisableData = input.ReadBool();
			this.MoveControl = input.ReadEnum<seKeyMoveContorl>();
			this.Success = input.ReadEnum<seKeySuccess>();
			this.StateType = input.ReadEnum<emElemState>();

			// 本来、ループ終了でなければ以下のデータはセーブしなくて良い。
			this.LoopCount = (int)input.ReadShort();
			this.ReturnFrame = input.ReadInt();

			this.Data = seStateUtil.Instance.CreateStateType( this.StateType );
			this.Data.Load( input );
			
			int iPointNum = (int)input.ReadByte();
			for (int i = 0; i < iPointNum; i++)
			{
				FePointInfo inPointInfo = new FePointInfo();
				inPointInfo.Load( input );
				this.lisPoint.Add( inPointInfo );
			}

#if false
			// 有効なキーの場合は登録
			if ( iKeyID != seKeyControlManager.InvalidID )
			{
				inSceneManager.KeyControlManager.Add( iKeyID, this );
			}
#endif	
			return iKeyID;
		}
	}

}
