﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using glb;

namespace evc
{
	// seElem描画、ヒット判定等をまとめます
	public partial class seElem
	{
		// サイズから矩形頂点算出用
		private static readonly glbVector2[] ainPosBase = new glbVector2[]
		{
			new glbVector2( 0.0f, 0.0f ),
			new glbVector2( 1.0f, 0.0f ),
			new glbVector2( 1.0f, 1.0f ),
			new glbVector2( 0.0f, 1.0f ),
		};


		// 任意の点が矩形内に存在するか否か
		public bool IsInRect ( glbVector2 inVector )
		{
			if ( this.ObjectWork == null ) return false;
			bool bHit = false;
			seMatrixWork.Instance.UseMatrixWork( delegate( IUseWorkMatrix gw )
			{
				IGLMatrix m = gw.GetMatrix();
				this.GetRectMatrix( m );
				glbVector2 sObjectSize = this.ObjectWork.GetSize();
				bHit = seMath.IsInRect( sObjectSize, m, inVector );
			});

			return bHit;
		}

		// ピクセル単位ヒット判定
		internal bool IsHitPixel( glbVector2 sPos )
		{
			if ( this.ObjectWork == null ) return false;
			seObject inObject = this.ObjectWork.Object;

			// 矩形内でテクスチャの場合、ドット単位で判定する
			if ( inObject.ElemType != emObjectType.Texture ) return false;

			// テクスチャが設定されていない場合はヒットしない
			siResourceTexture inTexture = ((IseObjResourceTexture)inObject).ResourceTexture;
			if (inTexture == null) return false;

			glbVector2 sObjectSize = this.ObjectWork.GetSize();

			bool bHit = false;
			seMatrixWork.Instance.UseMatrixWork( delegate( IUseWorkMatrix gw )
			{
				IGLMatrix mInv = gw.GetMatrix();
				mInv.Copy(this.NowMatrix);
				mInv.Invert();

				glbVector2 s = mInv.TransformCoordinate(sPos);
				// 何枚目のループにいるか
				int iLoopX = (int)s.X / (int)sObjectSize.X;
				int iLoopY = (int)s.Y / (int)sObjectSize.Y;

				// 表示オブジェクトのどの辺にあるか算出
				s.X -= (float)iLoopX * sObjectSize.X;
				s.Y -= (float)iLoopY * sObjectSize.Y;


				// オブジェクトのUVオフセット
				glbVector2 sObjUV = ((seoTextureSprite)inObject).UV;

				// UVが移動しているなら
				seStateFloat2 inUV = (seStateFloat2)this.State[emElemState.UV];
				glbVector2 sUVOffset = inUV.Value;
				sUVOffset.Add(sObjUV);

				// 整数部を消す。(1で割った余り)
				sUVOffset.X = sUVOffset.X - (float)((int)sUVOffset.X);
				sUVOffset.Y = sUVOffset.Y - (float)((int)sUVOffset.Y);
				// マイナスならループ
				if (sUVOffset.X < 0.0f) sUVOffset.X += 1.0f;
				if (sUVOffset.Y < 0.0f) sUVOffset.Y += 1.0f;
				sUVOffset.Mul(new glbVector2(inTexture.TextureSize));


				glbVector2I sHitPos = new glbVector2I((int)(s.X + sUVOffset.X),
														(int)(s.Y + sUVOffset.Y));

				// ループがあるならその分もスケールを加算
				seStateLoop inLoop = (seStateLoop)this.State[emElemState.Loop];
				if (inLoop != null)
				{
					// 反転で奇数のループ上にある場合
					if (inLoop.RevW && ((iLoopX & 1) == 1))
					{
						// 左右判定位置を逆にする
						sHitPos.X = (int)(sUVOffset.X + sObjectSize.X - s.X);
					}

					// 反転で奇数のループ上にある場合
					if (inLoop.RevH && ((iLoopY & 1) == 1))
					{
						// 上下判定位置を逆にする
						sHitPos.Y = (int)(sUVOffset.Y + sObjectSize.Y - s.Y);
					}
				}

				sHitPos.X %= inTexture.TextureSize.X;
				sHitPos.Y %= inTexture.TextureSize.Y;
				bHit = inTexture.IsHit(sHitPos);
			});
			return bHit;
		}



		// ループの分のサイズベースマトリクスに掛ける
		// @param m ：出力用マトリクス
		// @param mBase : ベースマトリクス
		private void MulLoopMatrix( IGLMatrix m, IGLMatrix mBase )
		{
			// オブジェクトワークが設定されていないなら何もしない
			if ( this.ObjectWork == null ) return;

			m.Copy( mBase );

			// マトリクスワークを使用
			seMatrixWork.Instance.UseMatrixWork( delegate( IUseWorkMatrix g )
			{
				// ループがあるならその分もスケールを加算
				seStateLoop loop = (seStateLoop)this.State[ emElemState.Loop ];
				if (loop != null)
				{
					IGLMatrix ml = g.GetMatrix();
					IGLMatrix mPosOffset = g.GetMatrix();
					glbVector2 sOffset = this.ObjectWork.GetSize();
					if (loop.LoopCountW >= 0 ) sOffset.X = glbVector2.Zero.X;
					if (loop.LoopCountH >= 0 ) sOffset.Y = glbVector2.Zero.Y;

					mPosOffset.SetPos(sOffset);
					ml.SetScale((float)loop.LoopCountW, (float)loop.LoopCountH);

					m.Mul( mPosOffset );
					m.Mul( ml );
				}

				// seoMeshChildの場合は特殊処理
				if ( this.ObjectWork.Object.ElemType == emObjectType.MeshChild )
				{
					seoMeshChild meshChild = (seoMeshChild)this.ObjectWork.Object;
					IGLMatrix mpos = g.GetMatrix();
					mpos.SetPos( meshChild.Pos );
					m.Mul( mpos );
				}
			});
		}


		// ループ数も加味した現在の要素の矩形マトリクスを取得
		// 要は要素の頂点を表すマトリクス
		public void GetRectMatrix( IGLMatrix m )
		{
			this.MulLoopMatrix( m, this.State.NowMatrix);
		}

#if true
		// 現状、再生ではイベントの矩形同士のヒットにのみ使われている。
		// 
		// →マトリクス(m)とサイズ(size)で表す四角形(quad)２つの当たり判定は
		// quad1 = m1 * size1
		// quad2 = m2 * size2
		// で、quad1とquad2の交差や内包判定するより、
		// quad2' = m1' * m2 * size2;  // m1'はm1の逆マトリクス
		// と、長方形size1と比較した方が早い。
		// 
		// ので、コレに変更した時に削除される予定です。
		// 

		// オブジェクオの矩形を取得
		// 現在のアニメーションを含めた矩形になっている。
		public glbVector2[] GetRect()
		{
			if (this.ObjectWork == null) return null;

			glbVector2 vSize = this.ObjectWork.GetSize();
			glbVector2[] aRetList = new glbVector2[seElem.ainPosBase.Length];

			seMatrixWork.Instance.UseMatrixWork(delegate(IUseWorkMatrix gw)
			{
				IGLMatrix m = gw.GetMatrix();
				this.GetRectMatrix(m);

				int iCount = 0;
				Array.ForEach( seElem.ainPosBase, delegate(glbVector2 sPos)
				{
					glbVector2 vWork = sPos;
					vWork.Mul(vSize);
					aRetList[iCount] = m.TransformCoordinate(vWork);
					iCount++;
				});
			});

			return aRetList;
		}

#endif



		// 自分以下の全ての子に対して描画処理
		public void Draw( siMaterial inMaterial )
		{
			// イベントで要素作り中に描画は入ってくることがある。
			if ( !this.IsVisible ) return;

			// 対象オブジェクトがないなら無処理
			if ( this.ObjectWork != null)
			{
				this.State.SetMaterial( inMaterial );

				// マスク適応処理
				switch (this.State.Attrbute.Mask)
				{
					// マスク領域を通知
					case seAttrMask.Data:
						seMatrixWork.Instance.UseMatrixWork( delegate( IUseWorkMatrix g )
						{
							IGLMatrix m = g.GetMatrix();

							glbVector2 vSize = this.ObjectWork.GetSize();
							this.GetRectMatrix( m );
							seRootScene.Instance.IOManager.RegisterMask( m, vSize );
						});
						break;
					default:
						break;
				}

				this.ObjectWork.Draw();

				// マスク終了なら
				if (this.State.Attrbute.Mask == seAttrMask.End)
				{
					// マスク終了を通知
					seRootScene.Instance.IOManager.ClearMask();
				}

			}
		}

	}
}
