﻿using System;
using System.Collections.Generic;
using System.Text;
using glb;


namespace evc
{

	// テクスチャ当たり判定
	public enum seTexHit
	{
		Non = 0,	// 未設定
		Rectangle,	// 矩形単位
		Pixel,		// ピクセル単位
	}

	// テクスチャアドレス設定
	public enum seTexAddress
	{
		Non = 0,	// 未設定(アニメーション時のみ設定可能です)
		Wrap,		// 範囲を超えたテクスチャはループ
		Clamp,		// 範囲を超えたテクスチャは最後のドットが伸ばされる。
	}


	// テクスチャ制御
	public partial class seStateTextureControl : IseStateType
	{
		public seTexHit		Hit		{ get; set; }		// テクスチャ当たり判定設定
		public seTexAddress Address { get; set; }		// テクスチャアドレス設定


		public seStateTextureControl()
		{
			this.Hit = seTexHit.Non;
			this.Address = seTexAddress.Non;
		}

		public void Set(IseStateType inInState)
		{
			seStateTextureControl t = (seStateTextureControl)inInState;
			this.Hit = t.Hit;
			this.Address = t.Address;
		}

		public void Add(IseStateType state)
		{
			seStateTextureControl tc = (seStateTextureControl)state;
			if (tc.Hit != seTexHit.Non) this.Hit = tc.Hit;
			if (tc.Address != seTexAddress.Non) this.Address = tc.Address;
		}

		public void Anim(IseStateType next, IseStateType before, float fRate){}

		public void Load(IseStateInput inInput)
		{
			this.Hit = inInput.ReadEnum<seTexHit>();
			this.Address = inInput.ReadEnum<seTexAddress>();
		}

		public IseStateType Clone()
		{
			seStateTextureControl c = new seStateTextureControl();
			c.Set( this );
			return c;
		}
	}



	// ロード設定
	public enum seLoadControl
	{
		Load = 0,	// ロード
		Release,	// 解放
	}

	// ロードコントロールクラス
	public partial class seStateLoadControl : IseStateType
	{
		// ステータスの取得・設定
		public seLoadControl State { get; set; }

		// 指定のデータが有効か否か
		public bool IsEnable( seLoadControl eCheckType )
		{
			return ((this.State & eCheckType) == eCheckType);
		}

		// コンストラクタ
		public seStateLoadControl(){}
		public seStateLoadControl( seLoadControl eInState )
		{
			this.State = eInState;
		}

		// クローン
		public IseStateType Clone ()
		{
			return new seStateLoadControl( this.State );
		}

		public void Set(IseStateType inInState)
		{
			this.State = ((seStateLoadControl)inInState).State;
		}

		public void Add(IseStateType state){ this.Set(state); }

		public void Anim(IseStateType next, IseStateType before, float fRate){}

		public void Load(IseStateInput inInput)
		{
			this.State = inInput.ReadEnum<seLoadControl>();
		}
	}





	// サウンド設定
	public enum seSoundControl
	{
		Non = 0,		// 無処理
		Play,			// サウンド再生
		PlayLoop,		// サウンドループ再生
		Stop,			// サウンド停止
	}


	// サウンドコントロールクラス
	public partial class seStateSoundControl : IseStateType
	{
		public seSoundControl State { get; set; }

		// コンストラクタ
		public seStateSoundControl(){}
		public seStateSoundControl( seSoundControl eInSoundControl )
		{
			this.State = eInSoundControl;
		}

		// クローン
		public IseStateType Clone ()
		{
			return new seStateSoundControl( this.State );
		}

		public void Set(IseStateType inInState)
		{
			this.State = ((seStateSoundControl)inInState).State;
		}

		public void Add(IseStateType state)
		{
			seStateSoundControl sc = (seStateSoundControl)state;
			if (sc.State != seSoundControl.Non) this.State = sc.State;
		}

		public void Anim(IseStateType next, IseStateType before, float fRate){}

		public void Load(IseStateInput inInput)
		{
			this.State = inInput.ReadEnum<seSoundControl>();
		}
	}







	// ブレンド設定
	public enum seAttrBlend
	{
		Non = 0,	// 未設定(アニメーション時のみ設定可能です)
		Normal,		// 通常ブレンド
		Add,		// 加算
		Sub,		// 減算
	}

	// マスク設定
	public enum seAttrMask
	{
		Non = 0,	// 未設定(アニメーション時のみ設定可能です)
		NoUse,		// マスク使用しない
		Data,		// マスク
		Reflection,	// マスク適応
		End,		// マスク終了
	}

	// 表示設定
	// 子要素で表示設定がされていない場合に限り
	// 親要素の表示設定を継承します。
	public enum seAttrVisible
	{
		Non = 0,	// 未設定(アニメーション時のみ設定可能です)
		Show,		// 表示
		Hide,		// 非表示
		Succes,		// 親の状態を継承
	}


	// 属性クラス
	public partial class seStateAttribute : IseStateType
	{
		public seAttrBlend	 Blend	 { get; set; }		// ブレンド設定
		public seAttrMask	 Mask	 { get; set; }		// マスク設定
		public seAttrVisible Visible { get; set; }		// 表示設定

		// コンストラクタ
		public seStateAttribute (){}

		// クローン
		public IseStateType Clone ()
		{
			seStateAttribute a = new seStateAttribute();
			a.Set( this );
			return a;
		}

		public void Set(IseStateType s)
		{
			seStateAttribute a = (seStateAttribute)s;
			this.Blend = a.Blend;
			this.Mask = a.Mask;
			this.Visible = a.Visible;
		}

		public void Add(IseStateType state)
		{
			seStateAttribute at = (seStateAttribute)state;
			if (at.Blend != seAttrBlend.Non) this.Blend = at.Blend;
			if (at.Mask != seAttrMask.Non) this.Mask = at.Mask;
			if (at.Visible != seAttrVisible.Non) this.Visible = at.Visible;
		}


		public void Anim(IseStateType next, IseStateType before, float fRate){}

		public void Load(IseStateInput inInput)
		{
			this.Blend = inInput.ReadEnum<seAttrBlend>();
			this.Mask = inInput.ReadEnum<seAttrMask>();
			this.Visible = inInput.ReadEnum<seAttrVisible>();
		}

	}



	// フレームコントロールクラス(ロード待ち、アニメーションジャンプ用)
	public partial class seStateFrameCtrl : IseStateType
	{
		internal seSceneManager SceneManager { get; set; }		// ターゲットシーン
		public int	FinishLoadAnimID { get; set; } 		// ロード終了時なら実行するアニメーション
		public int	LoadingAnimID	 { get; set; }		// ロード中なら移動するアニメーション
		public bool IsWatchAll		 { get; set; }		// 全て監視フラグ:

		// 監視リソースリスト
		public List<int>	lisWatchID = new List<int>();


		public seStateFrameCtrl()
		{
			this.FinishLoadAnimID = seAnimManager.InvalidID;
			this.LoadingAnimID = seAnimManager.InvalidID;
			this.IsWatchAll = false;
		}


		// 監視対象のリソースか判定
		public bool IsWatch( int iResourceID )
		{
			return this.lisWatchID.Contains( iResourceID );
		}

		// 監視リソースを追加
		public void AddWatch( int iResourceID )
		{
			if ( !this.lisWatchID.Contains( iResourceID ) )
			{
				this.lisWatchID.Add( iResourceID );
			}
		} 
		// 監視リソースを削除
		public void RemoveWatch( int iResourceID )
		{
			this.lisWatchID.Remove( iResourceID );
		} 
		// 監視リソースをクリア
		public void ClearWatch()
		{
			this.lisWatchID.Clear();
		}

		// チェック
		public void DoCheck()
		{
			seResourceManager inResourceManager = this.SceneManager.ResourceManager;
			bool bIsLoadEnd = true;
			
			// 全てのシーンリソースがロード終了しているなら
			if ( this.IsWatchAll )
			{
				// リソースマネージャ全てに対して処理意を実行
				inResourceManager.DoAll( delegate(int iResourceID, siResourceBase inResource )
				{
					seResourceScene inScene = inResource as seResourceScene;
					if ( inScene == null ) return true;
					if ( !inScene.IsEffective )
					{
						bIsLoadEnd = false;
						return false;
					}
					return true;
				});
			}
			// 特定のリソースのロードが終了しているなら
			else
			{
				foreach (int iResourceID in this.lisWatchID)
				{
					siResourceBase inResource = inResourceManager.Find( iResourceID );
					inResource.Update();
					// １つでも使用できないリソースがあるなら
					if ( !inResource.IsEffective )
					{
						bIsLoadEnd = false;
						break;
					}
				}
			}

			// ロードが終了しているなら
			int iNextAnimID = ( bIsLoadEnd ) ? this.FinishLoadAnimID
											 : this.LoadingAnimID;

			// 次のフレームで実行されるアニメーション
			if ( iNextAnimID != seAnimManager.InvalidID )
			{
				this.SceneManager.RequestAnim( iNextAnimID );
			}
		}


		// データ設定
		public void Set(IseStateType inInState )
		{
			seStateFrameCtrl m = (seStateFrameCtrl)inInState;
			
			this.FinishLoadAnimID = m.FinishLoadAnimID;
			this.LoadingAnimID = m.LoadingAnimID;
			this.IsWatchAll = m.IsWatchAll;

			this.lisWatchID.Clear();
			this.lisWatchID.AddRange( m.lisWatchID );
		}


		public void Add(IseStateType state) { this.Set(state);}



		public void Load ( IseStateInput inInput )
		{
			this.lisWatchID.Clear();

			this.FinishLoadAnimID = (int)inInput.ReadShort();
			this.LoadingAnimID = (int)inInput.ReadShort();
			this.IsWatchAll = inInput.ReadBool();

			int iWatchCount = (int)inInput.ReadShort();
			for (int i = 0; i < iWatchCount; i++)
			{
				this.lisWatchID.Add( (int)inInput.ReadShort() );
			}
		}

		// クローン
		public IseStateType Clone ()
		{
			seStateFrameCtrl c = new seStateFrameCtrl();
			c.Set( this );
			return c;
		}

		public void Anim(IseStateType next, IseStateType before, float fRate){}
	}



	// テキスト制御ステータス
	public enum seTextControl
	{
		AllDraw	= 0,		// 強制的に１ページ全表示
		TextSpeed,		// テキストの表示速度
	}

	// テキスト表示コントロールクラス
	public partial class seStateTextControl : IseStateType
	{
		public int			 Page	{ get; set; }
		public seTextControl State	{ get; set; }

		// コンストラクタ
		public seStateTextControl (){}


		// クローン
		public IseStateType Clone ()
		{
			seStateTextControl c = new seStateTextControl();
			c.Set( this );
			return c;
		}

		public void Set(IseStateType inInState)
		{
			seStateTextControl t = (seStateTextControl)inInState;
			this.Page = t.Page;
			this.State = t.State;
		}

		public void Add(IseStateType state) { this.Set(state);}

		public void Anim(IseStateType next, IseStateType before, float fRate){}

		public void Load(IseStateInput inInput)
		{
			this.Page = (int)inInput.ReadShort();
			this.State = inInput.ReadEnum<seTextControl>();
		}
	}


	// 数値イメージクラス(オブジェクト用)
	// オブジェクトとやりとりする時のみ使用されます。
	// AnimやSetは入りません
	public partial class seStateNumImage : IseStateType
	{
		private static readonly int ListNum = EnumList<seoNumImage>.GetValues().Length;

		public seObjectManager ObjectManager { get; set; }

		private seObject[] ainImageList = new seObject[ seStateNumImage.ListNum ];


		// 全てのイメージに対して処理を実行
		public bool DoAll( glbFunc<seoNumImage,seObject,bool> inFunc )
		{
			for (int i = 0; i < seStateNumImage.ListNum; i++)
			{
				if ( !inFunc((seoNumImage)i, this.ainImageList[ i ]) ) return false;
			}
			return true;
		}

		// イメージ設定
		public void SetImage( seoNumImage eImageType, seObject inObject )
		{
			this.ainImageList[ (int)eImageType ] = inObject;
		}

		// イメージ取得
		public seObject GetImage( seoNumImage eImageType )
		{
			return this.ainImageList[ (int)eImageType ];
		}

		// コンストラクタ
		public seStateNumImage ()
		{
			// 全て無しで埋める
			for (int i = 0; i < ainImageList.Length; i++)
			{
				this.ainImageList[ i ] = null;
			}
		}

		// クローン
		public IseStateType Clone ()
		{
			seStateNumImage c = new seStateNumImage();
			c.Set( this );
			return c;
		}

		public void Anim(IseStateType next, IseStateType before, float fRate){}

		public void Set(IseStateType inInState)
		{
			seStateNumImage n = (seStateNumImage)inInState;
			this.ObjectManager = n.ObjectManager;
			Array.Copy( n.ainImageList, this.ainImageList, this.ainImageList.Length );
		}

		public void Add(IseStateType state) { this.Set(state);}

		public void Load(IseStateInput inInput)
		{
			for (int i = 0; i < ainImageList.Length; i++)
			{
				this.ainImageList[ i ] = this.ObjectManager.Find( (int)inInput.ReadShort() );
			}
		}
	}



	// 文字揃え位置
	public enum seNumArrange
	{
		Non = 0,	// 未設定(アニメーション時のみ設定可能です)
		L,			// 左揃え
		R,			// 右揃え
		Point		// 小数点揃え
	}

	// 文字幅設定
	public enum seNumCharLen
	{
		Non = 0, // 未設定(アニメーション設定時のみ選択されます)
		Fix,	 // 固定:seStateNumInfo.NumLenが適応されます。
		Change,	 // 可変:数値の幅が適応されます。
	}

	// 文字列表示設定
	public enum seNumVisible
	{
		Non = 0, // 未設定(アニメーション設定時のみ選択されます)
		On,		 // 表示
		Off,	 // 非表示
	}

	// 数値ステータスクラス(アニメーション用)
	public partial class seStateNumInfo : IseStateType
	{
		public seNumArrange Arrange		 { get; set; }		// 属性(NumStateを参照)
		public seNumCharLen CharLen		 { get; set; }		// 文字幅設定
		public seNumVisible VisibleComma { get; set; }		// カンマ表示設定
		public seNumVisible VisibleMinus { get; set; }		// マイナス表示設定
		public seNumVisible ZeroBuries	 { get; set; }		// ０埋め
		public int MaxDisit	 { get; set; }		// 最大桁数
		public int PointDraw { get; set; }		// 小数点以下数
		public int NumLen	 { get; set; }		// 固定文字幅(固定文字幅表示の時のみ使用)
		public int CommaLen	 { get; set; }		// 固定カンマ幅


		// コンストラクタ
		public seStateNumInfo ()
		{
			this.CharLen = seNumCharLen.Change;
			this.VisibleComma = seNumVisible.Off;
			this.VisibleMinus = seNumVisible.Off;
			this.ZeroBuries = seNumVisible.Off;
		}


		// クローン
		public IseStateType Clone ()
		{
			seStateNumInfo c = new seStateNumInfo();
			c.Set( this );
			return c;
		}

		public void Anim(IseStateType next, IseStateType before, float fRate){}


		public void Set(IseStateType state)
		{
			seStateNumInfo n = (seStateNumInfo)state;
			this.Arrange = n.Arrange;
			this.CharLen = n.CharLen;
			this.VisibleComma = n.VisibleComma;
			this.VisibleMinus = n.VisibleMinus;
			this.MaxDisit = n.MaxDisit;
			this.PointDraw = n.PointDraw;
			this.NumLen =  n.NumLen;
			this.CommaLen = n.CommaLen;
			this.ZeroBuries = n.ZeroBuries;
		}

		public void Add(IseStateType state)
		{
			seStateNumInfo n = (seStateNumInfo)state;
			if (n.Arrange != seNumArrange.Non) this.Arrange = n.Arrange;
			if (n.CharLen != seNumCharLen.Non) this.CharLen = n.CharLen;
			if (n.VisibleComma != seNumVisible.Non) this.VisibleComma = n.VisibleComma;
			if (n.VisibleMinus != seNumVisible.Non) this.VisibleMinus = n.VisibleMinus;
			this.MaxDisit = n.MaxDisit;
			this.PointDraw = n.PointDraw;
			this.NumLen =  n.NumLen;
			this.CommaLen = n.CommaLen;
			if (n.ZeroBuries != seNumVisible.Non) this.ZeroBuries = n.ZeroBuries;
		}

	
		public void Load(IseStateInput inInput)
		{
			this.Arrange = inInput.ReadEnum<seNumArrange>();
			this.CharLen = inInput.ReadEnum<seNumCharLen>();
			this.VisibleComma = inInput.ReadEnum<seNumVisible>();
			this.VisibleMinus = inInput.ReadEnum<seNumVisible>();
			this.ZeroBuries = inInput.ReadEnum<seNumVisible>();
			this.MaxDisit = (int)inInput.ReadShort();
			this.PointDraw = (int)inInput.ReadShort();
			this.NumLen =  (int)inInput.ReadShort();
			this.CommaLen = (int)inInput.ReadShort();
		}
	}




	// 回転方向タイプ
	public enum seSpinType
	{
		Non = 0,	// 角度の近い方で回転
		Left,		// 左周り
		Right,		// 右周り
		Direction,	// 移動方向
	};

	// Rotクラス	
	public partial class seStateRot :IseStateType
	{
		private const float fCircle = 2.0f * (float)Math.PI;	// １週の角度
		private const float fCircleH = fCircle * 0.5f;			// 半分

		public float	  Rot	   { get; set; }	// 角度取得・設定
		public seSpinType SpinType { get; set; }	// 回転方向の取得・設定
		public int		  RotNum   { get; set; }	// 回転数の取得・設定



		public seStateRot(): this(0.0f, 0, seSpinType.Non ){}
		public seStateRot( float fRot, int iRotNum, seSpinType eSpinType )
		{
			this.Rot = fRot;
			this.RotNum = iRotNum;
			this.SpinType = eSpinType;
		}

		public IseStateType Clone ()
		{
			seStateRot r = new seStateRot();
			r.Set( this );
			return r;
		}

		
		// 専用アニメーション
		public void Anim(IseStateType next, IseStateType before, float fRate)
		{
			seStateRot r = (seStateRot)before;
			seStateRot nextRot = (seStateRot)next;

			float fRot = nextRot.Rot;		// 目標角度
			float fBefoRot = r.Rot;			// 元の角度

			switch (this.SpinType)
			{
				case seSpinType.Non:
					// 最終値の方が大きいとして考えてみる
					float fLRot = ( fRot < fBefoRot ) ? fRot + seStateRot.fCircle : fRot;
					// 左周りの方が短い
    				if ( (fLRot - fBefoRot) < seStateRot.fCircleH )
    				{
						fRot = fLRot;
						fRot += seStateRot.fCircle * (float)this.RotNum;
    				}
					// 右周りの方が短い
					else
					{
						// 右周りの場合は元の角度の方が大きいものとして考える
						if ( fBefoRot < fRot ){ fBefoRot += seStateRot.fCircle; }
						fBefoRot += seStateRot.fCircle * (float)this.RotNum;
					}
					break;

				// 右周り
				case seSpinType.Right:
					// 最終角度が０度を跨いでアニメーションするなら
					if ( fRot < fBefoRot ) { fRot += seStateRot.fCircle; }
					fRot += seStateRot.fCircle * (float)this.RotNum;
					break;
				// 左周り
				case seSpinType.Left:
					// 右周りの場合は元の角度の方が大きいものとして考える
					if ( fBefoRot < fRot ){ fBefoRot += seStateRot.fCircle; }
					fBefoRot += seStateRot.fCircle * (float)this.RotNum;
					break;
				case seSpinType.Direction:
					// 無処理
					break;
				default:
					break;
			}

			this.Rot = (fRot - fBefoRot) * fRate + fBefoRot;
		}


		public void Set(IseStateType state)
		{
			seStateRot r = (seStateRot)state;
			this.Rot = r.Rot;
			this.SpinType = r.SpinType;
			this.RotNum = r.RotNum;
		}

		public void Add(IseStateType state) { this.Set(state); }
	
		public void Load(IseStateInput inInput)
		{
			this.Rot = inInput.ReadFloat();
			this.SpinType = inInput.ReadEnum<seSpinType>();
			this.RotNum = inInput.ReadShort();
		}
	}


	// loop用クラス
	public partial class seStateLoop : IseStateType
	{
		public bool RevW	  { get; set; }		// 横方向に反転
		public int LoopCountW { get; set; }		// 横方向繰り返し数
		public bool RevH	  { get; set; }		// 縦方向に反転
		public int LoopCountH { get; set; }		// 縦方向繰り返し数


		public seStateLoop() : this( false, 0, false, 0 ){}
		public seStateLoop( bool bEnableRevW, int iInLoopCountW,
							bool bEnableRevH, int iInLoopCountH )
		{
			this.RevW = bEnableRevW;
			this.LoopCountW = iInLoopCountW;
			this.RevH = bEnableRevH;
			this.LoopCountH = iInLoopCountH;
		}

		public IseStateType Clone ()
		{
			return new seStateLoop( this.RevW, this.LoopCountW,
									this.RevH, this.LoopCountH );
		}
	
		public void Set(IseStateType inInState)
		{
			seStateLoop l = (seStateLoop)inInState;
			this.RevH = l.RevH;
			this.LoopCountH = l.LoopCountH;
			this.RevW = l.RevW;
			this.LoopCountW = l.LoopCountW;
		}

		public void Add(IseStateType state){ this.Set(state); }

		public void Anim(IseStateType next, IseStateType before, float fRate)
		{
			// ループ数のみアニメーション
			seStateLoop l = (seStateLoop)before;
			seStateLoop nl = (seStateLoop)next;
			this.LoopCountW = (int)((float)(nl.LoopCountW - l.LoopCountW) * fRate) + l.LoopCountW;
			this.LoopCountH = (int)((float)(nl.LoopCountH - l.LoopCountH) * fRate) + l.LoopCountH;
		}

		public void Load(IseStateInput inInput)
		{
			this.RevH = inInput.ReadBool();
			this.LoopCountH = (int)inInput.ReadShort();
			this.RevW = inInput.ReadBool();
			this.LoopCountW = (int)inInput.ReadShort();
		}
	}


	// 移動タイプ
	public enum sePosType
	{
		Line,	// ライン
		Path	// パス
	}

	// 位置クラス
	public partial class seStatePos : IseStateType
	{
		public const float Power = 3.0f;	// パワー係数

		public glbVector2 Value { get; set; }	// 位置
		public sePosType Type { get; set; }	// 移動タイプ
		public float[] ExData { get; private set; }		// タイプ別データ

		public seStatePos()
		{
			this.Value = glbVector2.Zero;
			this.Type = sePosType.Line;
			this.ExData = new float[ 2 ];	// 拡張データは最大領域を予め確保
		}
		public seStatePos( seStatePos mst )
		{
			this.ExData = new float[ 2 ];
			this.Set( mst );
		}

		public IseStateType Clone ()
		{
			return new seStatePos( this );
		}
	
		public void Set(IseStateType state)
		{
			seStatePos s = (seStatePos)state;
			this.Value = s.Value;
			this.Type = s.Type;
			Array.Copy(s.ExData, this.ExData, s.ExData.Length);
		}

		public void Add(IseStateType state) { this.Set(state);}

		public void Anim(IseStateType next, IseStateType before, float fRate)
		{
			seStatePos b = (seStatePos)before;
			seStatePos n = (seStatePos)next;
			glbVector2 v = n.Value;

			switch (this.Type)
			{
				// 直線
				case sePosType.Line:
					v.Sub( b.Value );
					v.Scale( fRate );
					v.Add( b.Value );
					break;

				// パス
				case sePosType.Path:
					glbVector2 bPower = glbVector2.Zero;
					// １つ前のタイプ
					switch (b.Type)
					{
						// 直線
						case sePosType.Line:
							break;
						// パス
						case sePosType.Path:
							bPower = new glbVector2(b.ExData[0], b.ExData[1]);
							break;
						default:
							break;
					}

					v = seMath.GetHermiteCurvePoint(
							b.Value, bPower,
							v, new glbVector2(n.ExData[0], n.ExData[1]),
							fRate, seStatePos.Power );
					break;
				default:
					break;
			}

			 this.Value = v;
		}

		public void Load(IseStateInput input)
		{
			this.Value	= seIOUtil.ReadVector2( input );
			this.Type = input.ReadEnum<sePosType>();
			int iCount = (int)input.ReadByte();

			// 拡張データロード
			for (int i = 0; i < iCount; i++)
			{
				this.ExData[ i ] = input.ReadFloat();
			}
		}
	}





	// グリッドコントロールクラス
	public partial class seStateGridPoint : IseStateType
	{
		public glbVector2 Value { get; set; }	// 位置
		public glbVector2 PowerH { get; set; }	// 水平制御点
		public glbVector2 PowerV { get; set; }	// 垂直制御点


		public seStateGridPoint()
		{
			this.Value = glbVector2.Zero;
			this.PowerH = glbVector2.Zero;
			this.PowerV = glbVector2.Zero;
		}
		public seStateGridPoint( seStateGridPoint mst )
		{
			this.Set(mst);
		}

		public IseStateType Clone () { return new seStateGridPoint( this ); }
	
		public void Set(IseStateType state)
		{
			seStateGridPoint mst = (seStateGridPoint)state;
			this.Value = mst.Value;
			this.PowerH = mst.PowerH;
			this.PowerV = mst.PowerV;
		}

		public void Add(IseStateType state) { this.Set(state);}

		public void Anim(IseStateType next, IseStateType before, float fRate)
		{
			glbFunc<glbVector2,glbVector2,glbVector2> vecAnim = delegate( glbVector2 nv, glbVector2 bv )
			{
				nv.Sub( bv );
				nv.Scale( fRate );
				nv.Add( bv );
				return nv;
			};
			seStateGridPoint b = (seStateGridPoint)before;
			seStateGridPoint n = (seStateGridPoint)next;
			this.Value = vecAnim( n.Value, b.Value );
			this.PowerH = vecAnim( n.PowerH, b.PowerH );
			this.PowerV = vecAnim( n.PowerV, b.PowerV );
		}

		public void Load(IseStateInput input)
		{
			this.Value	= seIOUtil.ReadVector2( input );
			this.PowerH	= seIOUtil.ReadVector2( input );
			this.PowerV	= seIOUtil.ReadVector2( input );
		}
	}


	/**
	 * Color用クラス
	 * 演算結果が255を超える場合は255にまるめる
	 * 演算結果が0以下の場合は0にまるめる
	 */
	public partial class seStateColor : IseStateType
	{
		public glbColor Color { get; set; }


		public seStateColor() : this( glbColor.White ){}
		public seStateColor( glbColor sColor )
		{
			this.Color = sColor;
		}
		public seStateColor( byte bR, byte bG, byte bB, byte bA )
		{
			this.Color = new glbColor( bR, bG, bB, bA );
		}

		public void Load ( IseStateInput inInput )
		{
			this.Color = seIOUtil.ReadColor( inInput );
		}
	
	
		// クローン
		public IseStateType Clone ()
		{
			return new seStateColor( this.Color );
		}
		
		// 白定義
		internal static readonly seStateColor White = new seStateColor( 255,255,255,255 );

		public void Set(IseStateType inInState)
		{
			this.Color = ((seStateColor)inInState).Color;
		}

		public void Add(IseStateType state) { this.Set(state);}


		public void Anim(IseStateType next, IseStateType before, float fRate)
		{
			glbFunc<byte,byte,byte> ColorParam = delegate( byte bNext, byte bBefor )
			{
				int iColor = (int)((float)(bNext - bBefor) * fRate) + bBefor;
				if ( iColor < 0 ) return 0;
				if ( iColor > 255 ) return 255;
				return (byte)iColor;
			};

			glbColor c = ((seStateColor)before).Color;
			glbColor nc = ((seStateColor)next).Color;
			c.R = ColorParam( nc.R, c.R );
			c.G = ColorParam( nc.G, c.G );
			c.B = ColorParam( nc.B, c.B );
			c.A = ColorParam( nc.A, c.A );
			this.Color = c;
		}
	}


	// パーティクル再生制御
	public enum ParticlesPlay
	{
		Non,		// 未設定
		Play,		// 開始
		PlayOne,	// １回だけ開始
		Stop,		// 停止
	}

	// 出力タイプ
	public enum ParticlesOutType
	{
		Non,	// 未設定
		Point,	// 点
		Plane,	// 面
	}

	// パーティクルスケールタイプ
	public enum ParticlesSuccession
	{
		Non,		// 未設定(直前の状態を継承)
		Use,		// 親の値を使用
		NoUse,		// 親の値を使用しない
	}

	// パーティクル回転タイプ
	public enum ParticlesRot
	{
		Non,		// 未設定(直前の状態を継承)
		Normal,		// 通常
		Succession,	// 親の値を使用
		Direction,	// 移動方向
	}

	// 描画順位
	public enum ParticlesDrawPriority
	{
		Non,		// 未設定(直前の状態を継承)
		Top,		// パーティクルは手前に置かれます。
		Interior,	// パーティクルは奥に置かれます。
	}


	// パーティクル出力エレメント
	public partial class seStateParticles : IseStateType
	{
		public float OutCount { get; set; }		// １フレームの出力数
		public int StartFrame { get; set; }		// スタートフレーム
		public ParticlesPlay Play { get; set; }	// 発射開始・停止
		public ParticlesDrawPriority Priority { get; set; }	// パーティクルの配置位置
		public ParticlesSuccession SuccessionScale { get; set; }// 親のスケールを継承するか否か
		public ParticlesRot SuccessionRot { get; set; }// 親の回転を継承するか否か
		public ParticlesOutType OutType { get; set; }	// 出力タイプ
		public glbVector2 OutVector { get; set; }	// 出力方向及びパワー
		public float OutScaleLenge { get; set; }	// 出力範囲(パワースケール)
		public float OutLenge { get; set; }	// 出力範囲(角度)
		public glbVector2 InfluenceVector { get; set; } // 影響方向及びパワー
		public float InfluenceScaleLenge { get; set; }	// 影響範囲(パワースケール)
		public float InfluenceLenge { get; set; } // 影響方向範囲


		// 既に出力したパーティクルに対しての途中制御は
		// 影響点や出力点を変更することで可能ですが、そのようなインタフェースを
		// 用意するかは需要次第です。
		// 

		public seStateParticles()
		{
			this.OutCount = 1.0f;
			this.StartFrame = 0;
			this.Play = ParticlesPlay.Play;
			this.Priority = ParticlesDrawPriority.Top;
			this.SuccessionRot = ParticlesRot.Normal;
			this.SuccessionScale = ParticlesSuccession.NoUse;
			this.OutType = ParticlesOutType.Plane;
			this.OutVector = glbVector2.Zero;
			this.OutScaleLenge = 1.0f;
			this.OutLenge = 0.0f;
			this.InfluenceVector = glbVector2.Zero;
			this.InfluenceScaleLenge = 1.0f;
			this.InfluenceLenge = 0.0f;
		}
		
		public seStateParticles( seStateParticles mst )
		{
			this.Set(mst);
		}

		public void Add(IseStateType state)
		{
			seStateParticles mst = (seStateParticles)state;
			this.OutCount = mst.OutCount;
			this.StartFrame = mst.StartFrame;
			if (mst.Play != ParticlesPlay.Non) this.Play = mst.Play;
			if (mst.Priority != ParticlesDrawPriority.Non) this.Priority = mst.Priority;
			if (mst.SuccessionRot != ParticlesRot.Non) this.SuccessionRot = mst.SuccessionRot;
			if (mst.SuccessionScale != ParticlesSuccession.Non) this.SuccessionScale = mst.SuccessionScale;
			this.OutCount = mst.OutCount;
			if (mst.OutType != ParticlesOutType.Non) this.OutType = mst.OutType;
			this.OutVector = mst.OutVector;
			this.OutScaleLenge = mst.OutScaleLenge;
			this.OutLenge = mst.OutLenge;
			this.InfluenceVector = mst.InfluenceVector;
			this.InfluenceScaleLenge = mst.InfluenceScaleLenge;
			this.InfluenceLenge = mst.InfluenceLenge;
		}

		public void Set(IseStateType state)
		{
			seStateParticles mst = (seStateParticles)state;
			this.OutCount = mst.OutCount;
			this.StartFrame = mst.StartFrame;
			this.Play = mst.Play;
			this.Priority = mst.Priority;
			this.SuccessionRot = mst.SuccessionRot;
			this.SuccessionScale = mst.SuccessionScale;
			this.OutCount = mst.OutCount;
			this.OutType = mst.OutType;
			this.OutVector = mst.OutVector;
			this.OutScaleLenge = mst.OutScaleLenge;
			this.OutLenge = mst.OutLenge;
			this.InfluenceVector = mst.InfluenceVector;
			this.InfluenceScaleLenge = mst.InfluenceScaleLenge;
			this.InfluenceLenge = mst.InfluenceLenge;
		}


		// アニメーションなし
		public void Anim(IseStateType next, IseStateType before, float fRate){}


		public void Load(IseStateInput input)
		{
			this.OutCount = input.ReadFloat();
			this.StartFrame = input.ReadInt();
			this.Play = input.ReadEnum<ParticlesPlay>();
			this.Priority = input.ReadEnum<ParticlesDrawPriority>();
			this.SuccessionRot = input.ReadEnum<ParticlesRot>();
			this.SuccessionScale = input.ReadEnum<ParticlesSuccession>();
			this.OutType = input.ReadEnum<ParticlesOutType>();
			this.OutVector = seIOUtil.ReadVector2(input);
			this.OutScaleLenge = input.ReadFloat();
			this.OutLenge = input.ReadFloat();
			this.InfluenceVector = seIOUtil.ReadVector2(input);
			this.InfluenceScaleLenge = input.ReadFloat();
			this.InfluenceLenge = input.ReadFloat();
		}

		public IseStateType Clone()
		{
			return new seStateParticles(this);
		}

	}


	// パーティクルオブジェクトステータス
	// オブジェクトでのみ使用
	public partial class seStateParticlesInfo : IseStateType
	{
		// アニメーション付きステータス
		public partial class State<T>
			where T : struct
		{
			public T Start { get; set; }	// 開始時
			public T End { get; set; }		// 終了時
			public List<FePointInfo> Anim { get; private set; }

#if PL_EDITOR
			public void Set( State<T> mst )
			{
				this.Start = mst.Start;
				this.End = mst.End;
				// アニメーションコピー
				this.Anim.Clear();
				foreach (var anim in mst.Anim)
				{
					this.Anim.Add(anim.Clone());
				}
			}
#endif

			// アニメーションロード
			public void LoadAnim(IseStateInput input)
			{
				this.Anim.Clear();
				int iCount = input.ReadInt();
				for (int i = 0; i < iCount; i++)
				{
					FePointInfo pi = new FePointInfo();
					pi.Load(input);
					this.Anim.Add(pi);
				}
			}


			public State()
			{
				FePointInfo endPoint = new FePointInfo(1.0f,1.0f);
				endPoint.CreateAnimFunc(FeAnimType.Liner);
				this.Anim = new List<FePointInfo>()
				{
					new FePointInfo(0.0f,0.0f),
					endPoint
				};
			}
		}

		// パーティクル１つに対するアニメーション
		public State<glbColor> Color { get; private set; }
		public State<float> Rot { get; private set; }
		public float RotLenge { get; set; }	// 角度幅
		public State<glbVector2> Scale { get; private set; }
		public float ScaleLenge { get; set; } // スケール幅
		public State<glbVector2> Center { get; private set; }
		public int LifeMin { get; set; }	// 最短寿命
		public int LifeMax { get; set; }	// 最長寿命

		public int MaxNum { get; set; }					// 最大描画数

		public seStateParticlesInfo()
		{
			this.Color = new State<glbColor>();
			this.Rot = new State<float>();
			this.RotLenge = 0.0f;
			this.Scale = new State<glbVector2>();
			this.ScaleLenge = 1.0f;
			this.Center = new State<glbVector2>();

			this.Color.Start = glbColor.White;
			this.Color.End = glbColor.White;
			this.Rot.Start = 0.0f;
			this.Rot.End = 0.0f;
			this.Scale.Start = glbVector2.One;
			this.Scale.End = glbVector2.One;
			this.Center.Start = glbVector2.Zero;
			this.Center.End = glbVector2.Zero;
			this.LifeMin = 10;
			this.LifeMax = 13;
			this.MaxNum = 100;
		}
		public seStateParticlesInfo( seStateParticlesInfo mst ) : this()
		{
			this.Set(mst);
		}

		public IseStateType Clone () { return new seStateParticlesInfo( this ); }
	
		public void Set(IseStateType state)
		{
			// オブジェクト専用ステータスの場合、Setは編集時のみ
			// 使用します。
#if PL_EDITOR
			seStateParticlesInfo mst = (seStateParticlesInfo)state;
			// パーティクル１つに対するアニメーション
			this.Color.Set(mst.Color);
			this.Rot.Set(mst.Rot);
			this.RotLenge = mst.RotLenge;
			this.Scale.Set(mst.Scale);
			this.ScaleLenge = mst.ScaleLenge;
			this.Center.Set(mst.Center);
			this.LifeMin = mst.LifeMin;
			this.LifeMax = mst.LifeMax;
			this.MaxNum = mst.MaxNum;
#endif
		}

		public void Add(IseStateType state){ this.Set(state); }

		// アニメーションなし
		public void Anim(IseStateType next, IseStateType before, float fRate){}


		public void Load(IseStateInput input)
		{
			this.Color.Start = seIOUtil.ReadColor(input);
			this.Color.End = seIOUtil.ReadColor(input);
			this.Rot.Start = input.ReadFloat();
			this.Rot.End = input.ReadFloat();
			this.RotLenge = input.ReadFloat();
			this.Scale.Start = seIOUtil.ReadVector2(input);
			this.Scale.End = seIOUtil.ReadVector2(input);
			this.ScaleLenge = input.ReadFloat();
			this.Center.Start = seIOUtil.ReadVector2(input);
			this.Center.End = seIOUtil.ReadVector2(input);
			this.LifeMin = input.ReadInt();
			this.LifeMax = input.ReadInt();
			// 出力時設定
			this.MaxNum = input.ReadInt();

			this.Color.LoadAnim(input);
			this.Rot.LoadAnim(input);
			this.Scale.LoadAnim(input);
			this.Center.LoadAnim(input);
		}
	}




}
