﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using glb;

namespace evc
{
	/**
	 * カラー描画時に必要なワーク
	 */
	internal partial class seoColorSpriteWork : seObjectWork
	{
		private seoColorSprite _obj;
		private seElem _elem;

		// オブジェクト取得
		public override seObject Object { get { return this._obj; } }

		// オブジェクトのサイズを取得
		public override glbVector2 GetSize ()
		{
			return this._obj.Size;
		}

		// コンストラクタ
		internal seoColorSpriteWork ( seElem elem, seoColorSprite obj )
		{
			this._elem = elem;
			this._obj = obj;
			// ステータスの拡張データをクリア
			this._elem.State.ClearStateData();
		}

		// 描画	
		internal override void Draw ()
		{
			this._obj.Draw( this._elem.ElemID,
							this._elem.NowMatrix,
							this._elem.State.NowColor );
		}

		// 更新
		internal override void Update ( float fAddTime ){}
	}

	/**
	 * カラー描画
	 */
	public partial class seoColorSprite : seObject
	{
		private siObjectColorSprite _sprite = null;

		// サイズ
		private glbVector2 _size = new glbVector2( 10.0f, 10.0f );
		public glbVector2 Size 
		{ 
			get { return this._size; }
			set 
			{
				this._size = value;
				this._sprite.UpdateSize( this._size );
			}
		}

		// カラー
		public glbColor Color { get; set; }

		// 描画オブジェクトタイプ
		public override emObjectType ElemType { get { return emObjectType.Color; } }


		// コンストラクタ
		public seoColorSprite()
		{
			this.Color = glbColor.White;
		}


		// 初期化
		internal override void Init(int objID, seSceneManager scene)
		{
			base.Init(objID, scene);
			// 新しいIDが割り当てられたので作りなおす
			if ( this._sprite != null )
			{
				this._sprite.Dispose();
			}
			this._sprite = seRootScene.Instance.IOManager.CreateColorSprite( this.ObjectID );
			this._sprite.UpdateSize( this._size );
		}


		// 描画時に必要なワーククラスを作成
		public override seObjectWork CreateDrawWork ( seElem inElem )
		{
			return new seoColorSpriteWork( inElem, this );
		}

		// オブジェクトに色をかけて表示
		public void Draw( int iElemID, IGLMatrix m, glbColor color )
		{
			color.Mul( this.Color );
			this._sprite.Draw( iElemID, m, color );
		}

		internal override void Load(IseStateInput input, seSceneManager manager)
		{
			base.Load(input, manager);
			this._size = seIOUtil.ReadVector2(input);
			this.Color = seIOUtil.ReadColor(input);
		}

	}
}
