﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.Diagnostics;
using glb;


namespace evc
{
	// フォントスタイル
	[Flags]
	public enum seFontStyle
	{
		Regular = 0,
		Bold	= 1,
		Italic	= 2,
	}

	// 特殊キータイプ
	public enum SpKeyType
	{
		Non = 0,		// 特殊キーではない
		Variable,		// "\\v[",	// 変数
		NextPage,		//	"\\n",	// 改ページ
		KeyWord,		//	"\\k[",	// キーワード
		Speed,			//	"\\s[",	// スピード
		Wait,			//	待機時間
		Anim,			// アニメーション呼び出し
		
	}

	// 引数タイプ
	public enum SpKeyInput
	{
		Non = 0,	// 引数なし
		Float,		// float
		String,		// string
	};

	// セーブ用フォント文字情報
	public enum SaveFontInfo
	{
		State = 0,	// スタイル
		Color,		// カラー
		Text,		// テキスト
		SpKey,		// 特殊キー
	};


	// 文字情報ベース
	public abstract partial class seFontTextBase
	{
		// 描画イメージ作成
		internal abstract void CreateImage( seFontData inFontData );

		// バイナリーロード
		internal abstract void Load( IseStateInput inInput );
	}

	// ステータス
	public partial class seFontTextState : seFontTextBase
	{
		private int iFontStyleID;

		public seFontStyle Style	{ get; set; }
		public float	   Height	{ get; set; }		// フォントの高さ取得・設定
		public string	   FontName { get; set; }		// フォント名の取得・設定


		// 描画イメージ作成
		internal override void CreateImage( seFontData inFontData )
		{
			this.iFontStyleID = inFontData.FontIO.CreateFontStyle( 
									this.FontName, this.Height, this.Style );
			inFontData.Style = this.iFontStyleID;
			inFontData.Height = this.Height;
		}

		// ロード
		internal override void Load(IseStateInput inInput)
		{
			this.FontName = inInput.ReadString();
			this.Height = inInput.ReadFloat();
			this.Style = (seFontStyle)inInput.ReadByte();
		}
	}

	// カラーデータ
	public partial class seFontTextColor : seFontTextBase
	{
		// カラー
		public glbColor Color {get; set; }


		// 描画イメージ作成
		internal override void CreateImage( seFontData inFontData )
		{
			// ベースカラーをかける
#if false
			glbColor c = this.sColor;
			c.Mul( inFontData.BaseColor );
			inFontData.Color = c;
#else
			// ベースカラーは描画時に掛ける
			inFontData.Color = this.Color;
#endif	
		}

		// ロード
		internal override void Load(IseStateInput inInput)
		{
			this.Color = seIOUtil.ReadColor( inInput );
		}

	}

	// 文字データ
	public partial class seFontTextString : seFontTextBase
	{
		// 文字列
		public string Text {get; set; }

		// 描画イメージ作成
		internal override void CreateImage( seFontData inFontData )
		{
			// １行もデータがないなら作成ここまでに設定された高さを
			// 行に反映させる為、ラインはここで作成する必要があります。
			seFontPage inPage = (inFontData.Page.Count == 0 ) ? 
									inFontData.CreateNewPage() : 
									inFontData.GetLastPage();

			if ( inPage.Lines.Count == 0 )
			{
				inPage.Add( new seFontLine( inFontData.Height ) );
			}

			foreach (char cChar in this.Text)
			{
				// 改行コードなら
				if ( cChar == '\n' )
				{
					inPage.Add( new seFontLine( inFontData.Height ) );
				}
				// それ以外は文字として登録
				else
				{
					siObjectFontChar inCharInfo = inFontData.FontIO.CreateChar( inFontData.Style, cChar );
					inFontData.AddChar( inCharInfo, inFontData.Color );
				}
			}
		}

		// ロード
		internal override void Load(IseStateInput inInput)
		{
			this.Text = inInput.ReadString();
		}
	}


	public partial class seFontSpKey : seFontTextBase
	{
		public object	 Tag	{ get; set; }		// キー別データ
		public SpKeyType SpKey	{ get; set; }

		// 描画イメージ作成
		internal override void CreateImage( seFontData inFontData )
		{
			switch (this.SpKey)
			{
				case SpKeyType.Non:
					break;
				case SpKeyType.Variable:
					
					break;
	
				case SpKeyType.NextPage:
					inFontData.CreateNewPage();
					break;
				// キーワード
				case SpKeyType.KeyWord:
					seFontTextString inFontString = new seFontTextString();
					inFontString.Text = inFontData.ParentObject.Scene.KeyWord[ (string)this.Tag ];
					inFontString.CreateImage( inFontData );
					break;
				case SpKeyType.Speed:
					inFontData.AddCharDataEx( new seFontCharExSpeed( (float)this.Tag ) );
					break;
				case SpKeyType.Wait:
					inFontData.AddCharDataEx( new seFontCharExWait( (float)this.Tag ) );
					break;
				case SpKeyType.Anim:
					inFontData.AddCharDataEx( new seFontCharExAnim( (string)this.Tag ) );
					break;
				default:
					break;
			}
		
		}

		// ロード
		internal override void Load(IseStateInput inInput)
		{
			this.SpKey = (SpKeyType)inInput.ReadByte();
			SpKeyInput eSpKeyInput = (SpKeyInput)inInput.ReadByte();

			switch (eSpKeyInput)
			{
				case SpKeyInput.Float:
					this.Tag = inInput.ReadFloat();
					break;
				case SpKeyInput.String:
					this.Tag = inInput.ReadString();
					break;
				default:
					break;
			}
		}

	}



	// メッセージ
	public partial class seFontData : IDisposable
	{
		internal float	  Height	   { get; set; }	// 最後に設定された高さ
		internal int	  Style		   { get; set; }			// フォントステータス
		internal glbColor Color		   { get; set; }			// カラー
		internal siObjectFont FontIO   { get; private set; }
		public glbVector2 Size		   { get; private set; }	// サイズ
		public glbColor	  BaseColor	   { get; set; }			// 文字ベースカラー
		internal seObject ParentObject { get; private set; }	// 親オブジェクト

		// 描画ワーク
		private List<IseFontCharEx> lisCharEx = new List<IseFontCharEx>();	// 最初のデータ

		// 任意のページ取得
		private List<seFontPage> lisPage = new List<seFontPage>();
		public ReadOnlyCollection<seFontPage> Page { get { return this.lisPage.AsReadOnly(); } }

		// フォント情報数取得
		private List<seFontTextBase> lisFontData = new List<seFontTextBase>();
		public ReadOnlyCollection<seFontTextBase> Font { get { return this.lisFontData.AsReadOnly(); } }


		// コンストラクタ
		public seFontData ( seObject inParent )
		{
			this.ParentObject = inParent;
			this.FontIO = seRootScene.Instance.IOManager.CreateFont( 0 /* とりあえず */ );
		}

		// 登録関係

		// 最後の行を取得
		internal seFontPage GetLastPage()
		{
			Debug.Assert( this.lisPage.Count > 0 );
			return this.lisPage[ this.lisPage.Count - 1 ];
		}
		// 最後のページの最後の行にデータを追加
		internal void AddChar( siObjectFontChar inCharInfo, glbColor sColor )
		{
			seFontChar inChar = new seFontChar( inCharInfo, sColor );
			// 拡張データを登録
			this.lisCharEx.ForEach( delegate( IseFontCharEx inDataEx )
			{
				inChar.AddData( inDataEx );
			});
			this.lisCharEx.Clear();
			
			this.GetLastPage().GetLastLine().Add( inChar );
		}
		// 文字拡張データを貯める
		internal void AddCharDataEx( IseFontCharEx inDataEx )
		{
			this.lisCharEx.Add( inDataEx );
		}
		
		
		// 新しい行を作成
		internal seFontPage CreateNewPage()
		{
			seFontPage page = new seFontPage();
			this.lisPage.Add( page );
			return page;
		}

		// 内部データクリア
		public void Clear()
		{
			this.lisFontData.Clear();
			this.lisPage.Clear();
			
			// 内部データ消す。
			this.FontIO.Clear();
		}

		// ステータス追加
		public void Add( seFontTextBase inData )
		{
			this.lisFontData.Add( inData );
		}


		// 描画クラス内部データ作成
		public void CreateImageText()
		{
			this.FontIO.Clear();
			this.lisPage.Clear();
			this.lisCharEx.Clear();


			// 全データを登録
			this.FontIO.BeginCharCreating();
			
			this.lisFontData.ForEach( delegate( seFontTextBase inData )
			{
				inData.CreateImage( this );
			});
			
			// 更新を通知
			this.FontIO.EndCharCreating();

			// サイズを再設定
			this.Size = glbVector2.Zero;
			
			this.lisPage.ForEach( delegate( seFontPage inPage )
			{
				inPage.UpdateSize();
				this.Size = new glbVector2( Math.Max( this.Size.X, inPage.Size.X ),
											Math.Max( this.Size.Y, inPage.Size.Y ) );
			});
		}

		// 任意のページの任意の文字数目の情報を取得
		internal seFontChar GetCharInfo( int iPage, int iTextNum )
		{
			if ( this.lisPage.Count == 0 ) return null;

			// リミットチェック
			if ( iPage < 0 ) iPage = 0;
			if ( iPage >= this.lisPage.Count ) iPage = this.lisPage.Count - 1;
			
			return this.lisPage[ iPage ].GetCharInfo( iTextNum );
		}


		// 描画
		public void Draw( int iElemID, IGLMatrix mParent, glbColor sColor, int iPage, int iDrawNum )
		{
			if ( this.lisPage.Count == 0 ) return;

			// リミットチェック
			if ( iPage < 0 ) iPage = 0;
			if ( iPage >= this.lisPage.Count ) iPage = this.lisPage.Count - 1;

			// 任意のページを描画
			this.lisPage[ iPage ].Draw( iElemID, mParent, sColor, iDrawNum );
		}

		// 解放
		public void Dispose ()
		{
			this.lisFontData.Clear();
			this.FontIO.Dispose();
		}


		// ロード
		internal void Load( IseStateInput inInput )
		{
			this.Clear();

			this.BaseColor = seIOUtil.ReadColor( inInput );
			this.Size = seIOUtil.ReadVector2( inInput );


			int iDataCount = inInput.ReadShort();
			for (int i = 0; i < iDataCount; i++)
			{
				SaveFontInfo eFontInfoType = (SaveFontInfo)inInput.ReadByte();
				seFontTextBase inFontText = null;
				switch (eFontInfoType)
				{
					case SaveFontInfo.State: inFontText = new seFontTextState(); break;
					case SaveFontInfo.Color: inFontText = new seFontTextColor(); break;
					case SaveFontInfo.Text: inFontText = new seFontTextString(); break;
					case SaveFontInfo.SpKey: inFontText = new seFontSpKey(); break;
					default: Debug.Assert( false ); break;
				}
				inFontText.Load( inInput );
				this.Add( inFontText );
			}
			this.CreateImageText();
		}
	}



}
