﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using glb;

namespace evc
{
	// １行管理
	internal class seFontLine
	{
		private List<seFontChar>	lisFontChar = new List<seFontChar>();
		public ReadOnlyCollection<seFontChar> Chars { get { return this.lisFontChar.AsReadOnly(); } }

				// 一行のサイズを取得
		internal glbVector2 Size	{ get; private set; }


		// コンストラクタ
		internal seFontLine ( float fHeight )
		{
			// 最低高さを設定
			this.Size = new glbVector2( 0.0f, fHeight );
		}


		// 最後のキャラを取得
		internal seFontChar GetLastChar()
		{
			if ( this.lisFontChar.Count <= 0 ) return null;
			return this.lisFontChar[ this.lisFontChar.Count - 1 ];
		}



		// データの反映
		internal void Apply()
		{
			// サイズの計算
			this.lisFontChar.ForEach( delegate( seFontChar fontChar )
			{
				glbVector2 size = fontChar.FontChar.Size;
				size.X += this.Size.X;
				size.Y = Math.Max(this.Size.Y, size.Y);
				this.Size = size;
			});
		}

		internal void Add( seFontChar inChar )
		{
			this.lisFontChar.Add( inChar );
		}



		// 描画
		internal bool Draw( int iElemID, float fHeight, IGLMatrix mParent, glbColor sColor, int iDrawNum, ref int iNowCount )
		{
			// 表示下部位置
			float fUnderY = fHeight + (float)this.Size.Y;
			
			bool bRet = true;
			int iLocalCount = iNowCount;
			seMatrixWork.Instance.UseMatrixWork( delegate( IUseWorkMatrix gw )
			{
				IGLMatrix mPos = gw.GetMatrix();
				float fPosX = 0.0f;

				foreach (seFontChar inFontChar　in this.lisFontChar)
				{
					glbVector2 sCharSize = inFontChar.FontChar.Size;
					mPos.SetPos( fPosX, fUnderY - (float)sCharSize.Y );
					mPos.MulR( mParent );
					inFontChar.Draw( iElemID, mPos, sColor );
					
					fPosX += (float)sCharSize.X;
					
					iLocalCount ++;

					if ( iDrawNum == seoFont.ALL_TEXT ) continue;
					if ( iLocalCount >= iDrawNum )
					{
						bRet = false;
						return;
					}
				}
			});
			iNowCount = iLocalCount;
			return bRet;
		}
		

	}
}
