﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using glb;

namespace evc
{
	// フォント描画用ワーク
	internal class seFontDrawInfo
	{
		internal int DrawNum { get; set; }	// これから表示する文字数(小数点切捨て)
		internal int Page	 { get; set; }
		internal float Speed { get; set; }	// 文字描画速度
		internal float Wait { get; set; }	// ウエイト時間
		internal float Time { get; set; }	// 更新予定時間

		// 描画対象シーン
		private seSceneManager _sceneManager;
		internal seSceneManager SceneManager
		{
			get { return this._sceneManager; }
			set 
			{
				this._sceneManager = value; 
				if (this._sceneManager != null)
				{
					this.Speed = this._sceneManager.Option.FrameTime;
				}
			} 
		}


		internal void Clear()
		{
			this.DrawNum = 0;
			if (this._sceneManager != null)
			{
				this.Speed = this._sceneManager.Option.FrameTime;
			}
			else
			{
				this.Speed = 0.0f;
			}
			this.Wait = 0.0f;
			this.Time = 0.0f;
		}

		// 次のページにいったときの処理
		internal void NextPage()
		{
			this.DrawNum = 0;
		}

		public seFontDrawInfo ()
		{
			this.Clear();
		}
	}


	// 文字矩形管理
	internal partial class seoWorkFont : seObjectWork
	{
		private int _iLastPage = -1;		// 最後に表示したページ

		private seFontDrawInfo	_drawInfo;
		private seElem _elem;
		private seoFont _obj;

		// 拡張ステータス
		private seStateTextControl _text;

		public override seObject Object { get { return this._obj; } }

		// コンストラクタ
		internal seoWorkFont ( seElem elem, seoFont objFont )
		{
			this._elem = elem;
			this._obj = objFont;

			this._drawInfo = new seFontDrawInfo();
			this._drawInfo.SceneManager = objFont.Scene;

			this._text = (seStateTextControl)seStateUtil.Instance.CreateStateType(emElemState.TextControl);

			// ステータスの拡張データをクリア
			this._elem.State.ClearStateData();
			this._elem.State.LinkStateData(emElemState.TextControl, this._text);
		}


		// オブジェクトのサイズを取得
		public override glbVector2 GetSize ()
		{
			return this._obj.FontData.Size;
		}

		// 描画	
		internal override void Draw ()
		{
			this._obj.Draw(this._elem.ElemID,
							this._elem.NowMatrix, 
							this._elem.State.NowColor,
							this._drawInfo.Page,
							this._drawInfo.DrawNum);
		}

		internal override void Reset ()
		{
			// リセット文字スピードはどうするか。。
			this._iLastPage = -1;
			this._drawInfo.Clear();
		}

		// 更新
		internal override void Update(float fAddTime)
		{
			seStateData data = this._elem.State.GetData(emElemState.TextControl);
			seFontData fontData = this._obj.FontData;

			// 新しい状態になったなら
			if ((data != null) && (data.MoveKey))
			{
				this._iLastPage = this._text.Page;
				this._drawInfo.NextPage();

				// 改ページ後は最初の１文字だけ特殊文字チェックをする
				seFontChar charInfo = fontData.GetCharInfo(this._iLastPage, 0);
				if (charInfo == null) return;
				// 状態更新
				charInfo.UpdateInfo(this._drawInfo);
			}

			// 編集時は全てのテキストを描画
			if (seRootScene.Instance.EditMode)
			{
				this._drawInfo.DrawNum = seoFont.ALL_TEXT;
				return;
			}

			// 更新予定時間を追加
			this._drawInfo.Time += fAddTime;
			// 追加経過時間から表示文字数を算出
			while ( true )
			{
				// 待機中なら待機時間を引く
				if (this._drawInfo.Wait > 0.0f)
				{
					// 待機時間の方が長いならこれ以上表示しない
					if (this._drawInfo.Wait >= this._drawInfo.Time)
					{
						this._drawInfo.Wait -= this._drawInfo.Time;
						this._drawInfo.Time = 0.0f;
						break;
					}
					// 待機時間の方が短いなら待機終了
					else
					{
						this._drawInfo.Time -= this._drawInfo.Wait;
						this._drawInfo.Wait = 0.0f;
					}
				}

				// １文字表示分に満たないなら終了
				if (this._drawInfo.Time < this._drawInfo.Speed) break;

				// １文字分、時間を引いて次へ
				this._drawInfo.Time -= this._drawInfo.Speed;

				this._drawInfo.DrawNum ++;
				// 次の文字を取得
				seFontChar charInfo = fontData.GetCharInfo(this._iLastPage, this._drawInfo.DrawNum);
				// 文字データが無いなら終了
				if (charInfo == null) break;

				// 状態更新
				charInfo.UpdateInfo(this._drawInfo );
			}

		}
	}




	// カラー描画
	public partial class seoFont : seObject
	{
		public const int ALL_TEXT = -1;		// 全ての文字を表示

		public siObjectColorSprite BgSprite { get; private set; }	// 背景スプライトを取得
		public glbColor			   Color	{ get; set; }
		public seFontData		   FontData { get; set; }			// オブジェクトが管理しているフォントを取得


		// 描画オブジェクトタイプ
		public override emObjectType ElemType { get { return emObjectType.Font; } }
	
		// コンストラクタ
		public seoFont()
		{
			this.Color = glbColor.White;
			this.FontData = new seFontData( this );
		}

		// 初期化
		internal override void Init(int iObjectID, seSceneManager scene)
		{
			base.Init(iObjectID, scene);
			// 新しいIDが割り当てられたので作りなおす
			if ( this.BgSprite != null )
			{
				this.BgSprite.Dispose();
			}
			this.BgSprite = seRootScene.Instance.IOManager.CreateColorSprite( this.ObjectID );
			this.BgSprite.UpdateSize( glbVector2.One );
		}

		// 描画時に必要なワーククラスを作成
		public override seObjectWork CreateDrawWork(seElem elem)
		{
			return new seoWorkFont(elem, this);
		}

	
		// 表示
		public void Draw(int iElemID, IGLMatrix mParent, glbColor baseColor, int iPage, int iDrawNum)
		{
			seFontData fondData = this.FontData;
			if (fondData != null)
			{
				glbColor bgColor = baseColor;
				bgColor.Mul(this.Color);

				seMatrixWork.Instance.UseMatrixWork(delegate(IUseWorkMatrix gw)
				{
//					glbVector2 size = fondData.Size;
					
					// 背景描画
					IGLMatrix m = gw.GetMatrix();
					m.SetScale(fondData.Size);
					m.MulR(mParent);
					this.BgSprite.Draw(iElemID, m, bgColor);

					// フォントを描画
					fondData.BaseColor = baseColor;
					// とりあえず全文字
					fondData.Draw(iElemID, mParent, baseColor, iPage, iDrawNum);
				});
			}
		}

		internal override void Load(IseStateInput input, seSceneManager manager)
		{
			base.Load(input, manager);
			this.Color = seIOUtil.ReadColor(input);
			this.FontData.Load(input);
		}


		// 解放処理
		internal override void Dispose ()
		{
			// サンプル描画を解放
			if ( this.BgSprite != null )
			{
				this.BgSprite.Dispose();
				this.BgSprite = null;
			}
			if ( this.FontData != null )
			{
				this.FontData.Dispose();
				this.FontData = null;
			}
		}
	}


}
