﻿using System;
using System.Diagnostics;
using glb;


namespace evc
{
	// テクスチャースプライト描画時に必要なワーク
	internal partial class seoFrameWork : seObjectWork
	{
		private seoFrame _frameObj; // 描画オブジェクト
		private seElem _elem; // エレメント
		private seStateFloat2 _uv; // UV

		public override seObject Object { get { return this._frameObj; } }

		// オブジェクトのサイズを取得
		public override glbVector2 GetSize ()
		{
			return this._frameObj.Size;
		}

		// コンストラクタ
		internal seoFrameWork ( seElem elem, seoFrame obj )
		{
			this._uv = (seStateFloat2)seStateUtil.Instance.CreateStateType(emElemState.UV);
			this._elem = elem;
			this._frameObj = obj;

			// ステータスの拡張データをクリア
			this._elem.State.ClearStateData();
			this._elem.State.LinkStateData( emElemState.UV, this._uv );
		}

		// 描画	
		internal override void Draw ()
		{
			this._frameObj.Draw( this._elem.ElemID, 
								this._elem.NowMatrix, 
								this._elem.State.NowColor,
								this._elem.State.Scale.Value,
								this._uv.Value );
		}

		// 更新
		internal override void Update ( float fAddTime ){}
	}


	// テクスチャ描画
	public partial class seoFrame : seObject, IseObjResourceTexture
	{
		// １パーツ描画
		private class FrameParts
		{
			public bool Visible { get; set; }
			internal siObjectTextureSprite Sprite { get; set; }	// 描画スプライト

			// コンストラクタ
			internal FrameParts( int iObjectID )
			{
				this.Visible = true;
				this.Sprite = seRootScene.Instance.IOManager.CreateTextureSprite( iObjectID );
			}

			// 終了処理
			internal void Dispose()
			{
				if ( this.Sprite != null )
				{
					this.Sprite.Dispose();
					this.Sprite = null;
				}
			}
		}


		// パーツは最大９分割される
		FrameParts[] ainImages = new FrameParts[ 9 ];

		// 更新フラグ
		private bool bUpdateSprite = false;

		// サイズ
		private glbVector2 sSize = new glbVector2( 10.0f, 10.0f );
		public glbVector2 Size 
		{ 
			get { return this.sSize; }
			set 
			{
				this.sSize = value;
				this.bUpdateSprite = true;
			}
		}

		// UV
		private glbVector2 sUV = glbVector2.Zero;
		public glbVector2 UV
		{
			get { return this.sUV ; }
			set 
			{
				this.sUV = value;
				this.bUpdateSprite = true;
			}
		}

		// 伸張枠の左上の位置
		public glbVector2 sExpansionStart = glbVector2.Zero;
		public glbVector2 ExpendStart 
		{ 
			get { return this.sExpansionStart; }
			set 
			{
				this.sExpansionStart = value;
				this.bUpdateSprite = true;
			}
		}

		// 伸張枠の右下の位置
		public glbVector2 sExpansionEnd = glbVector2.Zero;
		public glbVector2 ExpendEnd 
		{ 
			get { return this.sExpansionEnd; }
			set 
			{
				this.sExpansionEnd = value;
				this.bUpdateSprite = true;
			}
		}

		// コンストラクタ
		public seoFrame()
		{
		}



		// データ更新
		private void Apply()
		{
			if ( !this.bUpdateSprite ) return;

			// 伸張矩形とオブジェクトは必ず交わっているものとする。
			// (完全に交わっていないか判定いれる？)

			glbVector2 sImageSize = new glbVector2( this.ResourceTexture.TextureSize );

			glbVector2 es = new glbVector2( Math.Max( 0.0f, this.sExpansionStart.X ),
											Math.Max( 0.0f, this.sExpansionStart.Y ) );
			glbVector2 ee = new glbVector2( Math.Min( this.Size.X, this.sExpansionEnd.X ),
											Math.Min( this.Size.Y, this.sExpansionEnd.Y ) );


			// UVリスト
			glbVector2[] asPartsUV = new glbVector2[3];
			asPartsUV[ 0 ] = this.sUV;
			asPartsUV[ 1 ] = es;
			asPartsUV[ 1 ].Div( sImageSize );
			asPartsUV[ 1 ].Add( this.sUV );
			asPartsUV[ 2 ] = ee;
			asPartsUV[ 2 ].Div( sImageSize );
			asPartsUV[ 2 ].Add( this.sUV );

			// 幅リスト
			glbVector2[] asPartsSize = new glbVector2[3];
			asPartsSize[ 0 ] = es;
			asPartsSize[ 1 ] = ee;
			asPartsSize[ 1 ].Sub( es );
			asPartsSize[ 2 ] = this.Size;
			asPartsSize[ 2 ].Sub( ee );

			int iCount = 0;
			for (int y = 0; y < 3; y++)
			{
				for (int x = 0; x < 3; x++)
				{
					FrameParts inParts = this.ainImages[ iCount ];
					// 面積が0以下の場合は非表示
					if ( (asPartsSize[ x ].X * asPartsSize[ y ].Y) <= 0.0f )
					{
						inParts.Visible = false;
					}
					else
					{
						inParts.Visible = true;
						inParts.Sprite.UpdateTexture( this.ResourceTexture );
						inParts.Sprite.UpdateSize( new glbVector2( asPartsSize[ x ].X, asPartsSize[ y ].Y ) );
						inParts.Sprite.UpdateUV( new glbVector2( asPartsUV[ x ].X, asPartsUV[ y ].Y) );
					}
					iCount ++;
				}
			}

			this.bUpdateSprite = false;
		}



		// リソースID
		private int iResourceID = seResourceManager.InvalidID;
		public int ResourceID
		{
			get { return this.iResourceID; }
			set
			{
				this.iResourceID = value;
				this.inResourceTexture = null;
				if ( this.iResourceID == seResourceManager.InvalidID )
				{
					this.bUpdateSprite = true;
				}
			}
		}

		// テクスチャリソース取得
		private siResourceTexture inResourceTexture = null;
		public siResourceTexture ResourceTexture 
		{
			get 
			{
				if ( (this.inResourceTexture == null) &&
					 (this.iResourceID != seResourceManager.InvalidID) )
				{
					this.inResourceTexture = this.Scene.ResourceManager.Find(
												this.iResourceID ) as siResourceTexture;

					// 無い場合はリソース無し状態にする
					if ( this.inResourceTexture != null )
					{
						// リソースはあるが準備できていない場合
						if ( !this.inResourceTexture.IsEffective )
						{
							this.inResourceTexture = null;
						}
					}
					this.bUpdateSprite = true;
				}

				return this.inResourceTexture; 
			}
		}

		// カラー
		private glbColor sColor = glbColor.White;
		public glbColor Color { get { return this.sColor; } set { this.sColor = value; } }


		// 初期化
		internal override void Init(int iInObjectID, seSceneManager inInScene)
		{
			base.Init(iInObjectID, inInScene);

			for (int i = 0; i < this.ainImages.Length; i++)
			{
				// 既にあるなら一度消す。
				if ( this.ainImages[ i ] != null )
				{
					this.ainImages[ i ].Dispose();
				}
				this.ainImages[ i ] = new FrameParts( iInObjectID );
			}
			this.bUpdateSprite = true;
		}

		// 描画オブジェクトタイプ
		public override emObjectType ElemType { get { return emObjectType.Frame; } }


		// 描画時に必要なワーククラスを作成
		public override seObjectWork CreateDrawWork ( seElem inElem )
		{
			return new seoFrameWork( inElem, this );
		}


		public void Draw(int iElemID, IGLMatrix mParent, glbColor sMulColor,
										glbVector2 sMstScale, glbVector2 sUV )
		{
			// 描画更新
			this.Apply();

			sMulColor.Mul(this.Color);
			seMatrixWork.Instance.UseMatrixWork( delegate(IUseWorkMatrix gw )
			{
				// テクスチャをフレームバッファへ描画
				IGLMatrix m = gw.GetMatrix();
				IGLMatrix ms = gw.GetMatrix();
				IGLMatrix mUV = gw.GetMatrix();

				mUV.SetUV( sUV );

				glbVector2 es = new glbVector2( Math.Max( 0.0f, this.sExpansionStart.X ),
												Math.Max( 0.0f, this.sExpansionStart.Y ) );
				glbVector2 ee = new glbVector2( Math.Min( this.Size.X, this.sExpansionEnd.X ),
												Math.Min( this.Size.Y, this.sExpansionEnd.Y ) );

				// ローカルサイズ
				glbVector2 l0 = es;
				l0.Div( sMstScale );
				glbVector2 l2 = this.Size;
				l2.Sub( ee );
				l2.Div( sMstScale );

				// 位置
				glbVector2 sLastPos = this.Size;
				sLastPos.Sub( l2 );
				glbVector2[] sPutPos = new glbVector2[]
				{
					glbVector2.Zero, l0, sLastPos,
				};


				// スケール
				glbVector2 s = l0;
				s.Add( l2 );

				glbVector2 pw1 = ee;
				pw1.Sub( es );

				glbVector2 s2 = this.Size;
				s2.Sub( s );
				s2.Div( pw1 );
				glbVector2 s3 = glbVector2.One;
				s3.Div( sMstScale );
				glbVector2[] sPutScale = new glbVector2[] { s3, s2, s3 };


				int iCount = 0;
				glbVector2 sScale = new glbVector2();
				glbVector2 sPos = new glbVector2();
				for (int y = 0; y < 3; y++)
				{
					for (int x = 0; x < 3; x++)
					{
						FrameParts inParts = this.ainImages[ iCount ];
						iCount++;

						// 非表示なら次へ
						if ( !inParts.Visible ) continue;

						sScale.X = sPutScale[ x ].X;
						sScale.Y = sPutScale[ y ].Y;
						ms.SetScale( sScale );

						sPos.X = sPutPos[ x ].X;
						sPos.Y = sPutPos[ y ].Y;
						m.SetPos( sPos );

						m.Mul(ms);
						m.MulR( mParent );

						inParts.Sprite.Draw( iElemID, m, sMulColor, mUV);
					}
				}
			});
		}


		internal override void Dispose()
		{
			base.Dispose();

			Array.ForEach( this.ainImages, delegate( FrameParts inParts )
			{
				inParts.Dispose();
			});
		}


		internal override void Load(IseStateInput inInput, seSceneManager inManager)
		{
			base.Load(inInput, inManager);
			this.sSize = seIOUtil.ReadVector2( inInput );
			this.sUV = seIOUtil.ReadVector2( inInput );
			this.sExpansionStart = seIOUtil.ReadVector2( inInput );
			this.sExpansionEnd = seIOUtil.ReadVector2( inInput );
			this.ResourceID = (int)inInput.ReadShort();
			this.sColor = seIOUtil.ReadColor( inInput );
		}

	}
}
