﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using glb;


namespace evc
{
	// テクスチャースプライト描画時に必要なワーク
	internal partial class seoGridWork : seObjectWork
	{
		private seoGrid _obj; // 描画オブジェクト
		private seElem _elem; // エレメント

		private seStateSwitchInt1 _texture;
		private seStateTextureControl _textureControl;
		private seStateFloat2 _uv;
		private seStateGridPoint[] _gridPoint;
		private IGLMatrix _mUV = null;

		public override seObject Object { get { return this._obj; } }


		// オブジェクトのサイズを取得
		public override glbVector2 GetSize ()
		{
			return this._obj.Size;
		}

		// コンストラクタ
		internal seoGridWork ( seElem elem, seoGrid obj )
		{
			this._elem = elem;
			this._obj = obj;

			this._mUV = seRootScene.Instance.IOManager.CreateMatrix();
			this._mUV.Identity();

			seStateUtil su = seStateUtil.Instance;
			this._texture = (seStateSwitchInt1)su.CreateStateType( emElemState.Texture );
			this._textureControl = (seStateTextureControl)su.CreateStateType( emElemState.TextureControl );
			this._uv = (seStateFloat2)su.CreateStateType( emElemState.UV );
			this._gridPoint = new seStateGridPoint[ 4 ];
			this._gridPoint[ 0 ] = (seStateGridPoint)su.CreateStateType(emElemState.GridPos1);
			this._gridPoint[ 1 ] = (seStateGridPoint)su.CreateStateType(emElemState.GridPos2);
			this._gridPoint[ 2 ] = (seStateGridPoint)su.CreateStateType(emElemState.GridPos3);
			this._gridPoint[ 3 ] = (seStateGridPoint)su.CreateStateType(emElemState.GridPos4);

			// ステータスの拡張データをクリア
			seElemState state = this._elem.State;
			state.ClearStateData();
			state.LinkStateData( emElemState.Texture, this._texture );
			state.LinkStateData( emElemState.TextureControl, this._textureControl );
			state.LinkStateData( emElemState.UV, this._uv );
			state.LinkStateData( emElemState.GridPos1, this._gridPoint[ 0 ] );
			state.LinkStateData( emElemState.GridPos2, this._gridPoint[ 1 ] );
			state.LinkStateData( emElemState.GridPos3, this._gridPoint[ 2 ] );
			state.LinkStateData( emElemState.GridPos4, this._gridPoint[ 3 ] );
		}

		// 内部ワークのリセット
		internal override void Reset()
		{
			// 頂点リストを初期化？
			// 

		}

		internal const float PowerScale = 3.0f;

		// 描画サブルーチン
		// 再生エンジン以外で流量される為Drawと分けてあります。
		private void DrawSub(IGLMatrix mBase, glbColor color)
		{
			// 頂点を計算
			siObjectMesh mesh = this._obj.Mesh;
			if (mesh != null)
			{
				mesh.BeginEditVertex( this._elem.ElemID );

				seStateGridPoint[/*4*/] mstPos = this._gridPoint;
				int partX = this._obj.Partitions.X;
				int partY = this._obj.Partitions.Y;

				// 頂点操作
				int index = 0;
				for (int y = 0; y <= partY; y++)
				{
					float fRate = (float)y / (float)partY;

					// 左右のラインの座標を算出
					glbVector2 posL = seMath.GetHermiteCurvePoint(
									mstPos[0].Value, mstPos[0].PowerV,
									mstPos[3].Value, mstPos[3].PowerV,
									fRate, seoGridWork.PowerScale);
					glbVector2 posR = seMath.GetHermiteCurvePoint(
									mstPos[1].Value, mstPos[1].PowerV,
									mstPos[2].Value, mstPos[2].PowerV,
									fRate, seoGridWork.PowerScale);


					glbVector2 power1 = mstPos[3].PowerH;
					power1.Sub(mstPos[0].PowerH);
					power1.Scale(fRate);
					power1.Add(mstPos[0].PowerH);

					glbVector2 power2 = mstPos[2].PowerH;
					power2.Sub(mstPos[1].PowerH);
					power2.Scale(fRate);
					power2.Add(mstPos[1].PowerH);

					for (int x = 0; x <= partX; x++)
					{
						mesh.SetPos(index, seMath.GetHermiteCurvePoint(
											posL, power1,
											posR, power2,
											(float)x / (float)partX,
											seoGridWork.PowerScale));
						index++;
					}
				}

				mesh.EndEditVertex();
			}

			this._obj.Draw(this._elem.ElemID, mBase, color, this._mUV);
		}


		// 描画
		internal override void Draw()
		{
			this.DrawSub( this._elem.NowMatrix,
						  this._elem.State.NowColor);
		}


		// 更新
		internal override void Update( float fAddTime )
		{
			seStateFloat2 statUV = (seStateFloat2)this._elem.State[emElemState.UV];
			this._mUV.SetUV(statUV.Value);
		}
	}


	// メッシュ描画
	public partial class seoGrid : seObject, IseObjResourceTexture
	{
		internal siObjectMesh Mesh { get; private set; }

		// 描画オブジェクトタイプ
		public override emObjectType ElemType	{ get { return emObjectType.Grid; } }

		public glbVector2 UV { get; set; }			// ベースUV
		public glbVector2I Partitions { get; set; }	// 分割数
		public glbColor Color { get; set; }			// カラー
		public glbVector2 Size { get; set; }		// サイズ
		public glbVector2 UVRate { get; set; }		// UVレート

		private short[] _triIndex;		// トライアングルindex


		// リソースID
		private int iResourceID = seResourceManager.InvalidID;
		public int ResourceID
		{
			get { return this.iResourceID; }
			set
			{
				this.iResourceID = value;
				this._bUpdateImage = true;
			}
		}

		// テクスチャリソース取得
		private siResourceTexture _resTexture = null;
		public siResourceTexture ResourceTexture 
		{
			get 
			{
				this.UpdateImage(seElemManager.InvalidID);
				return this._resTexture;
			} 
		}



		// ResourceIDにあわせてテクスチャの更新
		private bool _bUpdateImage = false;
		private bool UpdateImage(int elemID)
		{
			if ( !this._bUpdateImage ) return false;

			// 無いなら
			if ( this.iResourceID == seResourceManager.InvalidID )
			{
				this._resTexture = null;
			}
			else
			{
				this._resTexture = this.Scene.ResourceManager.Find( this.iResourceID ) as siResourceTexture;

				// リソースはあるが準備できていない場合
				if ((this._resTexture != null) && !this._resTexture.IsEffective)
				{
					// 更新を終了しない
					this._resTexture = null;
					return false;
				}
			}

			// 更新された
			if (this.Mesh != null)
			{
				this.Mesh.UpdateTexture(this._resTexture);
				this.ResetUV(elemID);
			}

			this._bUpdateImage = false;
			return true;
		}


		// コンストラクタ
		public seoGrid()
		{
			this.Color = glbColor.White;
			this.Partitions = new glbVector2I(10,10);
			this.UVRate = glbVector2.Zero;
			this._bUpdateImage = false;
		}

		// 三角形indexリスト作成
		private void ResetTriIndex()
		{
			int itriNum = (this.Partitions.X * this.Partitions.Y * 2) * 3;

			if ( (this._triIndex == null) || (this._triIndex.Length != itriNum) )
			{
				// 三角形index作成
				this._triIndex = new short[ itriNum ];
			}

			// １ラインのオフセット
			int offsetY = this.Partitions.X + 1;
			int index = 0;
			for (int y = 0; y < this.Partitions.Y; y++)
			{
				int iLineY = y * offsetY;
				for (int x = 0; x < this.Partitions.X; x++)
				{
					this._triIndex[index    ] = (short)(iLineY + x);
					this._triIndex[index + 1] = (short)(this._triIndex[index ] + 1);
					this._triIndex[index + 2] = (short)(this._triIndex[index ] + offsetY);
					this._triIndex[index + 3] = this._triIndex[index + 2];
					this._triIndex[index + 4] = this._triIndex[index + 1];
					this._triIndex[index + 5] = (short)(this._triIndex[index + 2] + 1);
					index += 6;
				}
			}
		}

		// UVリストを作成
		private void ResetUV(int elemID)
		{
			siObjectMesh mv = this.Mesh;
			mv.BeginEditVertex(elemID);

			int partX = this.Partitions.X;
			int partY = this.Partitions.Y;

//			glbVector2I ti = this.ResourceTexture.TextureSize;
			glbVector2I ti = this._resTexture.TextureSize;
			glbVector2 texSize = new glbVector2((float)ti.X, (float)ti.Y);

			// 頂点操作
			int index = 0;
			for (int y = 0; y <= partY; y++)
			{
				float fRate = (float)y / (float)partY;

				// 画角補正：Y
				float fY = (float)this.UVRate.Y + 1.0f;
				fRate *= (1.0f - fY) * fRate + fY;


				// 左右のラインの座標を算出
				glbVector2 posL = seMath.GetHermiteCurvePoint(
											new glbVector2(0.0f,0.0f),
											glbVector2.Zero,
											new glbVector2(0.0f,this.Size.Y), 
											glbVector2.Zero,
											fRate, 
											seoGridWork.PowerScale);
				glbVector2 posR = seMath.GetHermiteCurvePoint(
											new glbVector2(this.Size.X,0.0f),
											glbVector2.Zero,
											this.Size, 
											glbVector2.Zero,
											fRate, 
											seoGridWork.PowerScale);


				for (int x = 0; x <= partX; x++)
				{
					float fRateX = (float)x / (float)partX;

					// 画角補正：X
					float fX = (float)this.UVRate.X + 1.0f;
					fRateX *= (1.0f - fX) * fRateX + fX;

					glbVector2 uv = seMath.GetHermiteCurvePoint(
										posL, glbVector2.Zero,
										posR, glbVector2.Zero,
										fRateX,
										seoGridWork.PowerScale);
					uv.Div(texSize);
					uv.Add(this.UV);
					Mesh.SetUV(index, uv);
					index++;
				}
			}

			mv.EndEditVertex();
		}

		// 内部バッファを再作成
		private void UpdateMesh()
		{
			if ( this.Mesh != null)
			{
				this.Mesh.Dispose();
			}

			// メッシュ作成
			this.Mesh = seRootScene.Instance.IOManager.CreateMesh( 
				this.ObjectID, 
				MeshIndex.TriangleList,
				(this.Partitions.X + 1) * (this.Partitions.Y + 1), // 頂点数
				this.Partitions.X * this.Partitions.Y * 2 * 3);	   // 三角形数

			this.Mesh.UpdateTexture(this.ResourceTexture);

			// 三角形indexリスト作成
			this.ResetTriIndex();
			this.Mesh.SetIndex(this._triIndex);
		}


		// 初期化
		internal override void Init(int objID, seSceneManager scene)
		{
			base.Init(objID, scene);

			// 内部バッファを再作成
			this.UpdateMesh();
		}


		// 描画時に必要なワーククラスを作成
		public override seObjectWork CreateDrawWork ( seElem elem )
		{
			// UVリストを作成
//			this.ResetUV( elem.ElemID );
			return new seoGridWork(elem, this);
		}


		public void Draw( int elemID, IGLMatrix m, glbColor color, IGLMatrix mUV )
		{
			this.UpdateImage(elemID);
			color.Mul(this.Color);
			this.Mesh.Draw(elemID, m, color, mUV);
		}


		internal override void Load(IseStateInput input, seSceneManager manager)
		{
			base.Load(input, manager);

			this.ResourceID = (int)input.ReadShort();
			this.UV = seIOUtil.ReadVector2(input);
			int ix = input.ReadInt();
			int iy = input.ReadInt();
			this.Partitions = new glbVector2I(ix,iy);
			this.Color = seIOUtil.ReadColor(input);
			this.Size = seIOUtil.ReadVector2(input);	// サイズ
			this.UVRate = seIOUtil.ReadVector2(input);
		}
	}
}
