﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using glb;


namespace evc
{
	// テクスチャースプライト描画時に必要なワーク
	internal partial class seoMeshWork : seObjectWork
	{
		private seoMesh inObject; // 描画オブジェクト
		private seElem inElem; // エレメント

		// 頂点リスト(計算用)
		internal siObjectMesh Vertex { set; get; }

		// ボーン
		public seElem[] BoneList { get; set; }

		public override seObject Object { get { return this.inObject; } }

		
		// オブジェクトのサイズを取得
		public override glbVector2 GetSize ()
		{
			return this.inObject.Size;
		}

		private static IGLMatrix mInit;


		// コンストラクタ
		internal seoMeshWork ( seElem elem, seoMesh obj, siObjectMesh vartex )
		{
			if ( mInit == null )
			{
				mInit = seRootScene.Instance.IOManager.CreateMatrix();
				mInit.Identity();
			}

			this.Vertex = vartex;
			this.inElem = elem;
			this.inObject = obj;

			// ステータスの拡張データをクリア
			this.inElem.State.ClearStateData();
		}

		// 内部ワークのリセット
		internal override void Reset()
		{
			// 頂点リストを初期化？
			// 

		}

		// 描画サブルーチン
		// 再生エンジン以外で流量される為Drawと分けてあります。
		private IGLMatrix[] matWork = new IGLMatrix[1];
		private void DrawSub( IGLMatrix mBase, glbColor color )
		{
			// 頂点を計算
			if ( (this.BoneList != null) && (this.Vertex != null) )
			{
				seoMeshPos[] mstPos = this.inObject.Vertex;
				int iVertextCount = mstPos.Length;

				seMatrixWork.Instance.UseMatrixWork( delegate(IUseWorkMatrix gw )
				{
					// サイズが違う場合のみ領域確保
					int iBoneCount = this.BoneList.Length;
					if ( matWork.Length != iBoneCount )
					{
						this.matWork = new IGLMatrix[ iBoneCount ];
					}

					// 大元の逆マトリクスを掛けたものを取っておく
					IGLMatrix m = gw.GetMatrix();
					m.Copy( mBase );
					m.Invert();
					for (int i = 1; i < iBoneCount; i++)
					{
						IGLMatrix mw = gw.GetMatrix();
						mw.Copy( m );

						if ( this.BoneList[ i ] == null)
						{
							mw.Identity();
						}
						else
						{
							mw.Mul( this.BoneList[ i ].NowMatrix );
						}
						this.matWork[ i ] = mw;
					}


					this.Vertex.BeginEditVertex( this.inElem.ElemID );
					for (int i = 0; i < iVertextCount; i++)
					{
						glbVector2 outPos = glbVector2.Zero;
						byte bNonMoveWeight = 255;
						seoMeshPos pos = mstPos[i];
						for (int j = 1; j < pos.Weight.Length; j++)
						{
							byte bWeight = pos.Weight[j];
							if ( bWeight == 0 ) continue;

							if (bWeight > bNonMoveWeight)
							{
								bWeight = bNonMoveWeight;
							}

							glbVector2 p = this.matWork[j].TransformCoordinate(pos.Pos);

							p.Scale((float)bWeight / 255.0f);
							outPos.Add( p );
							bNonMoveWeight -= bWeight;
							if (bNonMoveWeight == 0) break;
						}

						// 
						if ( bNonMoveWeight > 0 )
						{
							glbVector2 nonMove = pos.Pos;
							nonMove.Scale( (float)bNonMoveWeight / 255.0f );
							outPos.Add( nonMove );
						}

						this.Vertex.SetPos( i, outPos );
					}
					this.Vertex.EndEditVertex();
				
				});

			}

			this.inObject.Draw(this.inElem.ElemID, mBase, color);
		}


		// 描画
		internal override void Draw()
		{
			this.DrawSub(this.inElem.NowMatrix, this.inElem.State.NowColor);
		}


		// 更新
		internal override void Update( float fAddTime )
		{
		}
	}


	// メッシュ用頂点
	public partial class seoMeshPos
	{
		public byte[] Weight	{ get; internal set; }	// ウェイトリスト
		public glbVector2 Pos	{ get; set; }			// 位置
		public glbVector2 UV	{ get; set; }			// UV


		public seoMeshPos( glbVector2 p, glbVector2 u, byte[] wd )
		{
			this.Pos = p;
			this.UV = u;
			this.Weight = wd;
		}

		// クローン作成
		public seoMeshPos Clone()
		{
			return new seoMeshPos( this.Pos, this.UV, this.Weight );
		}
	}


	// メッシュ描画
	public partial class seoMesh : seObject, IseObjResourceTexture
	{
		private siObjectMesh mesh = null;
		private siObjectTextureSprite texSprite = null;

		// ボーン
		public seoMeshChild RootBone { get; set; }

		// 頂点リスト
		public seoMeshPos[] Vertex { get; private set; }

		// 三角形連結情報
		public short[] Triangle { get; private set; }

		// 重みリスト
		public List<byte[]> Weight { get; private set; }

		// サイズ
		public glbVector2 Size 
		{
			get 
			{
				return (this.ResourceTexture == null) 
									? glbVector2.Zero
									: new glbVector2( this.ResourceTexture.TextureSize );
			}
			set{}
		}

		// カラー
		public glbColor Color { get; set; }

		// リソースID
		private int iResourceID = seResourceManager.InvalidID;
		public int ResourceID
		{
			get { return this.iResourceID; }
			set
			{
				this.iResourceID = value;
				this.bUpdateImage = true;
			}
		}

		// ResourceIDにあわせてテクスチャの更新
		private bool bUpdateImage = false;
		private bool UpdateImage()
		{
			if ( !this.bUpdateImage ) return false;

			// 無いなら
			if ( this.iResourceID == seResourceManager.InvalidID )
			{
				this.resTexture = null;
			}
			else
			{
				this.resTexture = this.Scene.ResourceManager.Find( this.iResourceID ) as siResourceTexture;

				// リソースはあるが準備できていない場合
				if ( (this.resTexture != null) && !this.resTexture.IsEffective )
				{
					// 更新を終了しない
					this.resTexture = null;
					return false;
				}
			}

			// 更新された
			if ( this.mesh != null )
			{
				this.mesh.UpdateTexture( this.resTexture );
			}

			this.bUpdateImage = false;
			return true;
		}


		// テクスチャリソース取得
		private siResourceTexture resTexture = null;
		public siResourceTexture ResourceTexture 
		{
			get 
			{
				this.UpdateImage();
				return this.resTexture;
			} 
		}



		public seoMesh()
		{
			this.RootBone = new seoMeshChild();
			this.RootBone.Mesh = this;

			this.Color = glbColor.White;
			this.bUpdateImage = false;
		}

		// 初期化
		internal override void Init(int objID, seSceneManager scene)
		{
			base.Init(objID, scene);
			// 新しいIDが割り当てられたので作りなおす
			if ( this.mesh != null )
			{
				this.mesh.Dispose();
			}

			siIOManager iom = seRootScene.Instance.IOManager;
			if ( (this.Vertex != null) && (this.Triangle!=null) )
			{
				this.mesh = iom.CreateMesh( this.ObjectID, 
											MeshIndex.TriangleList,
											this.Vertex.Length,
											this.Triangle.Length );
				this.mesh.UpdateTexture( this.ResourceTexture );
				this.mesh.SetIndex( this.Triangle );
			}

			this.texSprite = iom.CreateTextureSprite(this.ObjectID );
		}

		// 描画オブジェクトタイプ
		public override emObjectType ElemType	{ get { return emObjectType.Mesh; } }


		// 描画時に必要なワーククラスを作成
		public override seObjectWork CreateDrawWork ( seElem inElem )
		{
			// ワーク作成時に１度だけUVを設定
			// エレメントと毎にバッファを持つ場合を想定します。
			siObjectMesh mv = this.mesh;
			mv.BeginEditVertex( inElem.ElemID );
			for (int i = 0; i < this.Vertex.Length; i++)
			{
				mv.SetUV( i, this.Vertex[ i ].UV );
			}
			mv.EndEditVertex();
			return new seoMeshWork( inElem, this, this.mesh );
		}


		public void Draw( int elemID, IGLMatrix m, glbColor color )
		{
			this.UpdateImage();
			color.Mul( this.Color );
			this.mesh.Draw( elemID, m, color, seMatrixWork.Instance.Identity );
		}


		internal override void Load(IseStateInput input, seSceneManager inManager)
		{
			base.Load(input, inManager);
			this.ResourceID = (int)input.ReadShort();
			this.Color = seIOUtil.ReadColor( input );

			// 階層構造ロード
			glbAction<BoneWork> sub = null;
			sub = delegate( BoneWork parent )
			{
				BoneWork bw = new BoneWork();
				bw.ObjID = input.ReadInt();
				short childCount = input.ReadShort();
				if ( parent != null )
				{
					parent.Child.Add( bw );
				}
				else
				{
					this.LoadBoneWork = bw;
				}

				for (int i = 0; i < childCount; i++)
				{
					sub( bw );
				}
			};
			sub( null );

			// 重みリスト
			if ( this.Weight == null )
			{
				this.Weight = new List<byte[]>();
			}
			else
			{
				this.Weight.Clear();
			}

			int iCount = (int)input.ReadShort();
			if ( iCount > 0 )
			{
				int weightCount = (int)input.ReadShort();
				for (int i = 0; i < iCount; i++)
				{
					byte[] weight = new byte[ weightCount ];
					for (int j = 0; j < weightCount; j++)
					{
						weight[ j ] = input.ReadByte();
					}
					this.Weight.Add( weight );
				}
			}


			// 頂点データ
			this.Vertex = null;
			int ivCount = (int)input.ReadShort();
			if ( ivCount > 0 )
			{
				input.ReadShort();
				input.ReadShort();
				short sx = input.ReadShort();
				short sy = input.ReadShort();
				glbVector2 uvRate = new glbVector2( (float)sx, (float)sy );
				uvRate.Sub( glbVector2.One );

				// データサイズ取得
				byte xsize = input.ReadByte();
				byte ysize = input.ReadByte();
				byte wsize = input.ReadByte();

				List<seoMeshPos> lisMeshPos = new List<seoMeshPos>();
				for (int i = 0; i < ivCount; i++)
				{
					int iX = this.ReadChangeability(input,xsize);
					int iY = this.ReadChangeability(input,ysize);
					int wID = this.ReadChangeability(input,wsize);

					glbVector2 vpos = new glbVector2( (float)iX, (float)iY );
					glbVector2 uv = vpos;
					uv.Div( uvRate );

					seoMeshPos pos = new seoMeshPos( vpos, uv, this.Weight[ wID ] );
					lisMeshPos.Add( pos );
				}
				this.Vertex = lisMeshPos.ToArray();
			}

			// indexリスト
			this.Triangle = null;
			int idCount = (int)input.ReadShort();
			if ( idCount > 0 )
			{
				byte isize = input.ReadByte();
				MeshIndex indexType = input.ReadEnum<MeshIndex>();

				short[] ids = new short[ idCount ];
				switch (indexType)
				{
					case MeshIndex.TriangleList:
						for (int i = 0; i < idCount; i++)
						{
							ids[ i ] = (short)this.ReadChangeability( input, isize );
						}
						break;
					default:
						Debug.Assert( false );
						break;
				}
				this.Triangle = ids;
			}



		}

		// 可変データを取得
		private int ReadChangeability( IseStateInput input, byte size )
		{
			switch (size)
			{
				case 1: return (int)input.ReadByte();
				case 2: return (int)input.ReadShort();
				default: Debug.Assert( false ); break;
			}
			return 0;
		}

		// 階層一時データから実データに変換
		internal void LinkMechChild( seObjectManager objManager )
		{
			glbAction<seoMeshChild,BoneWork> sub = null;
			sub = delegate( seoMeshChild parent, BoneWork bwork )
			{
				seoMeshChild meshChild = (seoMeshChild)objManager.Find( bwork.ObjID );
				meshChild.Mesh = this;
				if ( parent != null )
				{
					parent.Link( meshChild );
				}
				else
				{
					this.RootBone = meshChild;
				}

				foreach (var bone in bwork.Child)
				{
					sub( meshChild, bone );
				}
			};

			if ( this.LoadBoneWork != null )
			{
				sub( null, this.LoadBoneWork );
			}
			this.LoadBoneWork = null;
		}

	
		// 階層構造一時的補間用
		internal class BoneWork
		{
			public int ObjID { get; set; }
			public List<BoneWork> Child { get; set; }

			public BoneWork()
			{
				this.Child = new List<BoneWork>();
			}
		}

		private BoneWork LoadBoneWork;


		// メッシュにデータを登録
		internal void RegisterMesh()
		{
			if ( this.mesh != null )
			{
				this.mesh.Dispose();
			}
			this.mesh = seRootScene.Instance.IOManager.CreateMesh( 
													this.ObjectID, 
													MeshIndex.TriangleList,
													this.Vertex.Length,
													this.Triangle.Length );

			// この処理が必要か検証
			// 頂点データは描画前に設定されている
			// 

			this.mesh.UpdateTexture( this.ResourceTexture );
			this.mesh.SetIndex( this.Triangle );

			siObjectMesh mv = this.mesh;
			mv.BeginEditVertex( seElemManager.InvalidID );
			for (int i = 0; i < this.Vertex.Length; i++)
			{
				seoMeshPos v = this.Vertex[i];
				mv.SetPos( i, v.Pos );
				mv.SetUV( i, v.UV );
			}
			mv.EndEditVertex();
		}


		// メッシュの情報を使用するエレメントを更新
		public void UpdateMeshElem( seElem elem )
		{
			seoMeshWork meshWork = elem.ObjectWork as seoMeshWork;
			if ( meshWork == null ) return;

			Debug.Assert( (seoMesh)meshWork.Object == this );

			// メッシュ更新処理
			meshWork.Vertex = this.mesh;

			// seElemからseoMeshChildを取得該当しない場合はnull
			glbFunc<seElem,seoMeshChild> GetMeshChild = delegate( seElem subElem )
			{
				seObjectWork objWork = subElem.ObjectWork;
				if ( objWork == null ) return null;
				if ( objWork.Object.ElemType != emObjectType.MeshChild ) return null;
				return (seoMeshChild)objWork.Object;
			};

			// メッシュより下の連続するMeshChildをリスト化する。
			List<seElem> lisChild = new List<seElem>();
			glbAction<seElem> sub = null;
			int iMaxLabel = 0;
			sub = delegate( seElem subElem )
			{
				seoMeshChild meshChild = GetMeshChild( subElem );
				if ( meshChild == null ) return;
				// 親が違うmeshChildの場合は無処理
				if ( meshChild.Mesh != this ) return;

				// 登録できた場合は更に下位も見る
				lisChild.Add( subElem );
				iMaxLabel = Math.Max( iMaxLabel, meshChild.Label );


				foreach (var childElem in subElem.Child)
				{
					sub( childElem );
				}
			};

			foreach (var e in elem.Child)
			{
				sub(e);
			}


			// ボーンリストの作成・登録
			seElem[] elems = new seElem[ iMaxLabel + 1 ];
			foreach (var e in lisChild)
			{
				seoMeshChild meshChild = GetMeshChild( e );
				elems[ meshChild.Label ] = e;
			}
			meshWork.BoneList = elems;
		}

		// メッシュの情報を使用するエレメント全てを更新
		public void UpdateMeshElem()
		{
			// メッシュを使うエレメントに対して処理を実行
			this.DoUseElem( delegate( seElem elem )
			{
				// ボーン構造を再登録
				this.UpdateMeshElem(elem); 
				return true;
			});
		}


		// メッシュを使うエレメントに対して処理を実行
		public void DoUseElem( glbFunc<seElem,bool> func )
		{
			// 全てのエレメントに対して処理を実行
			this.Scene.ElemManager.DoAll(delegate(seElem elem)
			{
				// メッシュに対してのみ実行
				seoMeshWork meshWork = elem.ObjectWork as seoMeshWork;
				if (meshWork == null) return true;
				// 編集したメッシュオブジェクトを持つエレメントにのみ実行
				if (meshWork.Object != this) return true;

				// メッシュを持つエレメントに対して処理を実行
				return func(elem); 
			});
		}
	}
}
