﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using glb;


namespace evc
{
	// メッシュ描画時に必要なワーク
	internal partial class seoMeshChildWork : seObjectWork
	{
		public seElem Elem { get; private set; }

		private seoMeshChild obj { get; set; } // 描画オブジェクト
		public override seObject Object { get { return this.obj; } }


		// オブジェクトのサイズを取得
		public override glbVector2 GetSize ()
		{
			return this.obj.Size;
		}

		// コンストラクタ
		internal seoMeshChildWork( seElem elem, seoMeshChild child )
		{
			this.Elem = elem;
			this.obj = child;

			// ステータスの拡張データをクリア
			this.Elem.State.ClearStateData();
		}

		// 内部ワークのリセット
		internal override void Reset(){}
		// 描画	
		internal override void Draw(){}
		// 更新
		internal override void Update( float fAddTime ){}
	}


	// メッシュ描画
	public partial class seoMeshChild : seObject
	{
		// ラベル
		public int Label { get; private set; }
		// 大元のメッシュ
		public seoMesh Mesh { get; set; }
		// 自分の親
		public seoMeshChild Parent { get; private set; }
		// 子リスト
		public List<seoMeshChild> Child { get; private set; }

		// サイズ
		public glbVector2 Size { get; set; }
		// 位置
		public glbVector2 Pos { get; private set; }


		// 描画オブジェクトタイプ
		public override emObjectType ElemType	{ get{return emObjectType.MeshChild;} }


		// 子を追加
		public void Link( seoMeshChild child )
		{
			this.Child.Add( child );
			child.Parent = this;
			child.Mesh = this.Mesh;
		}

		// コンストラクタ
		public seoMeshChild()
		{
			this.Child = new List<seoMeshChild>();
		}

		// 初期化
		internal override void Init( int objID, seSceneManager scene)
		{
			base.Init(objID, scene);
		}

		// 描画時に必要なワーククラスを作成
		public override seObjectWork CreateDrawWork ( seElem elem )
		{
			return new seoMeshChildWork( elem, this );
		}


		public void Draw(int iElemID, IGLMatrix m, glbColor sMulColor){}

		internal override void Load(IseStateInput input, seSceneManager scene)
		{
			base.Load(input, scene);

			this.Label = input.ReadInt();
			this.Size = seIOUtil.ReadVector2( input );
			this.Pos = seIOUtil.ReadVector2( input );
		}

	}
}
