﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.Diagnostics;
using glb;

namespace evc
{
	// 数値イメージリスト
	public enum seoNumImage
	{
		NUM_0 = 0,
		NUM_1,
		NUM_2,
		NUM_3,
		NUM_4,
		NUM_5,
		NUM_6,
		NUM_7,
		NUM_8,
		NUM_9,
		Comma,
		Minus,
	};


	// 数値矩形管理
	// 他のテクスチャオブジェクトを(0～9,カンマ、マイナス)をまとめて
	// 数値を表示する
	
	internal partial class seoNumWork : seObjectWork
	{
		private seElem inElem;
		private seoNum inObject;
		// 拡張ステータス
		private seStateInt1 inNum;
		private seStateNumInfo inNumInfo;


		public override seObject Object { get { return this.inObject; } }

		// コンストラクタ
		internal seoNumWork ( seElem inInElem, seoNum inInObjct )
		{
			this.inElem = inInElem;
			this.inObject = inInObjct;

			this.inNum = (seStateInt1)seStateUtil.Instance.CreateStateType( emElemState.Num );
			this.inNumInfo = (seStateNumInfo)seStateUtil.Instance.CreateStateType( emElemState.NumState );

			// ステータスの拡張データをクリア
			this.inElem.State.ClearStateData();
			this.inElem.State.LinkStateData( emElemState.Num, this.inNum );
			this.inElem.State.LinkStateData( emElemState.NumState, this.inNumInfo );
		}

		// 描画
		internal override void Draw ()
		{
			this.inObject.Draw( this.inElem.ElemID, 
								this.inElem.NowMatrix, 
								this.inElem.State.NowColor,
								this.inNum.Value,
								this.inNumInfo );
		}

		// 更新
		internal override void Update ( float fAddTime ) {}

		// オブジェクトのサイズを取得
		public override glbVector2 GetSize ()
		{
			// 文字幅を返す
			float fNumSize = this.inObject.GetNumWidth( this.inNum.Value, this.inNumInfo );

			// 揃え別処理
			switch (this.inNumInfo.Arrange)
			{
				case seNumArrange.L:
				case seNumArrange.Point:
					break;

				case seNumArrange.R:
				default:
					// 右そろえの場合は符号をひっくり返す。
					fNumSize = -fNumSize;
					break;
			}

			// 下を基準点にする為、Ｙはひっくり返す。
			return new glbVector2( fNumSize, -this.inObject.Height );
		}
	
	}




	// カラー描画
	public partial class seoNum : seObject
	{
		private static readonly int ListNum = EnumList<seoNumImage>.GetValues().Length;
		internal ReadOnlyCollection<seoNumTexture> Images { get; private set; }
		
		public glbColor Color { get; set; }		// カラー
		internal float Height { get; private set; }		// 高さ

		// 描画オブジェクトタイプ
		public override emObjectType ElemType { get { return emObjectType.Num; } }


		// コンストラクタ
		public seoNum ()
		{
			this.Color = glbColor.White;

			// イメージリストIDを初期化
			seoNumTexture[] imageList = new seoNumTexture[ seoNum.ListNum ];
			for (int i = 0; i < imageList.Length; i++)
			{
				imageList[ i ] = new seoNumTexture( this );
			}

			this.Images = new ReadOnlyCollection<seoNumTexture>(imageList);
		}


		// イメージ管理：ObjectIDを設定しておくと、必要な時にテクスチャオブジェクトを返します
		// ロード時等、ID取得時にオブジェクトが存在しない場合の遅延処理。
		// 
		internal class seoNumTexture
		{
			private seObject _parent;
			private seoTextureSprite _image = null;

			// イメージ取得
			public seoTextureSprite Image
			{
				get
				{
					if ( (this._image == null) && (this._index != seObjectManager.InvalidID) )
					{
						this._image = _parent.Scene.ObjectManager.Find( this._index ) as seoTextureSprite;
					}
					return this._image;
				}
			}

			private int _index = seObjectManager.InvalidID;
			public int ID 
			{
				get { return this._index; }
				set 
				{
					this._index = value;
					this._image = null;
				}
			}

			// コンストラクタ
			public seoNumTexture( seObject parent )
			{
				this._parent = parent;
			}
			private seoNumTexture( seoNumTexture mst )
			{
				this._parent = mst._parent;
				this._image = mst._image;
			}
	
			// クローン
			public seoNumTexture Clone() { return new seoNumTexture(this); }
		}


		// イメージ設定
		public void SetImage( seoNumImage eImageType, seoTextureSprite obj )
		{
			this.Images[ (int)eImageType ].ID = (obj==null)
												? seObjectManager.InvalidID
												: obj.ObjectID;
			// 一番大きな高さを取得
			this.Height = 0.0f;
			foreach (var sprite in this.Images)
			{
				if (sprite == null) continue;
				this.Height = Math.Max( this.Height, (float)sprite.Image.Size.Y );
			}
		}

		public bool DoAll( glbFunc<seoNumImage, seoTextureSprite,bool> func )
		{
			for (int i = 0; i < this.Images.Count; i++)
			{
				if(!func((seoNumImage)i, this.Images[i].Image)) return false;
			}
			return true;
		}

		// イメージ取得
		public seoTextureSprite GetImage( seoNumImage eImageType )
		{
			return this.Images[ (int)eImageType ].Image;
		}



		// 描画時に必要なワーククラスを作成
		public override seObjectWork CreateDrawWork ( seElem inElem )
		{
			return new seoNumWork( inElem, this );
		}

	
		// 描画
		internal void Draw( int iElemID, 
							IGLMatrix mParent, 
							glbColor sMulColor, 
							int iNum,
							seStateNumInfo inNumInfo )
		{
			// カラーの算出
			sMulColor.Mul( this.Color );


			// 固定文字幅
			int iFixationWidth = inNumInfo.NumLen;
			int iCommaWidth = inNumInfo.CommaLen;

			// 可変文字幅
			bool bFixCharLenge = (inNumInfo.CharLen == seNumCharLen.Fix);

			float fWidth = 0.0f;

			// 数値情報を設定
			List<seoTextureSprite> lisImage = new List<seoTextureSprite>();
			List<float> lisWidth = new List<float>();
			this.SetNumInfo( iNum, inNumInfo, delegate(seoNumImage eNumImage)
			{
				seoTextureSprite texture = this.GetImage(eNumImage);
				if (texture == null) return;

				// 固定文字幅で１の位以上なら
				float fNowWidth;
				if ( bFixCharLenge && (lisImage.Count > 0))
				{
					fNowWidth = (eNumImage == seoNumImage.Comma) ? (float)iCommaWidth
																 : (float)iFixationWidth;
				}
				else
				{
					fNowWidth = (float)texture.Size.X;
				}

				fWidth += fNowWidth;

				// 見え方の違い
				lisWidth.Insert(0, fNowWidth );
				lisImage.Insert(0, texture );
//				lisWidth.Add( fNowWidth );
//				lisImage.Add( texture );
			});


			// 設定済みのainFontListから文字幅を取得
			float fXOffset = 0.0f;

			switch (inNumInfo.Arrange)
			{
				// 左揃え
				case seNumArrange.L:
					fXOffset = 0.0f;
					break;
				// ポイント揃え
				case seNumArrange.Point:
					break;
				// それ以外は右揃え
				case seNumArrange.R:
				default:
					fXOffset = -fWidth;
					break;
			}

			seMatrixWork.Instance.UseMatrixWork( delegate( IUseWorkMatrix gw )
			{
				IGLMatrix m = gw.GetMatrix();
			
				List<float>.Enumerator it = lisWidth.GetEnumerator();
				it.MoveNext();
				lisImage.ForEach( delegate( seoTextureSprite texture )
				{
					m.SetPos( fXOffset, (float)-texture.Size.Y );
					m.MulR( mParent );
					
					// 数値描画
					texture.Draw( iElemID, m, sMulColor );
					// 差分移動
					fXOffset += it.Current;
					it.MoveNext();
				});
			});
		}


		// 表示数値データを入れる
		// @retval : 表示数値幅
		internal void SetNumInfo( int iNum, seStateNumInfo numInfo, glbAction<seoNumImage> func )
		{
			// 数値がマイナスの場合
			bool bMinus = false;
			if ( iNum < 0 )
			{
				iNum = -iNum;
				// マイナス表示なら
				bMinus = (numInfo.VisibleMinus == seNumVisible.On);
			}

			// 表示限界(最小は１)
			int iMaxDisit = Math.Max( numInfo.MaxDisit, 1 );
			// 数値解析
			for(int i = 0; i < iMaxDisit; i++ )
			{
				// カンマ挿入
				if ( numInfo.VisibleComma == seNumVisible.On )
				{
					if ( (i>0) && ((i%3)==0) )
					{
						func(seoNumImage.Comma);
					}
				}

				// 数値情報を設定
				func((seoNumImage)(seoNumImage.NUM_0 + (iNum % 10)));

				iNum /= 10;
				// ０埋めでなくて、もう数値が無い場合は終了
				if ( (iNum <= 0) && (numInfo.ZeroBuries != seNumVisible.On) )
					break;
			}

			// マイナスなら
			if ( bMinus )
			{
				func(seoNumImage.Minus);
			}
		}


		// 設定済みのainFontListから文字幅を取得
		internal float GetNumWidth( int iNum, seStateNumInfo numInfo)
		{
			// 固定文字幅
			int iFixationWidth = numInfo.NumLen;
			int iCommaWidth = numInfo.CommaLen;
			// 可変文字幅
			bool bFixCharLenge = (numInfo.CharLen == seNumCharLen.Fix);

			float fWidth = 0.0f;
			bool bFirst = true;
			this.SetNumInfo( iNum, numInfo, delegate(seoNumImage eNumImage)
			{
				seoTextureSprite texture = this.GetImage(eNumImage);
				if (texture == null) return;

				// 固定文字幅で１の位以上なら
				if ( bFixCharLenge && !bFirst )
				{
					fWidth += (eNumImage == seoNumImage.Comma) ? (float)iCommaWidth
															   : (float)iFixationWidth;
				}
				else
				{
					fWidth += (float)texture.Size.X;
				}
				bFirst = false;
			});
			return fWidth;
		}

		// 解放処理
		internal override void Dispose ()
		{
		}


		// ロード
		internal override void Load(IseStateInput input, seSceneManager manager)
		{
			base.Load(input, manager);
			this.Color = seIOUtil.ReadColor( input );
			this.Height = input.ReadFloat();

			// イメージリスト
			for (int i = 0; i < seoNum.ListNum; i++)
			{
				this.Images[ i ].ID = (int)input.ReadShort();
			}
		}
	
	}


}
