﻿using System;
using glb;

using System.Diagnostics;


namespace evc
{
	// テクスチャースプライト描画時に必要なワーク
	internal partial class seoParticlesWork : seObjectWork
	{
		// パーティクル１つ管理
		internal class ParticleOne
		{
			public ParticleOne Befor { get; set; } // 前のデータ
			public ParticleOne Next { get; set; } // 次のデータ

			public glbColor Color { get; private set; }	// ローカルカラー
			public IGLMatrix LocalMatrix { get; private set; } // ローカルマトリクス

			private glbColor _baseColor;	// ベースカラー
			private float _lifeTime;	// 生存期間
			private float _elapsedTime;	// 生まれてから経過した時間
			private seoParticlesWork _objWork;	// 元
			public glbVector2 _power;	// 移動先
			public glbVector2 _power2;	// 影響力
			public glbVector2 _startPos; // 開始時の位置

			private float _baseRot;		// 補正回転角度
			private glbVector2 _baseScale;	// 補正スケール

			private bool _bRotDirection; // 移動方向から角度を算出するモードか否か

			// コンストラクタ
			public ParticleOne( seoParticlesWork objWork)
			{
				this._objWork = objWork;
				this.LocalMatrix = seRootScene.Instance.IOManager.CreateMatrix();
			}

			// １パーティクル開始
			// @param rand : ランダム数値提供
			// @param fTime : 経過時間
			// @retval : true 再生中 : false 再生が終了した
			public bool Start( Random rand, float fTime, glbVector2 offset )
			{
				seStateParticlesInfo partInfo = this._objWork._particlesObj.Info;

				int min = partInfo.LifeMin;
				int max = partInfo.LifeMax;
				this._lifeTime = (float)((max >= min) ? rand.Next(min, max)
													  : rand.Next(max, min)) *
								 seRootScene.Instance.RootScene.Option.FrameTime;

				// 再生時間より経過時間の方が長い場合は何もせず終了
				if (fTime >= this._lifeTime) return false;

				this._baseRot = ((float)rand.NextDouble() * partInfo.RotLenge * 2.0f) - partInfo.RotLenge;
				float fscale = ((float)rand.NextDouble() * (partInfo.ScaleLenge - 1.0f)) + 1.0f;
				this._baseScale.Set(fscale, fscale);

				seElem elem = this._objWork._elem;


				// 変換後の座標取得
				IGLMatrix mElem = elem.NowMatrix;
				glbVector2 wZero = mElem.TransformCoordinate(glbVector2.Zero);
				glbVector2 wWidth = mElem.TransformCoordinate(new glbVector2(1.0f, 0.0f));
				glbVector2 wHeight = mElem.TransformCoordinate(new glbVector2(0.0f, 1.0f));
				float fElemRot = seMath.GetRot(wZero, wWidth);

				glbVector2 objSize = elem.ObjectWork.GetSize();
				this._baseColor = elem.State.NowColor;


				// スケール継承なら
				if (this._objWork._successionScale)
				{
					this._baseScale.X *= ((float)seMath.GetDistance(wZero,wWidth));
					this._baseScale.Y *= ((float)seMath.GetDistance(wZero,wHeight));
				}

				// 回転モード
				this._bRotDirection = false;
				switch (this._objWork._successionRot)
				{
					case ParticlesRot.Succession:
						this._baseRot += fElemRot;
						break;
					// 移動方向から角度を算出
					case ParticlesRot.Direction:
						this._bRotDirection = true;
						break;
					default:
						break;
				}


				seMatrixWork.Instance.UseMatrixWork( delegate( IUseWorkMatrix gw )
				{
					// アクティブな要素にサイズを掛けたマトリクスを算出
					IGLMatrix mSize = gw.GetMatrix();
					IGLMatrix mOffset = gw.GetMatrix();
					IGLMatrix m = gw.GetMatrix();

					elem.GetRectMatrix( m );
					mSize.SetScale(objSize);

					// パーティクルステータスから、中心位置を取得
					glbVector2 center = this.GetCenter(partInfo, 0.0f);
					center.Scale(-1.0f);
					mOffset.SetPos(center);

					m.MulR( mOffset );
					m.Mul( mSize );

					// 出力位置、放射情報
					switch (this._objWork._outType)
					{
						// 面
						case ParticlesOutType.Plane:
							this._startPos = m.TransformCoordinate(
							new glbVector2((float)rand.NextDouble(),
										   (float)rand.NextDouble()));
							break;
						// 点
						case ParticlesOutType.Point:
							this._startPos = m.TransformCoordinate(new glbVector2(0.5f, 0.5f));
							break;
						default:
							break;
					}
				});

				this._startPos.Add(offset);


				float fAddRot = 0.0f;
				glbFunc<glbVector2,float,float,glbVector2> createPower = delegate(
						glbVector2 v, float rotLen, float scaleLen )
				{
					// 移動パワー
					v.Scale(((scaleLen - 1.0f) * (float)rand.NextDouble()) + 1.0f);

					// 範囲(ランダム角度幅＋出力の回転角)
					double dRot = (rotLen * 2.0f * (float)rand.NextDouble()) - rotLen;
					dRot = (dRot * Math.PI / 180.0) + fAddRot;
					float sinR = (float)Math.Sin(dRot);
					float cosR = (float)Math.Cos(dRot);
					return new glbVector2(v.X * cosR - v.Y * sinR,
										  v.X * sinR + v.Y * cosR);
				};

				// エレメントの角度を加算
				seStateParticles state = this._objWork._partState;
				fAddRot = fElemRot;
				this._power = createPower(state.OutVector, 
							state.OutLenge, state.OutScaleLenge );

				fAddRot = 0.0f;	// 角度を加算しない。
				this._power2 = createPower(state.InfluenceVector, 
							state.InfluenceLenge, state.InfluenceScaleLenge );


				this._elapsedTime = 0.0f;
				return this.Update(fTime);
			}

			// パーティクルステータスから、中心位置を取得
			private glbVector2 GetCenter( seStateParticlesInfo partInfo, float rate )
			{
				seStateParticlesInfo.State<glbVector2> centerState = partInfo.Center;
				glbVector2 center = centerState.End;
				center.Sub(centerState.Start);
				center.Scale(FePointInfo.GetPos(rate, centerState.Anim));
				center.Add(centerState.Start);
				return center;
			}


			// マトリクスとカラーを更新
			private void UpdateMatrixAndColor( float rate, glbVector2 pos )
			{
				seStateParticlesInfo partInfo = this._objWork._particlesObj.Info;
				// スケール
				seStateParticlesInfo.State<glbVector2> scaleState = partInfo.Scale;
				glbVector2 scale = scaleState.End;
				scale.Sub(scaleState.Start);
				scale.Scale(FePointInfo.GetPos(rate, scaleState.Anim));
				scale.Add(scaleState.Start);
				// ぶれ幅
				scale.Mul(this._baseScale);
				// パーティクルステータスから、中心位置を取得
				glbVector2 center = this.GetCenter( partInfo, rate );

				// カラー
				seStateParticlesInfo.State<glbColor> colorState = partInfo.Color;
				glbColor sc = colorState.Start;
				glbColor ec = colorState.End;
				float fstateRate = FePointInfo.GetPos(rate, colorState.Anim);
				glbColor c = new glbColor(
					(byte)((int)((float)(ec.R - sc.R) * fstateRate) + sc.R),
					(byte)((int)((float)(ec.G - sc.G) * fstateRate) + sc.G),
					(byte)((int)((float)(ec.B - sc.B) * fstateRate) + sc.B),
					(byte)((int)((float)(ec.A - sc.A) * fstateRate) + sc.A));
				c.Mul(this._baseColor);
				this.Color = c;


				// 回転
				float fRot = 0.0f;

				// 移動方向角度
				if ( this._bRotDirection) 
				{
					// ２点から角度を算出
					glbVector2 np = this.GetPos(this._elapsedTime + seRootScene.Instance.RootScene.Option.FrameTime * 0.5f);
					fRot = seMath.GetRot(pos,np);
				}
				// それ以外
				else
				{
					seStateParticlesInfo.State<float> rs = partInfo.Rot;
					fRot = (rs.End - rs.Start) * FePointInfo.GetPos(rate, rs.Anim) + rs.Start;
					fRot = (fRot + this._baseRot) % 360.0f;
					fRot = fRot / 180.0f * (float)Math.PI;
				}

				seMath.Transformation2D(this.LocalMatrix, center, scale, fRot, pos);
			}

			// 任意の時間の位置を取得
			// メンバを更新せずに位置を取得できます。
			private glbVector2 GetPos( float fTime )
			{
				// 移動量加算
				float rate = Math.Min(1.0f, Math.Max(0.0f, fTime / this._lifeTime));

				glbVector2 p1 = this._power;
				p1.Scale(rate);
				glbVector2 p2 = this._power2;
				p2.Scale(rate);
				p2.Add(this._power);

				glbVector2 p = p2;
				p.Sub(p1);
				p.Scale(rate);
				p.Add(p1);
				p.Add(this._startPos);
				return p;
			}


			// 状態の更新
			// これは前回の値から足すような形ではなく、PosとPowerから計算する。
			public bool Update( float fAddTime )
			{
				this._elapsedTime += fAddTime;

				// 寿命を超えているなら
				if (this._elapsedTime >= this._lifeTime)
					return false;

				// 任意の時間の位置を取得
				glbVector2 p = this.GetPos( this._elapsedTime );
				float rate = this._elapsedTime / this._lifeTime;
				this.UpdateMatrixAndColor(rate, p);
				return true;
			}


			// パーティクルの追加/挿入
			// パーティクルは１つづつ追加されるものとする。
			public void AddOne( ParticleOne next )
			{
				next.Next = this.Next;
				next.Befor = this;
				if (this.Next != null)
				{
					this.Next.Befor = next;
				}
				this.Next = next;
			}

			// パーティクルの削除
			public void Cut()
			{
				if (this.Befor != null)
				{
					this.Befor.Next = this.Next;
				}
				if (this.Next != null)
				{
					this.Next.Befor = this.Befor;
				}
				this.Next = null;
				this.Befor = null;
			}

		}

		private Random _rand = new Random(DateTime.Now.Millisecond);
		private seoParticles _particlesObj;		// 描画オブジェクト
		private seElem _elem; // エレメント
		private ParticleOne _particlesList;		// パーティクルリスト
		private ParticleOne _lastParticles;		// パーティクルリストの最後尾
		private ParticleOne _unuseList;			// 未使用パーティクルリスト
		private bool _bPlay; // パーティクル発生中か否か
		private bool _successionScale; // スケール継承中か否か
		private ParticlesRot _successionRot; // 回転継承中か否か
		private ParticlesDrawPriority _drawPriority; // 描画モード
		private ParticlesOutType _outType; // 現在の出力状態


		// ステータス
		private seStateParticles _partState;	// パーティクルステータス
		private seStateFloat2 _uv;

		// 前回パーティクルを処理してからの経過時間
		// パーティクルの出現量は１フレームに対する数で指定されます。
		// 経過時間がフレームタイムで割り切れない場合、余りを次回に繰越し
		// パーティクルの出現量を調整します。
		private float _elapsedTime;

		public override seObject Object { get { return this._particlesObj; } }


		// オブジェクトのサイズを取得
		public override glbVector2 GetSize ()
		{
			return this._particlesObj.Size;
		}

		// 未使用リストの先頭に追加
		private void InsertUnuseTop( ParticleOne p )
		{
			p.Next = this._unuseList;
			if ( this._unuseList != null)
			{
				this._unuseList.Befor = p;
			}
			this._unuseList = p;
			p.Befor = null;
		}

		// 未使用リストから１つ取得
		private ParticleOne GetUnuseInstance()
		{
			ParticleOne ret = this._unuseList;
			if ( ret != null)
			{
				this._unuseList = ret.Next;
				if( this._unuseList != null)
				{
					this._unuseList.Befor = null;
				}
				ret.Next = null;
			}
			return ret;
		}

		// 次に取得される未使用パーティクルを取得
		private ParticleOne PeekNextUnuse()
		{
			return this._unuseList;
		}

		// パーティクルバッファのリサイズ
		private void ResizeParticlesBuffer( int iPartNum )
		{
			this._particlesList = null;
			this._lastParticles = null;
			this._unuseList = null;
			// パーティクル情報を作成して未使用リストに登録
			for (int i = 0; i < iPartNum; i++)
			{
				this.InsertUnuseTop(new ParticleOne( this ));
			}
		}

		// パーティクルバッファを全て未使用にする
		private void ResetParticlesBuffer()
		{
			// パーティクルを全てカット
			while( this._particlesList != null)
			{
				ParticleOne p = this._particlesList;
				this._particlesList = p.Next;
				p.Next = null;
				p.Befor = null;
				this.InsertUnuseTop(p);
			}
			this._lastParticles = null;
		}


		// コンストラクタ
		internal seoParticlesWork ( seElem elem, seoParticles obj)
		{
			// パーティクルバッファのリサイズ
			this.ResizeParticlesBuffer( obj.Info.MaxNum );
			this._elem = elem;
			this._particlesObj = obj;
			this._elapsedTime = 0.0f;

			this._partState = (seStateParticles)seStateUtil.Instance.CreateStateType(emElemState.ParticleControl);
			this._uv = (seStateFloat2)seStateUtil.Instance.CreateStateType( emElemState.UV );

			// ステータスの拡張データをクリア
			this._elem.State.ClearStateData();
			this._elem.State.LinkStateData(emElemState.ParticleControl, this._partState);
			this._elem.State.LinkStateData(emElemState.UV, this._uv);
		}

		// 内部ワークのリセット
		internal override void Reset()
		{
			// パーティクルバッファを全て未使用にする
			this.ResetParticlesBuffer();
			this._elapsedTime = 0.0f;
			this._bPlay = false;
			this._successionRot = ParticlesRot.Normal;
			this._successionScale = false;
			this._drawPriority = ParticlesDrawPriority.Top;
			this._outType = ParticlesOutType.Plane;
		}

		// 描画	
		internal override void Draw ()
		{
			if (seRootScene.Instance.EditMode)return;

			seMatrixWork.Instance.UseMatrixWork( delegate(IUseWorkMatrix gw )
			{
				IGLMatrix m = gw.GetMatrix();

				ParticleOne p = this._particlesList;
				while(p != null)
				{
					m.Copy(p.LocalMatrix);
					m.Mul(this._elem.State.ParentMatrix);

					glbColor color = p.Color;
					// パーティクル出力の親のマトリクスを継承する
					color.Mul(this._elem.Parent.State.NowColor);
					this._particlesObj.Draw( this._elem.ElemID, m, color);
					p = p.Next;
				}
			});
		}

		// 更新
		internal override void Update ( float fAddTime )
		{
			// 出力中のパーティクル更新
			ParticleOne p = this._particlesList;
			while(p != null)
			{
				ParticleOne next = p.Next;
				// 非表示なら未使用リストへ
				if (!p.Update(fAddTime))
				{
					// 最後尾なら
					if(p == this._lastParticles)
					{
						this._lastParticles = p.Befor;
					}
					// 先頭なら
					if(p == this._particlesList)
					{
						this._particlesList = p.Next;
					}

					// 使用パーティクルリストからはずす
					p.Cut();
					// 未使用リストに登録
					this.InsertUnuseTop(p);
				}
				p = next;
			}

			seStateData data = this._elem.State.GetData( emElemState.ParticleControl );
			bool bNext = true;
			// 新しい状態になったなら出力フラグ更新
			if ((data != null) && (data.MoveKey))
			{
				switch (this._partState.Play)
				{
					case ParticlesPlay.Play: 
						this._bPlay = true;
						bNext = true;
						break;
					case ParticlesPlay.PlayOne:
						this._bPlay = true;
						bNext = false;
						break;
					case ParticlesPlay.Stop:
						this._bPlay = false;
						this._elapsedTime = 0.0f;
						break;
					// 
					default:
						break;
				}


				if( this._partState.SuccessionRot != ParticlesRot.Non )
				{
					this._successionRot = this._partState.SuccessionRot;
				}

				if (this._partState.Priority != ParticlesDrawPriority.Non )
				{
					this._drawPriority = this._partState.Priority;
				}

				if (this._partState.OutType != ParticlesOutType.Non )
				{
					this._outType = this._partState.OutType;
				}

				switch (this._partState.SuccessionScale)
				{
					case ParticlesSuccession.Use: this._successionScale = true; break;
					case ParticlesSuccession.NoUse: this._successionScale = false; break;
					default: break;
				}
			}

			if (this._bPlay && (this._partState.OutCount > 0.0f))
			{
				// 経過時間から発生させるパーティクル数を算出
				this._elapsedTime += fAddTime;

				float fFrameTime = seRootScene.Instance.RootScene.Option.FrameTime;

				// １度だけ再生なら1フレーム分の時間を加算
				if (this._partState.Play == ParticlesPlay.PlayOne)
				{
					this._elapsedTime = fFrameTime;
				}

				glbVector2 bfoPos = this._elem.State.BfoPos;
				glbVector2 nowPos = this._elem.State.Pos.Value;

				float fMstTime = this._elapsedTime;

				// パーティクルが発生するまでの時間
				float fPfs = fFrameTime / (float)this._partState.OutCount;
				// パーティクル発生
				for(;this._elapsedTime >= fPfs; this._elapsedTime -= fPfs)
				{
					float timeLeft = this._elapsedTime - fPfs;

					// 前回の位置からパーティクルの発生位置を線形補正
					glbVector2 offset = nowPos;
					offset.Sub(bfoPos);
					offset.Scale(-(timeLeft / fMstTime));

					// 未使用領域が無い場合は何もせず終了
					p = this.PeekNextUnuse();
					if (p == null) continue;
					// 表示予定のパーティクルが終了している場合も無処理
					if (!p.Start(this._rand, timeLeft, offset)) continue;

					p = this.GetUnuseInstance();

					// 最初の１つの場合
					if (this._lastParticles == null)
					{
						this._particlesList = this._lastParticles = p;
					}
					else
					{
						// 最全面に配置の場合
						if (this._drawPriority == ParticlesDrawPriority.Top)
						{
							this._lastParticles.AddOne(p);
							this._lastParticles = p; 
						}
						// それ以外は最奥に配置
						else
						{
							p.Next = this._particlesList;
							this._particlesList.Befor = p;
							this._particlesList = p;
							p.Befor = null;
						}
					}
				}

				this._bPlay = bNext;
			}
		}
	}


	// パーティクルオブジェクト
	public partial class seoParticles : seObject
	{
		siObjectTextureSprite _sprite = null;

		// カラー
		public glbColor Color { get; set; }

		// サイズ
		private glbVector2 _size = new glbVector2( 10.0f, 10.0f );
		public glbVector2 Size 
		{ 
			get { return this._size; }
			set 
			{
				this._size = value;
				this._sprite.UpdateSize( this._size );
			}
		}

		// UV
		private glbVector2 _uv = glbVector2.Zero;
		public glbVector2 UV
		{
			get { return this._uv ; }
			set 
			{
				this._uv = value;
				this._sprite.UpdateUV( this._uv );
			}
		}

		// リソースID
		private int iResourceID = seResourceManager.InvalidID;
		public int ResourceID
		{
			get { return this.iResourceID; }
			set
			{
				this.iResourceID = value;
				this.bUpdateImage = true;
			}
		}

		// ResourceIDにあわせてテクスチャの更新
		private bool bUpdateImage = false;
		private void UpdateImage()
		{
			if ( !this.bUpdateImage ) return;

			// 無いなら
			if ( this.iResourceID == seResourceManager.InvalidID )
			{
				this.resTexture = null;
			}
			else
			{
				this.resTexture = this.Scene.ResourceManager.Find( this.iResourceID ) as siResourceTexture;

				// リソースはあるが準備できていない場合
				if ( (this.resTexture != null) && !this.resTexture.IsEffective )
				{
					// 更新を終了しない
					this.resTexture = null;
					this._sprite.UpdateTexture( null );
					return;
				}
			}

			this._sprite.UpdateTexture( this.resTexture );
			this.bUpdateImage = false;
		}

		// テクスチャリソース取得
		private siResourceTexture resTexture = null;
		public siResourceTexture ResourceTexture 
		{
			get 
			{
				this.UpdateImage();
				return this.resTexture;
			} 
		}


		public seStateParticlesInfo Info { get; private set; }

		// ステータス設定
		internal void SetState( seStateParticlesInfo state )
		{
			this.Info.Set( state );
		}

		// 描画オブジェクトタイプ
		public override emObjectType ElemType	{ get { return emObjectType.Particles; } }




		// コンストラクタ
		public seoParticles()
		{
			this.Color = glbColor.White;
			this.bUpdateImage = false;
			this.Info = new seStateParticlesInfo();
		}



		// 初期化
		internal override void Init(int iInObjectID, seSceneManager inInScene)
		{
			base.Init(iInObjectID, inInScene);
			// 新しいIDが割り当てられたので作りなおす
			if ( this._sprite != null )
			{
				this._sprite.Dispose();
			}
			this._sprite = seRootScene.Instance.IOManager.CreateTextureSprite( this.ObjectID );
			this._sprite.UpdateTexture( this.ResourceTexture );
			this._sprite.UpdateSize( this._size );
			this._sprite.UpdateUV( this._uv );
		}


		// 描画時に必要なワーククラスを作成
		public override seObjectWork CreateDrawWork ( seElem elem )
		{
			return new seoParticlesWork(elem, this);
		}


		public void Draw(int iElemID, IGLMatrix m, glbColor color)
		{
			this.UpdateImage();
			color.Mul(this.Color);
			this._sprite.Draw(iElemID, m, color);
		}

		public void Draw( int iElemID, IGLMatrix m, IGLMatrix mUV, glbColor color )
		{
			this.UpdateImage();
			color.Mul( this.Color );
			this._sprite.Draw( iElemID, m, color, mUV );
		}


		internal override void Load(IseStateInput input, seSceneManager manager)
		{
			base.Load(input, manager);
			this._size = seIOUtil.ReadVector2( input );
			this._uv = seIOUtil.ReadVector2( input );
			this.ResourceID = (int)input.ReadShort();
			this.Color = seIOUtil.ReadColor( input );
			this.Info.Load(input);
		}

	}
}
