﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using glb;

namespace evc
{

	// シーン描画時に必要なワーク
	public partial class seoSceneWork : seObjectWork
	{
		private seSceneManager _scene = null; // 保持しているシーン
		private seoScene _sceneObj;
		private seElem _elem;

		// 拡張ステータス
		private seStateLoadControl _loadControl;
		private seStateString _sceneAnim; // 本来はIDの方が早い

		// オブジェクト
		public override seObject Object { get { return this._sceneObj; } }

		// ワークのシーン
		public seSceneManager LocalScene { get { return this._scene; }}

		// コンストラクタ
		internal seoSceneWork ( seElem elem, seoScene scene )
		{
			this._elem = elem;
			this._sceneObj = scene;

			this._loadControl = (seStateLoadControl)seStateUtil.Instance.CreateStateType( emElemState.Load );
			this._sceneAnim = (seStateString)seStateUtil.Instance.CreateStateType( emElemState.SceneAnim );

			// ステータスの拡張データをクリア
			this._elem.State.ClearStateData();
			this._elem.State.LinkStateData( emElemState.Load, this._loadControl );
			this._elem.State.LinkStateData( emElemState.SceneAnim, this._sceneAnim );
		}


		// 描画サブルーチン
		// 再生エンジン以外で流量する為に分けてあります。
		private void DrawSub( IGLMatrix m, glbColor c)
		{
			if (this._scene == null) return;

			// アニメーションの要求がないならまだ表示できる段階にない。
			seElemManager elemManager = this._scene.ElemManager;
			if (elemManager.LastAnim == seAnimManager.InvalidID) return;

			// 背景描画
			this._sceneObj.DrawBg( this._elem.ElemID, m, c );

			// inScene.Root.Elemの親マトリクスにデータを設定
			if (elemManager.RootElem != null)
			{
				elemManager.Succession( m );
				elemManager.Draw();
			}
		}

		// 描画	
		internal override void Draw ()
		{
			this.DrawSub(this._elem.NowMatrix, this._elem.State.NowColor);
		}

		// 内部ワークのリセット
		internal override void Reset ()
		{
			if ( this._scene == null ) return;
			this._scene.ElemManager.PlayTime = 0.0f;
			this._scene.ElemManager.ResetAnim();
		}

		// イベントの実行
		internal void DoEvent()
		{
			if ( this._scene == null ) return;
			// アニメーションの要求がないならまだ表示できる段階にない。
			if ( this._scene.ElemManager.LastAnim == seAnimManager.InvalidID ) return;
		
			// ヒット判定は終了しているので、イベントのみ実行
			this._scene.ElemManager.DoEvent();
		}


		// 更新
		internal override void Update ( float fAddTime )
		{
			// ロードフラグが更新されたなら
			seStateData inLoadData = _elem.State.GetData( emElemState.Load );
			if ( inLoadData.MoveKey )
			{
				seResourceScene inResource = null;

				switch ( this._loadControl.State )
				{
					// ロード開始
					case seLoadControl.Load:
						// 既にロード済み
						if ( this._scene != null ) break;
						// シーンのロード開始
						inResource = this._sceneObj.ResourceScene;
						// 自分が帰属するシーンにロード
						inResource.Load( this._sceneObj.Scene );
						break;

					// シーンリリース
					case seLoadControl.Release:
						inResource = this._sceneObj.ResourceScene;
						inResource.Cancel();
						if ( this._scene != null )
						{
							this._scene.Clear();
							this._scene = null;
						}
						break;
					default:
						break;
				}
			}


			// シーン初期化チェック
			if ( this._scene == null )
			{
				// シーンリソースが使用可能なら登録
				seResourceScene inResource = this._sceneObj.ResourceScene;
				if ( inResource.IsEffective )
				{
					this._scene = inResource.Scene.Clone();
					this._scene.Parent = this._sceneObj.Scene;
				}
				// シーンが無い場合はなにもしない。
				else
				{
					return;
				}
			}

			// 更新フラグ
			bool bDoUpdate = false;

			// カラーを継承＆シーンに反映
			seStateData data = _elem.State.GetData( emElemState.SceneAnim );
			if (data != null)
			{
				bDoUpdate |= data.MoveKey;
			}
			
			
			seElemManager lem = this._scene.ElemManager;
			lem.Color = this._elem.State.NowColor;

			// アニメーションの更新があった。
			if ( bDoUpdate )
			{
				seAnimManager inAnimManager = this._scene.AnimManager;
				
				int iAnimID = inAnimManager.FindID( this._sceneAnim.Value );
				lem.SetAnim( iAnimID );
				// アクティブなアニメーションを設定
				inAnimManager.ActiveAnim = iAnimID;
				// 現在シーンが管理している時間へアニメーションを移動
				lem.PlayTime = 0.0f;
				lem.ResetAnim();

				bDoUpdate = false;
			}
			// 通常アニメーション
			else
			{
				this.LocalScene.UpdateAnim();
				lem.Update( this._elem.NowMatrix, fAddTime );
			}
		}


		// オブジェクトのサイズを取得
		public override glbVector2 GetSize ()
		{
			seResourceScene sr = this._sceneObj.ResourceScene;
			return (sr == null) ? seoScene.sDefaultSize
								: sr.Scene.Option.ScreenSize;
		}
	}


	/**
	 * シーンを再帰的に呼び出す用スプライト
	 */
	public partial class seoScene : seObject
	{
		internal static readonly glbVector2	sDefaultSize = new glbVector2( 10.0f, 10.0f );

		private siObjectColorSprite _colorSprite = null;	// ＢＧ描画用

		public glbColor sBgColor = new glbColor( 0,0,0,255 ); 

		// リソースの入出力
		private int iResourceID = seResourceManager.InvalidID;
		public int ResourceID 
		{
			get { return this.iResourceID; }
			set 
			{ 
				this.iResourceID = value;
				this._targetScene = null;
			}
		}

		// シーンキャッシュ：このメンバを直接触ることは控えて下さい。
		// CreateDrawWorkで作成したシーンを操作して下さい。
		private seResourceScene _targetScene = null;
		public seResourceScene ResourceScene
		{
			get 
			{
				if ( this._targetScene == null ) 
				{
					// リソースの取得
					seResourceScene inSceneResource = (seResourceScene)base.Scene.ResourceManager.Find( this.iResourceID );
					if ( inSceneResource == null ) return null;
					if ( !inSceneResource.IsEffective ) return null;
					this._targetScene = inSceneResource;
				}
				return this._targetScene;
			}
		}


		public seoScene ()
		{
		}


		// 描画オブジェクトタイプ
		public override emObjectType ElemType {get { return emObjectType.Scene; }}


		// 描画時に必要なワーククラスを作成
		public override seObjectWork CreateDrawWork ( seElem elem )
		{
			return new seoSceneWork( elem, this );
		}

		// 背景描画
		internal void DrawBg( int iElemID, IGLMatrix m, glbColor sMulColor )
		{
			// 背景スプライトが作成されていない場合のみ
			if ( this._colorSprite == null )
			{
				seResourceScene inResource = this.ResourceScene;
				if ( inResource == null ) return;

				seOption inOption = inResource.Scene.Option;
				this._colorSprite = seRootScene.Instance.IOManager.CreateColorSprite( this.ObjectID );
				this._colorSprite.UpdateSize( inOption.ScreenSize );
				this.sBgColor = inOption.BgColor;
			}

			sMulColor.Mul( this.sBgColor );
			this._colorSprite.Draw( iElemID, m, sMulColor );
		}


		internal override void Dispose ()
		{
		}

		internal override void Load(IseStateInput inInput, seSceneManager inManager)
		{
			base.Load(inInput, inManager);
			this.sBgColor = seIOUtil.ReadColor( inInput );
			this.ResourceID = (int)inInput.ReadShort();
		}



	}
}
