﻿using glb;

namespace evc
{

	// サウンド実行時に必要なワーク
	internal partial class seoSoundWork : seObjectWork
	{
		private siObjectSound		inObjectSound = null;
		// サウンド属性		
		private bool				bDoUpdate = false;			// 更新フラグ
		private seSoundControl		eSoundState = seSoundControl.Non;

		private seElem _elem;
		private seoSound _obj;

		// 拡張ステータス
		private seStateInt1 inVolume;
		private seStateInt1 inPan;
		private seStateSoundControl inSoundControl;


		public override seObject Object { get { return this._obj; } }

		// オブジェクトのサイズを取得
		public override glbVector2 GetSize () { return seoSound.Size.Value; }

		// コンストラクタ
		internal seoSoundWork( seElem elem, seoSound obj )
		{
			this._elem = elem;
			this._obj = obj;

			this.inVolume = (seStateInt1)seStateUtil.Instance.CreateStateType( emElemState.Volume );
			this.inPan = (seStateInt1)seStateUtil.Instance.CreateStateType( emElemState.Pan );
			this.inSoundControl = (seStateSoundControl)seStateUtil.Instance.CreateStateType( emElemState.SoundControl );

			// ステータスの拡張データをクリア
			this._elem.State.ClearStateData();
			this._elem.State.LinkStateData( emElemState.Volume, this.inVolume );
			this._elem.State.LinkStateData( emElemState.Pan, this.inPan );
			this._elem.State.LinkStateData( emElemState.SoundControl, this.inSoundControl );
		}

		// 内部ワークをリセット
		internal override void Reset ()
		{
			this.bDoUpdate = false;
			this.eSoundState = seSoundControl.Non;
			if ( this.inObjectSound != null )
			{
				this.inObjectSound.Stop();
			}
		}

		// サウンド鳴らす
		private void PlaySound( bool bLoop )
		{
			if ( this._obj.SoundTime >= 0.0f )
			{
				this.inObjectSound.Play( this._elem.ElemID, this._obj.SoundStart, this._obj.SoundTime, bLoop );
			}
			else
			{
				this.inObjectSound.Play( this._elem.ElemID, this._obj.SoundStart, bLoop );
			}
		}


		// 描画	
		internal override void Draw ()
		{
			if ( this.inObjectSound == null )
			{
				this.inObjectSound = seRootScene.Instance.IOManager.CreateSound( 0, this._obj.ResourceSound );
			}

			// ボリューム設定
			int iVolume = this.inVolume.Value;
			int iPan = this.inPan.Value;

			this.inObjectSound.UpdateVolume( iVolume );
			this.inObjectSound.UpdatePan( iPan );

			// 再生モードでしか発音しないようにしよう。
			if (!seRootScene.Instance.EditMode && this.bDoUpdate)
			{
				// 状態別処理
				switch ( this.eSoundState )
				{
					// 停止中に再生が来たなら。
					case seSoundControl.Play:	  this.PlaySound( false ); break;
					case seSoundControl.PlayLoop: this.PlaySound( true ); break;
					// 再生中に停止がきたなら
					case seSoundControl.Stop:	  this.inObjectSound.Stop(); break;
					// 無処理
					case seSoundControl.Non: break;
				}

				this.bDoUpdate = false;
				this.eSoundState = seSoundControl.Non;
			}

			this.inObjectSound.Draw( this._elem.NowMatrix, iVolume, iPan );
									
		}

		// 更新
		internal override void Update ( float fAddTime ) 
		{
			seStateData data = _elem.State.GetData( emElemState.SoundControl );
			// 新しい状態になったなら
			if ( (data != null) && (data.MoveKey) )
			{
				this.bDoUpdate = true;
				this.eSoundState = this.inSoundControl.State;
			}
		}
	}


	// サウンド定義
	public partial class seoSound : seObject
	{
		// サンプル描画用
		public static readonly seStateFloat2 Size = new seStateFloat2( new glbVector2( 32.0f, 32.0f ) );
		public static readonly seStateColor Color = new seStateColor( 255,255,255,255 );
		
		private int iResourceID = seResourceManager.InvalidID;
		public int ResourceID 
		{ 
			get { return this.iResourceID; }
			set 
			{ 
				this.iResourceID = value; 
				this.inResourceSound = null;
			}
		}

		// サウンドリソース取得
		private siResourceSound inResourceSound = null;
		public siResourceSound ResourceSound
		{
			get 
			{ 
				if ( this.inResourceSound == null )
				{
					this.inResourceSound = (siResourceSound)this.Scene.ResourceManager.Find( this.iResourceID );
				}
				return this.inResourceSound;
			}
		}

		private float fSoundStart = 0.0f;
		public float SoundStart { get { return this.fSoundStart; } set { this.fSoundStart = value; }}

		// 
		private float fSoundTime = 0.0f;
		public float SoundTime { get { return this.fSoundTime; } set { this.fSoundTime = value; } }


		// 描画オブジェクトタイプ
		public override emObjectType ElemType		{get { return emObjectType.Sound; }}

		// 描画時に必要なワーククラスを作成
		public override seObjectWork CreateDrawWork ( seElem inElem )
		{
			return new seoSoundWork( inElem, this );
		}

		internal override void Load(IseStateInput inInput, seSceneManager inManager)
		{
			base.Load(inInput, inManager);
			this.ResourceID = (int)inInput.ReadShort();
			this.fSoundStart = inInput.ReadFloat();
			this.fSoundTime = inInput.ReadFloat();
		}

	}
}
