﻿using System;
using glb;

using System.Diagnostics;


namespace evc
{
	// テクスチャースプライト描画時に必要なワーク
	internal partial class seoTextureSpriteWork : seObjectWork
	{
		private seoTextureSprite _textureObj; // 描画オブジェクト
		private seElem _elem; // エレメント

		// 次の描画テクスチャ
		private seoTextureSprite _drawTexture = null;

		// 描画ワーク
		private IGLMatrix _mUV = null;
		private IGLMatrix[] ainUV = new IGLMatrix[ 4 ];
		private glbVector2 _uvOffset = glbVector2.Zero;

		// 拡張ステータス
		private seStateSwitchInt1 _texture;
		private seStateTextureControl _textureControl;
		private seStateLoop _loop;
		private seStateFloat2 _uv;

		public override seObject Object { get { return this._textureObj; } }

		// 描画するテクスチャオブジェクトを取得
		private seoTextureSprite TextureObject
		{
			get { return (this._drawTexture == null) ? this._textureObj : this._drawTexture; }
		}

		// オブジェクトのサイズを取得
		public override glbVector2 GetSize ()
		{
			return this.TextureObject.Size;
		}

		// コンストラクタ
		internal seoTextureSpriteWork ( seElem inInElem, seoTextureSprite inInObjct )
		{
			this._elem = inInElem;
			this._textureObj = inInObjct;
			this._mUV = seRootScene.Instance.IOManager.CreateMatrix();

			this._texture = (seStateSwitchInt1)seStateUtil.Instance.CreateStateType( emElemState.Texture );
			this._textureControl = (seStateTextureControl)seStateUtil.Instance.CreateStateType( emElemState.TextureControl );
			this._loop = (seStateLoop)seStateUtil.Instance.CreateStateType( emElemState.Loop );
			this._uv = (seStateFloat2)seStateUtil.Instance.CreateStateType( emElemState.UV );

			// ステータスの拡張データをクリア
			this._elem.State.ClearStateData();
			this._elem.State.LinkStateData( emElemState.Texture, this._texture );
			this._elem.State.LinkStateData( emElemState.TextureControl, this._textureControl );
			this._elem.State.LinkStateData( emElemState.Loop, this._loop );
			this._elem.State.LinkStateData( emElemState.UV, this._uv );
		}

		// 内部ワークのリセット
		internal override void Reset()
		{
			this._drawTexture = null;
		}


		// 描画サブルーチン
		// 再生エンジン以外で使用する為、Drawと処理を切り離してあります。
		private void DrawSub ( IGLMatrix mBase, glbColor sColor )
		{
			seoTextureSprite inTextureObject = this.TextureObject;
			int iElemID = this._elem.ElemID;

			// ループがないなら
			if ( (this._loop.LoopCountH == 1) &&
				 (this._loop.LoopCountW == 1) )
			{
				inTextureObject.Draw( iElemID, mBase, this._mUV, sColor );
			}
			// ループがあるなら
			else
			{
				seMatrixWork.Instance.UseMatrixWork( delegate( IUseWorkMatrix gw )
				{
					glbVector2 sObjectSize = this._textureObj.Size;

					IGLMatrix m = gw.GetMatrix();
					IGLMatrix ms = gw.GetMatrix();

					int iHeightNum = Math.Abs(this._loop.LoopCountH);
					int iWidthNum = Math.Abs(this._loop.LoopCountW);
					
					float fAddW = (float)((this._loop.LoopCountW >= 0) ? sObjectSize.X : -sObjectSize.X);
					float fAddH = (float)((this._loop.LoopCountH >= 0) ? sObjectSize.Y : -sObjectSize.Y);


					int iHSwitchID = 0;
					int iVSwitchID = 0;

					// 反転用ＵＶマトリクス
					this.ainUV[ 0 ] = this._mUV;
					this.ainUV[ 1 ] = gw.GetMatrix();
					this.ainUV[ 2 ] = gw.GetMatrix();
					this.ainUV[ 3 ] = gw.GetMatrix();

					IGLMatrix mWork = gw.GetMatrix();

					// 左右反転
					if ( this._loop.RevW )
					{
						iHSwitchID = 1;
						ainUV[ 1 ].SetScale( -1.0f, 1.0f );
						mWork.SetUV( (float)this._uvOffset.X, 0.0f );
						ainUV[ 1 ].MulR( mWork );
						ainUV[ 1 ].MulR( this._mUV );
					}

					// 上下反転
					if ( this._loop.RevH )
					{
						iVSwitchID = 2;
						ainUV[ 2 ].SetScale( 1.0f, -1.0f );
						mWork.SetUV( 0.0f, (float)this._uvOffset.Y );
						ainUV[ 2 ].MulR( mWork );
						ainUV[ 2 ].MulR( this._mUV );

						// 上下左右反転
						if ( this._loop.RevW )
						{
							ainUV[ 3 ].SetScale( -1.0f, -1.0f );
							mWork.SetUV( this._uvOffset );
							ainUV[ 3 ].MulR( mWork );
							ainUV[ 3 ].MulR( this._mUV );
						}
					}


					int iVSwitch = 0;
					for (int i = 0; i < iHeightNum; i++)
					{
						int iHSwitch = 0;
						for (int j = 0; j < iWidthNum; j++)
						{
							m.SetPos( j * fAddW, i * fAddH );
							m.MulR( mBase );
							inTextureObject.Draw( iElemID, m, ainUV[ iVSwitch | iHSwitch ], sColor );
							iHSwitch ^= iHSwitchID;
						}
						iVSwitch ^= iVSwitchID;
					}
				});
			}
		}

		// 描画	
		internal override void Draw ()
		{
			this.DrawSub(this._elem.NowMatrix, this._elem.State.NowColor);
		}


		// 更新
		internal override void Update ( float fAddTime )
		{
			// テクスチャ更新チェック
			seStateData inTextureData = _elem.State.GetData( emElemState.Texture );
			if ( inTextureData.MoveKey )
			{
				int iValue = this._texture.Value;
				this._drawTexture = (iValue == seObjectManager.InvalidID) ?
						null :
						this._textureObj.Scene.ObjectManager.Find( iValue ) as seoTextureSprite;
			}

			// ＵＶマトリクスを作成
			this._mUV.SetUV( this._uv.Value );

			// UVのサイズを算出
			siResourceTexture inImage = this.TextureObject.ResourceTexture;
			
			if ( inImage == null )
			{
				this._uvOffset = glbVector2.Zero;
			}
			else
			{
				glbVector2 sRate = this._textureObj.Size;
				sRate.Div( new glbVector2( inImage.TextureSize ) );
				this._uvOffset = this._textureObj.UV;
				this._uvOffset.Scale( 2.0f );
				this._uvOffset.Add( sRate );
			}
		}
	}


	// テクスチャ描画
	public partial class seoTextureSprite : seObject
	{
		siObjectTextureSprite _sprite = null;

		// カラー
		public glbColor Color { get; set; }

		// サイズ
		private glbVector2 _size = new glbVector2( 10.0f, 10.0f );
		public glbVector2 Size 
		{ 
			get { return this._size; }
			set 
			{
				this._size = value;
				this._sprite.UpdateSize( this._size );
			}
		}

		// UV
		private glbVector2 _uv = glbVector2.Zero;
		public glbVector2 UV
		{
			get { return this._uv ; }
			set 
			{
				this._uv = value;
				this._sprite.UpdateUV( this._uv );
			}
		}

		// リソースID
		private int iResourceID = seResourceManager.InvalidID;
		public int ResourceID
		{
			get { return this.iResourceID; }
			set
			{
				this.iResourceID = value;
				this.bUpdateImage = true;
			}
		}

		// ResourceIDにあわせてテクスチャの更新
		private bool bUpdateImage = false;
		private void UpdateImage()
		{
			if ( !this.bUpdateImage ) return;

			// 無いなら
			if ( this.iResourceID == seResourceManager.InvalidID )
			{
				this.resTexture = null;
			}
			else
			{
				this.resTexture = this.Scene.ResourceManager.Find( this.iResourceID ) as siResourceTexture;

				// リソースはあるが準備できていない場合
				if ( (this.resTexture != null) && !this.resTexture.IsEffective )
				{
					// 更新を終了しない
					this.resTexture = null;
					this._sprite.UpdateTexture( null );
					return;
				}
			}

			this._sprite.UpdateTexture( this.resTexture );
			this.bUpdateImage = false;
		}


		// テクスチャリソース取得
		private siResourceTexture resTexture = null;
		public siResourceTexture ResourceTexture 
		{
			get 
			{
				this.UpdateImage();
				return this.resTexture;
			} 
		}



		public seoTextureSprite()
		{
			this.Color = glbColor.White;
			this.bUpdateImage = false;
		}



		// 初期化
		internal override void Init(int iInObjectID, seSceneManager inInScene)
		{
			base.Init(iInObjectID, inInScene);
			// 新しいIDが割り当てられたので作りなおす
			if ( this._sprite != null )
			{
				this._sprite.Dispose();
			}
			this._sprite = seRootScene.Instance.IOManager.CreateTextureSprite( this.ObjectID );
			this._sprite.UpdateTexture( this.ResourceTexture );
			this._sprite.UpdateSize( this._size );
			this._sprite.UpdateUV( this._uv );
		}

		// 描画オブジェクトタイプ
		public override emObjectType ElemType	{ get { return emObjectType.Texture; } }


		// 描画時に必要なワーククラスを作成
		public override seObjectWork CreateDrawWork ( seElem inElem )
		{
			return new seoTextureSpriteWork( inElem, this );
		}


		public void Draw(int iElemID, IGLMatrix m, glbColor sMulColor)
		{
			this.UpdateImage();
			sMulColor.Mul(this.Color);
			this._sprite.Draw(iElemID, m, sMulColor);
		}

		public void Draw( int iElemID, IGLMatrix m, IGLMatrix mUV, glbColor color )
		{
			this.UpdateImage();
			color.Mul( this.Color );
			this._sprite.Draw( iElemID, m, color, mUV );
		}


		internal override void Load(IseStateInput input, seSceneManager manager)
		{
			base.Load(input, manager);
			this._size = seIOUtil.ReadVector2( input );
			this._uv = seIOUtil.ReadVector2( input );
			this.ResourceID = (int)input.ReadShort();
			this.Color = seIOUtil.ReadColor( input );
		}

	}
}
