﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using glb;

namespace evc
{
	public interface IseObjResourceTexture { siResourceTexture ResourceTexture { get; } }
	
	/// <summary>
	/// 要素オブジェクトクラス
	/// (主に描画物管理クラス)
	/// seElemObjectを消す時はリンクしているIElemObjectWorkも全てseElemから削除しないとまずい。
	/// </summary>
	public abstract partial class seObject
	{
		// 描画オブジェクトタイプ
		public abstract emObjectType ElemType { get; }

		// オブジェクトＩＤ取得
		public int ObjectID { get; private set; }

		// 自分の親のシーンマネージャ取得
		public seSceneManager Scene { get; private set; }

		// コンストラクタ
		public seObject()
		{
			this.ObjectID = seObjectManager.InvalidID;
		}


		// 内部初期化オブジェクトＩＤが確定した後に１度だけ呼ばれます。
		internal virtual void Init( int objID, seSceneManager scene )
		{
			this.ObjectID = objID;
			this.Scene = scene;
		}

		// 描画時に必要なワーククラスを作成
		public abstract seObjectWork CreateDrawWork ( seElem inElem );

		// 解放
		internal virtual void Dispose() {}

		// ロード
		internal virtual void Load( IseStateInput input, seSceneManager scene )
		{
			this.ObjectID = input.ReadShort();
		}

	
		/**
		 * 要素オブジェクトを作成(オブジェクトＩＤ指定版)
		 */
		public static seObject Create( emObjectType eElemType )
		{
			switch (eElemType)
			{
				// 該当オブジェクトなし
				case emObjectType.Null: return null;
				// カラースプライト
				case emObjectType.Color: return new seoColorSprite();
				// テクスチャスプライト
				case emObjectType.Texture: return new seoTextureSprite();
				// シーン
				case emObjectType.Scene: return new seoScene();
				// サウンド
				case emObjectType.Sound: return new seoSound();
				// 文字列
				case emObjectType.Font: return new seoFont();
				// 数値
				case emObjectType.Num: return new seoNum();
				// 枠
				case emObjectType.Frame: return new seoFrame();
				// メシュ
				case emObjectType.Mesh: return new seoMesh();
				// メッシュ制御点
				case emObjectType.MeshChild: return new seoMeshChild();
				// グリッド
				case emObjectType.Grid: return new seoGrid();
				// パーティクル
				case emObjectType.Particles : return new seoParticles();
				// その他ならアサート
				default: break;
			}
			Debug.Assert(false);
			return null;
		}
	
	}
}
