﻿using System.Diagnostics;
using glb;

namespace evc
{
	// テクスチャイメージ管理
	
	public class seResourceScene : siResourceBase
	{
		// シーン取得
		public seSceneManager Scene { get; private set; }

		// コンストラクタ(シーンコピー)
		public seResourceScene( string strFileName, seSceneManager inScene )
			: base( strFileName )
		{
			this.Scene = inScene;
			this.State = siResourceState.Effective;
		}
		// コンストラクタ(バイナリファイルロード)
		public seResourceScene( string strFileName )
			: base( strFileName ){}


		// リソースタイプ
		public override seResourceType Type	{get{return seResourceType.Scene;}}


		// 初期読み込みをするリソースに対してのみ処理を実行
		private void DoFirstLoadAll( glbAction<siResourceBase> func )
		{
			// シーンで使うテクスチャもロード
			this.Scene.ResourceManager.DoAll( delegate( int iResourceID, siResourceBase resource )
			{
				// テクスチャか、シーンリソースで事前読み込みの場合
				if ( (resource is siResourceTexture) || 
					 ((resource is seResourceScene) && resource.IsAutoLoad ) )
				{
					func( resource );
				}
				return true;
			});
		}


		// シーンを初期化
		public override siResourceInitType Init( byte[] abData )
		{
			// まだシーンが初期化されていないなら
			if ( this.Scene == null )
			{
				this.Scene = new seSceneManager( base.Name );
				bnInput inReader = new bnInput();
				inReader.Open( abData );
				// デバッグ表示宣言
				seDefine.StartDebugTrace( inReader.Stream );
				this.Scene.Load( inReader );
				inReader.Close();

				// 初期読み込みするリソースに対して処理を実行
				this.DoFirstLoadAll( delegate( siResourceBase inResource )
				{
					inResource.Load( this.Scene );
				});
			}

			// シーンの作成に失敗している場合はエラー
			if ( this.Scene == null ) return siResourceInitType.Failure;


			// シーンで使うテクスチャのロード終了チェック
			bool bInited = true;
			bool bErr = false;
			// 初期読み込みするリソースに対して処理を実行
			this.DoFirstLoadAll( delegate( siResourceBase inResource )
			{
				inResource.Update();

				// 初期化が終わっていないものがあるなら
				if ( !inResource.IsEffective )
				{
					bInited = false;
					// リソースの登録に失敗したなら
					if ( inResource.State == siResourceState.Invalidity )
					{
						bErr = true;
					}
				}
			});

			// シーンと全てのテクスチャがロード済の状態なら
			if ( bInited )
			{
				// シーンにテクスチャを登録
				this.Scene.ObjectManager.DoAll( delegate( int iObjectID, seObject inObject )
				{
					if ( !(inObject is seoTextureSprite) ) return true;
					// テクスチャの内部状態を更新
					((seoTextureSprite)inObject).ResourceID = ((seoTextureSprite)inObject).ResourceID;
					return true;
				});
			}


			// １つでもリソースの初期化に失敗たならシーンの登録失敗
			if ( bErr ) return siResourceInitType.Failure;
			return (bInited) ? siResourceInitType.Success
							 : siResourceInitType.Continuation;
		}

		// 現在のシーンが使用可能か否か
		public bool IsEnableScene
		{
			get	
			{
				if ( this.Scene == null ) return false;

				// シーン内のテクスチャが全て使用可能ならtrueそれ以外はfalse				
				return this.Scene.ResourceManager.DoAll( delegate( int iResourceID, siResourceBase inResource)
				{
					if ( inResource is siResourceTexture )
					{
						if( !inResource.IsEffective ) return false;
					}
					return true;
				});
			}
		}


		// シーンを譲渡する。
		// 渡したシーンはこのクラスでは管理しなくなります。
		// seResourceSceneをDisposeしてもシーンを保持していたい時に使用して下さい。
		internal seSceneManager PassesScene()
		{
			// 初期化が済んでいない場合は譲渡不可。
			if ( !this.IsEffective )
			{
				Debug.Assert(false );
				return null;
			}

			seSceneManager inRetScene = this.Scene;
			this.Scene = null;

			return inRetScene;
		}

		// リソース解放
		public override void Dispose()
		{
			if ( this.Scene != null ) 
			{
				this.Scene.Clear();
			}
		}
	}
}
