﻿
namespace glb
{
	// glb 用カラー構造体
	public struct glbColor
	{
		public byte	R;
		public byte	G;
		public byte	B;
		public byte	A;

		// 32bitのARGB配列を取得
		public int ARGB
		{
			get
			{
				return  ((int)this.A << 24) |
						((int)this.R << 16) | 
						((int)this.G <<  8) |
						((int)this.B <<  0);
			}
		}
		// 32bitのRGBA配列を取得
		public int RGBA
		{
			get
			{
				return  ((int)this.R << 24) |
						((int)this.G << 16) | 
						((int)this.B <<  8) |
						((int)this.A <<  0);
			}
		}

		// コンストラクタ
		public glbColor ( byte R, byte G, byte B, byte A )
		{
			this.R = R;
			this.G = G;
			this.B = B;
			this.A = A;
		}
		public glbColor ( glbColor sColor )
		{
			this.R = sColor.R;
			this.G = sColor.G;
			this.B = sColor.B;
			this.A = sColor.A;
		}
		public glbColor ( int iRGBA )
		{
			this.R = (byte)((iRGBA >> 24) & 0x000000ff);
			this.G = (byte)((iRGBA >> 16) & 0x000000ff);
			this.B = (byte)((iRGBA >>  8) & 0x000000ff);
			this.A = (byte)(iRGBA & 0x000000ff);
		}


		// 値を設定
		public void Set(  byte R, byte G, byte B, byte A )
		{
			this.R = R;
			this.G = G;
			this.B = B;
			this.A = A;
		}	
		public void Set( glbColor sColor )
		{
			this.R = sColor.R;
			this.G = sColor.G;
			this.B = sColor.B;
			this.A = sColor.A;
		}	

		// カラー掛け算
		public void Mul( glbColor sColor )
		{
			this.R = (byte)(((int)this.R * sColor.R) / 255 );
			this.G = (byte)(((int)this.G * sColor.G) / 255 );
			this.B = (byte)(((int)this.B * sColor.B) / 255 );
			this.A = (byte)(((int)this.A * sColor.A) / 255 );
		}

		// 白
		public static readonly glbColor White = new glbColor(255,255,255,255);
	}
}
