﻿
namespace glb
{
	// 二次元ベクトル
	public struct glbVector2
	{
#if ENABLE_DOUBLE_VECTOR
		public double X;
		public double Y;
#else
		public float X;
		public float Y;
#endif
		// コンストラクタ
		public glbVector2( float fX, float fY )
		{
			this.X = (float)fX;
			this.Y = (float)fY;
		}
		public glbVector2( glbVector2I v )
		{
			this.X = (float)v.X;
			this.Y = (float)v.Y;
		}
		
		public void Set( float fX, float fY )
		{
			this.X = (float)fX;
			this.Y = (float)fY;
		}
		public void Set( glbVector2 sV )
		{
			this.X = sV.X;
			this.Y = sV.Y;
		}
		
		// 四則演算
		public void Sub( glbVector2 sVector2 )
		{
			this.X -= sVector2.X;
			this.Y -= sVector2.Y;
		}
		public void Add( glbVector2 sVector2 )
		{
			this.X += sVector2.X;
			this.Y += sVector2.Y;
		}
		public void Mul( glbVector2 sVector2 )
		{
			this.X *= sVector2.X;
			this.Y *= sVector2.Y;
		}
		public void Div( glbVector2 sVector2 )
		{
			this.X /= sVector2.X;
			this.Y /= sVector2.Y;
		}

		// スケール
		public void Scale( float fScale )
		{
			this.X *= fScale;
			this.Y *= fScale;
		}

#if SILVERLIGHT
		public glbVector2( double fX, double fY )
		{
			this.X = fX;
			this.Y = fY;
		}
		public void Set( double fX, double fY )
		{
			this.X = fX;
			this.Y = fY;
		}
#endif 
	
		public static readonly glbVector2 Zero = new glbVector2( 0.0f, 0.0f );
		public static readonly glbVector2 One = new glbVector2( 1.0f, 1.0f );
	}


	// 二次元ベクトル(整数)
	public struct glbVector2I
	{
		public int X;
		public int Y;

		// コンストラクタ
		public glbVector2I( int iX, int iY )
		{
			this.X = iX;
			this.Y = iY;
		}
		
		public void Set( int iX, int iY )
		{
			this.X = iX;
			this.Y = iY;
		}
		public void Set( glbVector2I sV )
		{
			this.X = sV.X;
			this.Y = sV.Y;
		}
		
		// 四則演算
		public void Sub( glbVector2I sVector2 )
		{
			this.X -= sVector2.X;
			this.Y -= sVector2.Y;
		}
		public void Add( glbVector2I sVector2 )
		{
			this.X += sVector2.X;
			this.Y += sVector2.Y;
		}

		public static readonly glbVector2I Zero = new glbVector2I( 0, 0 );
		public static readonly glbVector2I One = new glbVector2I( 1, 1 );
	}
}
