﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using evc;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using evcIO;
using glb;


namespace evcPlayer
{
	internal partial class MainForm : Form
	{
		private seRootScene inScene = null;
		private trIOManager inIOManager = null;
		private dx_control inDxControl = null;

		// 背景色を取得
		public int ARGB 
		{ 
			get { return (this.inScene == null) ? 0 : this.inScene.BgColor.ARGB; }
		}



		// 初期化時
		public void Init( dx_control inInDxControl )
		{
			this.inDxControl = inInDxControl;
	
			// evcの初期化
			this.inIOManager = new trIOManager( this.inDxControl.DxDevice, this );
			trEventIO inEventIO = new trEventIO();
			inEventIO.SetTarget( this );
			this.inScene = new seRootScene( this.inIOManager, inEventIO, new trMaterial() );
			this.inScene.EventListener = new EventListener();

			this.inScene.OnInit += new glb.glbAction<seSceneManager>(inScene_OnInit);
#if DEBUG
			// テスト
//			this.ReqPlay( "../../data/loop.pra", "default" );
//			this.ReqPlay( "../../data/num_tes.pra", "default" );
#endif	
		}


		// 初期化時
		void inScene_OnInit(seSceneManager inSceneManager )
		{
			glbVector2 sSize = inSceneManager.Option.ScreenSize;
			this.ClientSize = new Size( (int)sSize.X, (int)sSize.Y );
			this.OnResizeEnd( new EventArgs() );
		}


		// 毎フレーム処理
		public void Proc( int iAddTime )
		{
			if ( this.inScene == null ) return;

			// 入力処理更新
			this.inScene.UpdateInput();

			// アニメーションの実行
			this.inScene.DoAnim(iAddTime);

			// 要素の描画
			this.inScene.Draw();
		}


		// 再生リクエスト
		public void ReqPlay( string strFile, string strAnimName )
		{
			if ( this.inScene == null ) return;

			// praファイルがある位置をカレントディレクトリとする
			string strPath = Path.GetDirectoryName( strFile );
			if ( strPath.Length > 0 )
			{
				Environment.CurrentDirectory = strPath;
			}


			// 直前のデータを初期化
			this.inScene.Clear();
			// データのロード
			this.inScene.Load( Path.GetFileName( strFile ), strAnimName );
		}




		// コンストラクタ
		public MainForm()
		{
			InitializeComponent();
			this.Disposed += new EventHandler(MainForm_Disposed);
		}


		// 終了時
		void MainForm_Disposed(object sender, EventArgs e)
		{
			if ( this.inScene != null )
			{
				this.inScene.Clear();
				this.inScene = null;
			}
		}

		// ドラッグ＆ドロップ完了時
		private void MainForm_DragDrop(object sender, DragEventArgs e)
		{
			if ( !e.Data.GetDataPresent(DataFormats.FileDrop)) return;

			string[] strFiles = (string[])e.Data.GetData(DataFormats.FileDrop);

			string strPath = Path.GetDirectoryName( strFiles[ 0 ] );
			string strFile = Path.GetFileName( strFiles[ 0 ] );
			Environment.CurrentDirectory = strPath;

			this.ReqPlay( strFile, "default" );
		}

		// ドラッグ＆ドロップ中
		private void MainForm_DragEnter(object sender, DragEventArgs e)
		{
						// ファイル以外は無効
			if ( !e.Data.GetDataPresent(DataFormats.FileDrop) ) goto none;

			// .htmlファイル以外は無効
			string[] strFiles = (string[])e.Data.GetData(DataFormats.FileDrop);
			string strExt = Path.GetExtension( strFiles[ 0 ] );
			if ( strExt != ".pra" ) goto none;

			e.Effect = DragDropEffects.Move;
			return ;
none:
			e.Effect = DragDropEffects.None;

		}


	}
}
