using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Diagnostics;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace evcPlayer
{
	// DirectXRg[T|[g
	// 

	// foCX
	internal struct DeviceInfo
	{
		public readonly DeviceType	Type;
		public readonly CreateFlags Flags;

		// RXgN^
		public DeviceInfo ( DeviceType eInType, CreateFlags eInFlags )
		{
			this.Type = eInType;
			this.Flags = eInFlags;
		}
	}

	internal class dx_control : IDisposable
	{
		private static Device inDevice = null;
		private static PresentParameters inParams = new PresentParameters();
		private static int iRefCount = 0;


		private Control inTarget = null;
		private SwapChain inSwapChain = null;
		private Surface	inZBuff = null;
		private Surface	inSurface = null;


		#region DirectXʏ

		// DirextX
		private static Device Init(Control inTarget)
		{
			inParams.Windowed = true;
			inParams.PresentFlag = PresentFlag.None;
			inParams.BackBufferCount = 1;
			inParams.BackBufferFormat = Format.Unknown;
			inParams.SwapEffect = SwapEffect.Discard;
			inParams.EnableAutoDepthStencil = true;
			inParams.AutoDepthStencilFormat = DepthFormat.D16;
			inParams.PresentationInterval = PresentInterval.One;

			// Direct3D foCX̍쐬
			DeviceInfo[] asDeviceInfo = new DeviceInfo[]
			{
				new DeviceInfo( DeviceType.Hardware, CreateFlags.HardwareVertexProcessing ),
				new DeviceInfo( DeviceType.Hardware, CreateFlags.SoftwareVertexProcessing ),
				new DeviceInfo( DeviceType.Reference, CreateFlags.SoftwareVertexProcessing ),
			};

			foreach (DeviceInfo inInfo in asDeviceInfo)
			{
				try
				{
					dx_control.inDevice = new Device(0, inInfo.Type,
								inTarget, inInfo.Flags, inParams);
					break;
				}
				catch (DirectXException e)
				{
					Trace.Write(e.ErrorString + "\n");
				}
			}

			if (dx_control.inDevice == null)
			{
				MessageBox.Show("foCX̍쐬Ɏs܂", "G[",
						   MessageBoxButtons.OK, MessageBoxIcon.Error);
				return null;
			}

			// J̐ݒ
			dx_control.inDevice.Transform.Projection = Matrix.OrthoOffCenterLH(
									   0,
									   inTarget.ClientSize.Width,
									   inTarget.ClientSize.Height,
									   0,
									   0.0f,
									   10.0f);
			dx_control.inDevice.Transform.View = Matrix.LookAtLH(
								new Vector3(0.0f, 0.0f, 0.0f),
								new Vector3(0.0f, 0.0f, 1.0f),
								new Vector3(0.0f, 1.0f, 0.0f));

			// CgOFF
			dx_control.inDevice.RenderState.Lighting = false;

			dx_control.inDevice.DeviceResizing += new System.ComponentModel.CancelEventHandler(inDevice_DeviceResizing);

			return dx_control.inDevice;
		}

		// TCYύX(DirectXʒm)
		private static void inDevice_DeviceResizing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			e.Cancel = true;
		}

		// foCX̍č\z
		private static void DeviceReset()
		{
			Device d = dx_control.inDevice;

			int result;
			// Zbg\ǂ`FbN
			if (!d.CheckCooperativeLevel(out result))
			{
				switch ((ResultCode)result)
				{
					// Zbg\Ȃ烊Zbg
					case ResultCode.DeviceNotReset:
						d.Reset(dx_control.inParams);
						break;

					// ܂ZbgłȂ΁A΂炭X[v
					case ResultCode.DeviceLost:
						System.Threading.Thread.Sleep(20);
						break;
					// ȊO
					default:
						Debug.Assert(false);
						break;
				}
			}
		}

		#endregion


		// foCX̎擾
		public Device DxDevice { get { return dx_control.inDevice; } }



		// RXgN^
		public dx_control( Form inInTarget )
		{
			// ΃foCX̏
			if ( dx_control.inDevice == null )
			{
				dx_control.Init( inInTarget );
			}
			// QƃJE^𑝂₷
			dx_control.iRefCount ++;

			this.inTarget = inInTarget;
			inInTarget.ResizeEnd += new EventHandler(inInTarget_ResizeEnd);
		}


		private void inInTarget_ResizeEnd(object sender, EventArgs e)
		{
			this.Reset();

			// vWFNVJ̍Đݒ
			Size sClientSize = this.inTarget.ClientSize;
			dx_control.inDevice.Transform.Projection = Matrix.OrthoOffCenterLH(
			                           0,
									   sClientSize.Width,
									   sClientSize.Height,
			                           0,
			                           0.0f,
			                           10.0f );
		}

		// obt@̃Zbg(ʃobt@蒼)
		internal void Reset()
		{
			if ( this.inSurface != null )
			{
				this.inSurface.Dispose();
				this.inSurface = null;
			}

			// ɂȂ폜
			if ( this.inSwapChain != null )
			{
				this.inSwapChain.Dispose();
				this.inSwapChain = null;
			}

			if ( this.inZBuff != null )
			{
				this.inZBuff.Dispose();
				this.inZBuff = null;
			}
		}


		// `揈
		internal delegate void DoRenderFunc();
		internal void DoRender( int iArgb, DoRenderFunc inFunc )
		{
			Device d = dx_control.inDevice;


			// _[^[Qbg쐬
			if (this.inSwapChain == null)
			{
				PresentParameters p = new PresentParameters( dx_control.inParams );
				p.BackBufferWidth = this.inTarget.ClientSize.Width;
				p.BackBufferHeight = this.inTarget.ClientSize.Height;

				this.inSwapChain = new SwapChain( d, p );
			}

			// ^[Qbg쐬
			if ( this.inSurface == null )
			{
				this.inSurface = this.inSwapChain.GetBackBuffer( 0, BackBufferType.Mono );
			}

			// yobt@쐬
			if ( this.inZBuff == null )
			{
				this.inZBuff = d.CreateDepthStencilSurface(
										this.inTarget.ClientSize.Width,
										this.inTarget.ClientSize.Height,
										d.PresentationParameters.AutoDepthStencilFormat,
										MultiSampleType.None,
										0,
										false );
			}
	
			
			// yobt@̐ݒ
			d.DepthStencilSurface = this.inZBuff;

			Surface inBackupTarget = d.GetRenderTarget( 0 );

			// ^[Qbg̐ݒ
			d.SetRenderTarget( 0, this.inSurface );

			d.Clear( ClearFlags.Target|ClearFlags.ZBuffer,
										Color.FromArgb( iArgb ),
										1.0f,
										0 );
			d.BeginScene();


			// tbgeNX`pݒ
			// _ɐFꍇ͂ł͂߁B
			// 炭}eAJ[ł̓J[Aj[V͕\łȂ̂ŁA
			// t[_đKvłĂB
			RenderStateManager rs = d.RenderState;
			rs.Lighting = true;

			// ݒ
			rs.AlphaBlendEnable = true;
			rs.AlphaBlendOperation = BlendOperation.Add;
			rs.SourceBlend = Blend.SourceAlpha;
			rs.DestinationBlend = Blend.InvSourceAlpha;

			rs.SpecularEnable = false;
			// QƂ}eA
			rs.AmbientMaterialSource = ColorSource.Material;
			rs.DiffuseMaterialSource = ColorSource.Material;
			rs.EmissiveMaterialSource = ColorSource.Material;
			// _J[OFF
			rs.ColorVertex = false;
			rs.Ambient = Color.FromArgb(255, 255, 255, 255);

			rs.CullMode = Cull.None;

			// eNX`At@ݒ
			d.SetTextureStageState(0, TextureStageStates.AlphaOperation, (int)TextureOperation.Modulate);
			d.SetTextureStageState(0, TextureStageStates.AlphaArgument1, (int)TextureArgument.TextureColor);
			d.SetTextureStageState(0, TextureStageStates.AlphaArgument2, (int)TextureArgument.Current);

			// UVbvl
			d.SetSamplerState(0, SamplerStageStates.AddressU, (int)TextureAddress.Wrap);
			d.SetSamplerState(0, SamplerStageStates.AddressV, (int)TextureAddress.Wrap);

			// eNX`}gNXݒ
			d.TextureState[0].TextureTransform = TextureTransform.Count2;
			d.TextureState[0].TextureCoordinateIndex = 0;


			// }eAJ[ĝLighting͗LɂKv邪A
			// Cg͑SĖɂȂƐFe󂯂Ă܂B
			foreach (Light inLight in d.Lights)
			{
				inLight.Enabled = false;
			}

			inFunc();


			d.EndScene();
			try
			{
				if ( this.inSwapChain != null )
				{
					this.inSwapChain.Present( this.inTarget );
				}
			}
			// foCX̃`FbNƃZbg
			catch (DeviceLostException)
			{
				dx_control.DeviceReset();
			}


			// _[^[Qbgɖ߂B
			d.SetRenderTarget( 0, inBackupTarget );
			// QƃJE^炷
			inBackupTarget.Dispose();

		}

		#region IDisposable o

		public void Dispose()
		{
			this.Reset();

			dx_control.iRefCount --;

			// QƐOŃfoCX݂Ȃ
			if ( dx_control.iRefCount <= 0 )
			{
				if ( dx_control.inDevice != null )
				{
					dx_control.inDevice.Dispose();
					dx_control.inDevice = null;
				}
			}

		}

		#endregion
	}
}
