﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Interop;
using System.Diagnostics;
using System.Windows.Media.Imaging;  
using System.Windows.Shapes;
using System.Windows.Browser;
using evc;
using evcIO;
using glb;
using sl;


namespace sl_evcPlayer
{
	internal class Main : Canvas
	{
		public static Main Target { get; private set; }

		private int iNextTick; // 経過時間カウンタ
		private seRootScene inScene;
		private bool bIsPlay;

		internal trIOManager IOManager	{ get; private set; }
		internal trMaterial Material	{ get; private set; }


		private TextBlock inStopHelp = new TextBlock();
		private Rectangle inHelpBase = new Rectangle();

		public Main( string strPackName, string strAnimName, bool bLostFocusStop )
		{
			Main.Target = this;

			this.inStopHelp.Text = "処理を停止しています。再開するには画面をクリックして下さい。";
			this.inStopHelp.Foreground = new SolidColorBrush( Color.FromArgb( 255,255,255,255 ) );
			this.inHelpBase.Fill = new SolidColorBrush( Color.FromArgb( 196,0,0,0 ) );

			// このクラスを描画ターゲットに登録
			slPutManager.Instance.SetParent( this.Children );
			// アニメーションの初期化
			this.IOManager = new trIOManager();
			// 入力インタフェース初期化
			trEventIO inEventIO = new trEventIO();
			inEventIO.SetTarget(this);
			this.Material = new trMaterial();
			this.inScene = new seRootScene(this.IOManager, inEventIO, this.Material );
			this.inScene.EventListener = new slEventListener();
			this.inScene.OnInit += new glbAction<seSceneManager>(inScene_OnInit);
			this.inScene.Load(strPackName, strAnimName);

			this.bIsPlay = true;

			// フォーカスを失った時にアニメーションをストップします。
			if ( bLostFocusStop )
			{
				this.LostFocus += new RoutedEventHandler(Main_LostFocus);
				this.GotFocus += new RoutedEventHandler(Main_GotFocus);
			}
			this.iNextTick = Environment.TickCount;
			CompositionTarget.Rendering += new EventHandler(CompositionTarget_Rendering);
		}


		// フォーカスを得た
		void Main_GotFocus(object sender, RoutedEventArgs e)
		{
			TranslateTransform t = new TranslateTransform();
			this.bIsPlay = true;
			Content c = App.Current.Host.Content;
			double dFontHeight = this.inStopHelp.ActualHeight;
			this.inHelpBase.Width = c.ActualWidth;
			this.inHelpBase.Height = dFontHeight;

			t.X = 0;
			t.Y = c.ActualHeight - dFontHeight;
			this.inHelpBase.RenderTransform = t;
			this.inStopHelp.RenderTransform = t;

			this.Children.Remove( this.inHelpBase );
			this.Children.Remove( this.inStopHelp );
		}

		// フォーカスを失った
		void Main_LostFocus(object sender, RoutedEventArgs e)
		{
			this.Children.Add( this.inHelpBase );
			this.Children.Add( this.inStopHelp );
			this.bIsPlay = false;
		}


		// 描画時
		void CompositionTarget_Rendering(object sender, EventArgs e)
		{
			int iNowTick = Environment.TickCount;

			// シーンとフォーカスが有効な時のみ実行
			if ( (this.inScene != null) && this.bIsPlay )
			{
				// 入力処理更新
				this.inScene.UpdateInput();

				// アニメーションの実行
				this.inScene.DoAnim( (float)(iNowTick - iNextTick) );

				slPutManager.Instance.BeginDraw();

				// 要素の描画
				this.inScene.Draw();
				slPutManager.Instance.EndDraw();
			}

			this.iNextTick = iNowTick;
		}

		// シーン初期化終了時
		void inScene_OnInit(seSceneManager inSceneManager)
		{
			seOption option = inSceneManager.Option;
			glbColor c = option.BgColor;

#if true
			this.Background = new SolidColorBrush(Color.FromArgb(
									(byte)c.A, (byte)c.R,
									(byte)c.G, (byte)c.B));
#endif
			this.Width = option.ScreenSize.X;
			this.Height = option.ScreenSize.Y;

			// スクリーンサイズの設定
			slPutManager.Instance.Resize( (int)this.Width, (int)this.Height );
		}

		// 終了時
		~Main()
		{
			if ( this.inScene != null )
			{
				this.inScene.Clear();
			}

			if ( this.IOManager != null )
			{
				this.IOManager.Dispose();
			}
		}
	}
}
