﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Effects;
using System.Diagnostics;
using System.Reflection;



namespace evcIO
{

	// weightバッファをみながらテクスチャをマッピングします。
	// 部分的に歪ませるようなメッシュを作成するのに有効です。
	public class UVMapEffectBase : ShaderEffect
	{
		private static readonly DependencyProperty TextureProperty = ShaderEffect.RegisterPixelShaderSamplerProperty(
										"Texture", typeof(UVMapEffectBase), 0);
		private static readonly DependencyProperty WeightProperty = ShaderEffect.RegisterPixelShaderSamplerProperty(
										"Weight", typeof(UVMapEffectBase), 1);
		private static readonly DependencyProperty ColorProperty = DependencyProperty.Register(
										"ColorKey", 
										typeof(Color), 
										typeof(UVMapEffectBase), 
										new PropertyMetadata( Colors.Green, PixelShaderConstantCallback(0)));

		public UVMapEffectBase( ImageBrush ib, string path )
		{
			this.PixelShader = effectManager.Instance.GetShader( path );
			this.UpdateShaderValue(TextureProperty);
			this.UpdateShaderValue(WeightProperty);
			this.UpdateShaderValue(ColorProperty);

			this.Weight = ib;
		}

		// カラー設定
		public Color BaseColor
		{
            get { return (Color)GetValue(ColorProperty); }
            set { SetValue(ColorProperty, value); }
		}

		// 表示テクスチャ
		public Brush Texture
		{
			get { return (Brush)GetValue(TextureProperty); }
			set { SetValue(TextureProperty, value); }
		}

		// ウェイトバッファ
        public Brush Weight
        {
            get { return (Brush)GetValue(WeightProperty); }
            set { SetValue(WeightProperty, value); }
        }
	}

	public class UVMapEffect : UVMapEffectBase
	{
		public UVMapEffect(ImageBrush ib ) : base( ib, "/sl_evcPlayer;component/evcIO/object/effect/MeshNormal.ps" ) {}
	}

	public class UVMapEffectAdd : UVMapEffectBase
	{
		public UVMapEffectAdd(ImageBrush ib) : base( ib, "/sl_evcPlayer;component/evcIO/object/effect/MeshAdd.ps" ) {}
	}




}

