﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Effects;
using System.Diagnostics;
using System.Reflection;



namespace evcIO
{
	// カラートイメージを設定するシェーダーとの受け渡し用クラス
	// プロパティとシェーダーのレジスタは以下の様に対応しています。
	// BaseColor : register(C0);
	// Texture   : register(S0);
	public class effectBase : ShaderEffect
	{
		public static readonly string PathBase = "/sl_evcPlayer;component/evcIO/object/effect/";

		private static readonly DependencyProperty TextureProperty =
									ShaderEffect.RegisterPixelShaderSamplerProperty(
										"Texture", 
										typeof(effectBase), 
										0 );
        private static readonly DependencyProperty ColorProperty = DependencyProperty.Register(
										"ColorKey", 
										typeof(Color), 
										typeof(effectBase), 
										new PropertyMetadata( Colors.Green, PixelShaderConstantCallback(0)));

		public effectBase( string path )
		{
			this.PixelShader = effectManager.Instance.GetShader( path );
			this.UpdateShaderValue(ColorProperty);
			this.UpdateShaderValue(TextureProperty);
		}

		// カラー設定
		public Color BaseColor
		{
            get { return (Color)GetValue(ColorProperty); }
            set { SetValue(ColorProperty, value); }
		}

		// レンダーターゲット
        public Brush Texture
        {
            get { return (Brush)GetValue(TextureProperty); }
            set { SetValue(TextureProperty, value); }
        }
	}

	// テクスチャ:通常ブレンド
	public class TexNormalEffect : effectBase
	{
		public TexNormalEffect() : base( effectBase.PathBase + "TexNormal.ps" ){}
	}

	// テクスチャ:色加算
	public class TexAddEffect : effectBase
	{
		public TexAddEffect() : base( effectBase.PathBase + "TexAdd.ps" ){}
	}


	// 単色：通常ブレンド
	public class ColorNormalEffect : effectBase
	{
		public ColorNormalEffect () : base( effectBase.PathBase + "ColorNormal.ps" ){}
	}

	// 単色:色加算
	public class ColorAddEffect : effectBase
	{
		public ColorAddEffect () : base( effectBase.PathBase + "ColorAdd.ps" ){}
	}

	// フォント:色加算
	public class FontAddEffect : effectBase
	{
		public FontAddEffect () : base( effectBase.PathBase + "FontAdd.ps" ){}
	}


}

