﻿using System.Collections.Generic;
using System.IO;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using glb;

namespace evcIO
{
	// マスク用ブラシ管理
	// 
	
	internal class trMaskControl
	{
		// シングルトン
		private static trMaskControl inInstance = new trMaskControl();
		public static trMaskControl Instance { get { return trMaskControl.inInstance; } }

		private Dictionary<int, ImageBrush> dicMask = new Dictionary<int,ImageBrush>();
		private trMatrix inMatrixWork = new trMatrix();

		// アクティブなマスクを取得
		internal trObjectTextureSprite ActiveMask	{ get; private set; }
		// アクティブなマスクブラシを取得
		internal ImageBrush ActiveBrush				{ get; private set; }
		// アクティブなマスクマトリクスを取得
		internal IGLMatrix ActiveMatrix				{ get; private set; }


		public trMaskControl()
		{
		}

		// アクティブなマスクを設定
		internal void SetActive( int iElemID, trObjectTextureSprite inMask, IGLMatrix inMatrix )
		{
			this.ActiveMask = inMask;
			this.ActiveBrush = this.dicMask[ iElemID ];
			this.ActiveMatrix.Copy( inMatrix );
		}

		// 表示対象のマトリクスとマスクのマトリクスからローカルマトリクスを算出
		// マトリクスは絶対座標で入っている。
		// silverlightで表示する時は親子関係になる為、マトリクスを補正する必要がある。
		internal void SetLocalMatrix( IGLMatrix mParent )
		{
			this.inMatrixWork.Copy( mParent );
			this.inMatrixWork.Invert();
			this.inMatrixWork.Mul( this.ActiveMatrix );

			MatrixTransform m = (MatrixTransform)this.ActiveBrush.Transform;
			m.Matrix = this.inMatrixWork.GetSlMatrix();
		}


		// 要素ＩＤからマスク用のブラシを取得
		internal ImageBrush this[ int iElemID ]
		{
			get { return this.dicMask[ iElemID ]; }
		}


		// マスクイメージを登録
		internal void Add( int iElemID, Stream inImageStream )
		{
			// 既にあるなら無処理
			if ( this.dicMask.ContainsKey( iElemID ) ) return;

			BitmapImage inBmp = new BitmapImage();
			inBmp.SetSource(inImageStream);

			ImageBrush inImageBrush = new ImageBrush();
			inImageBrush.ImageSource = inBmp;
			inImageBrush.Transform = new MatrixTransform();

			this.dicMask[ iElemID ] = inImageBrush;
		}

		// マスクデータをクリア
		internal void Clear()
		{
			this.dicMask.Clear();
		}
	}
}
