using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Media;
using System.Windows;
using glb;
using sl;

namespace evcIO
{
	internal class trMatrix : IGLMatrix
	{
		public double m11, m12,
					  m21, m22, 
					  m31, m32;


		// RXgN^
		public trMatrix ()
		{
			this.Identity();
		}

		// Pʃ}gNXɐݒ
		public void Identity()
		{
			this.m11 = this.m22 = 1.0;
			this.m12 = this.m21 = this.m31 = this.m32 = 0.0;
		}

		// |Z
		public void MulR( IGLMatrix mTarget )
		{
			trMatrix m = (trMatrix)mTarget;
			double r11 = this.m11 * m.m11 + this.m12 * m.m21;
			double r12 = this.m11 * m.m12 + this.m12 * m.m22;
			double r21 = this.m21 * m.m11 + this.m22 * m.m21;
			double r22 = this.m21 * m.m12 + this.m22 * m.m22;
			double r31 = this.m31 * m.m11 + this.m32 * m.m21 + m.m31;
			double r32 = this.m31 * m.m12 + this.m32 * m.m22 + m.m32;
			this.m11 = r11;
			this.m12 = r12;
			this.m21 = r21;
			this.m22 = r22;
			this.m31 = r31;
			this.m32 = r32;
		}

		// |Z(t)
		public void Mul(IGLMatrix mTarget)
		{
			trMatrix m = (trMatrix)mTarget;
			double r11 = m.m11 * this.m11 + m.m12 * this.m21;
			double r12 = m.m11 * this.m12 + m.m12 * this.m22;
			double r21 = m.m21 * this.m11 + m.m22 * this.m21;
			double r22 = m.m21 * this.m12 + m.m22 * this.m22;
			double r31 = m.m31 * this.m11 + m.m32 * this.m21 + this.m31;
			double r32 = m.m31 * this.m12 + m.m32 * this.m22 + this.m32;
			this.m11 = r11;
			this.m12 = r12;
			this.m21 = r21;
			this.m22 = r22;
			this.m31 = r31;
			this.m32 = r32;
		}


		// ʒuݒ
		public void SetPos(float fX, float fY)
		{
			this.m11 = this.m22 = 1.0;
			this.m12 = this.m21 = 0.0;
			this.m31 = (double)fX;
			this.m32 = (double)fY;
		}
		public void SetPos(glbVector2 sPos)
		{
			this.m11 = this.m22 = 1.0;
			this.m12 = this.m21 = 0.0;
			this.m31 = sPos.X;
			this.m32 = sPos.Y;
		}

		// XP[ݒ
		public void SetScale(float fX, float fY)
		{
			this.m11 = (double)fX;
			this.m22 = (double)fY;
			this.m12 = this.m21 = this.m31 = this.m32 = 0.0;
		}
		public void SetScale(glbVector2 sScale)
		{
			this.m11 = sScale.X;
			this.m22 = sScale.Y;
			this.m12 = this.m21 = this.m31 = this.m32 = 0.0;
		}

		// ]Fy
		public void SetRotZ(float fZ)
		{
			double fSin = Math.Sin((double)-fZ);
			double fCos = Math.Cos((double)-fZ);

			this.m11 = fCos;
			this.m21 = fSin;
			this.m12 = -fSin;
			this.m22 = fCos;

			this.m31 = this.m32 = 0.0;
		}


		// Silverlightpmatrixɕϊ
		public Matrix GetSlMatrix()
		{
			return new Matrix( this.m11, this.m12, this.m21, this.m22, this.m31, this.m32 );
		}


		// eNX`}gNXpUVZbg
		public void SetUV(float fU, float fV)
		{
			this.m11 = this.m22 = 1.0;
			this.m12 = this.m21 = 0.0;
			this.m31 = (double)fU;
			this.m32 = (double)fV;
		}
		public void SetUV(glbVector2 sUV)
		{
			this.m11 = this.m22 = 1.0;
			this.m12 = this.m21 = 0.0;
			this.m31 = sUV.X;
			this.m32 = sUV.Y;
		}


		// t}gNXɕϊ
		public void Invert ()
		{
			double a = this.m22;
			double b = - this.m12;

			double fDetA = 1.0 / (this.m11 * a + this.m21 * b);

			double d = -this.m21;
			double e = this.m11;
			double g = this.m21 * this.m32 - this.m22 * this.m31;
			double h = this.m12 * this.m31 - this.m11 * this.m32;

			this.m11 = a * fDetA;
			this.m12 = b * fDetA;
			this.m21 = d * fDetA;
			this.m22 = e * fDetA;
			this.m31 = g * fDetA;
			this.m32 = h * fDetA;
		}

		// lRs[
		public void Copy( IGLMatrix inMatrix )
		{
			trMatrix m = (trMatrix)inMatrix;
			this.m11 = m.m11; this.m12 = m.m12;
			this.m21 = m.m21; this.m22 = m.m22;
			this.m31 = m.m31; this.m32 = m.m32;
	
		}

		// SilverlightpglbVector2 * Matrix
		public Point TransformCoordinate(Point sMstPos)
		{
			return new Point(this.m11 * sMstPos.X + this.m21 * sMstPos.Y + this.m31,
							  this.m12 * sMstPos.X + this.m22 * sMstPos.Y + this.m32);
		}

		// glbVector2 * Matrix
		public glbVector2 TransformCoordinate ( glbVector2 sMstPos )
		{
			double dMstX = (double)sMstPos.X;
			double dMstY = (double)sMstPos.Y;
			return new glbVector2(
					(this.m11 * dMstX + this.m21 * dMstY + this.m31),
					(this.m12 * dMstX + this.m22 * dMstY + this.m32));
		}
	}
}
