﻿using System;
using System.Windows;
using System.Windows.Shapes;
using System.Windows.Controls;
using System.Collections.Generic;
using System.Text;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using evc;
using glb;
using evcIO;


namespace sl
{
	// Silverlight用配置全体管理
	internal class slPutManager
	{
		// シングルトン
		private static slPutManager inInstance = new slPutManager();
		public static slPutManager Instance { get { return slPutManager.inInstance; } }

		private Image _screen;
		private MatrixTransform _transform;

		public WriteableBitmap RenderTarget { get; private set; }
		public UIElementCollection Children { get; private set; }

		public event Action<WriteableBitmap> RenderTargetChange;


		// スクリーンサイズの設定
		public void Resize( int iWidth, int iHeight )
		{
			this.RenderTarget = new WriteableBitmap( iWidth, iHeight );
			this._screen.Width = iWidth;
			this._screen.Height = iHeight;
			this._screen.Source = this.RenderTarget;

			if ( this.RenderTargetChange != null)
			{
				this.RenderTargetChange(this.RenderTarget);
			}
		}

		// 描画呼び出し
		public void Draw( UIElement elem, IGLMatrix m)
		{
			this._transform.Matrix = ((trMatrix)m).GetSlMatrix();
			this.RenderTarget.Render( elem, this._transform );
		}

		// 描画処理開始
		public void BeginDraw()
		{
			Array.Clear( this.RenderTarget.Pixels, 0, this.RenderTarget.Pixels.Length );
		}

		// 描画処理終了
		public void EndDraw()
		{
			this.RenderTarget.Invalidate();
		}

		// 親を設定
		public void SetParent( UIElementCollection elemCollection )
		{
			this.Children = elemCollection;
			elemCollection.Add( this._screen );
		}


		// コンストラクタ
		private slPutManager()
		{
			this._transform = new MatrixTransform();
			this._screen = new Image();
			// テスト的に。
			this.Resize( 40, 30 );
		}
	}
}
