﻿using System;
using System.Collections.Generic;
using System.Text;

namespace evc
{
#if true
	// SortedDictionaryの代用:
	// データ挿入時にソートフラグを立てておき、
	// GetEnumerator,Keys,Valuesで連続したデータを取得する時に
	// フラグが立っていればソートを実行します。
	// 

	internal class SortedDictionary<TKey,TValue> : Dictionary<TKey,TValue>
		where TKey : IComparable
	{
		private bool bUpdateSort = false;

		public new TValue this[TKey tIndex]
		{
			get { return base[tIndex]; }
			set 
			{
				base[tIndex] = value;
				this.bUpdateSort = true;
			}
		}

		public new void Add(TKey key, TValue value)
		{
			base.Add(key, value);
			this.bUpdateSort = true;
		}

		public new Dictionary<TKey, TValue>.KeyCollection Keys
		{
			get 
			{
				this.Sort();
				return base.Keys;
			}
		}

		public new Dictionary<TKey, TValue>.ValueCollection Values 
		{
			get
			{
				this.Sort();
				return base.Values;
			}
		}

		public new Dictionary<TKey, TValue>.Enumerator GetEnumerator()
		{
			this.Sort();
			return base.GetEnumerator();
		}

	
		// Keyでソート
		private void Sort()
		{
			if ( !this.bUpdateSort ) return;

			List<KeyValuePair<TKey, TValue>> lisSort = new List<KeyValuePair<TKey, TValue>>(this);
			lisSort.Sort(delegate( KeyValuePair<TKey, TValue> kvp1, 
								   KeyValuePair<TKey, TValue> kvp2 )
			{
				return kvp1.Key.CompareTo(kvp2.Key);
			});
			this.Clear();
			lisSort.ForEach(delegate(KeyValuePair<TKey, TValue> inSortPair)
			{
				base.Add(inSortPair.Key, inSortPair.Value);
			});
			this.bUpdateSort = false;
		}
	}
#endif
}
